;****I* idl/idl.atan2
; NAME
; atan2
; 
; PURPOSE
; Simulate the Fortran ATAN2 function where the returned result ranges
; from -180 to 180 instead of the default range of ATAN (-90,90)
; HOWEVER, RETURNED ANGLES ARE IN RADIANS ... JUST LIKE ATAN.
; 
; USAGE
; angle = atan2(y,x)
; 
; INPUT
; y,x - these numbers would normally y/x for the atan function, but
;       remain separate to retain the information lost when dividing.
;       
; OUTPUT
; angle (RADIANS) ranging from (-PI,PI)
; 
; RESULT
; EXAMPLE
; IDL> print,atan(-1./(-1.))/!dtor
;      45.0000
; IDL> print,atan2(-1.,(-1.))/!dtor
;      -135.00001
;      
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; ???
; 
; HISTORY
; * 27Jun07 - Documentation added.
; 
; SOURCE

function atan2,a1,b1

if (n_elements(b1) eq 0) then begin
    print,'FUNCTION: ATAN2(A,B)'
    atn2 = -1
    return,atn2
endif
a=double(a1)
b=double(b1)
atn2 = a*0.
z = where (b eq 0,nz,comp = notz,ncomp=nnotz)
if (nz gt 0) then begin
    pos = where(a(z) gt 0.,npos,compl=neg,ncompl=nneg)
    if (npos gt 0) then atn2(z(pos)) = !pi/2.
    if (nneg gt 0) then atn2(z(neg)) = 3.*!pi/2.
endif

if (nnotz gt 0) then begin
    atn2(notz) = atan(a(notz)/b(notz))
    Q1 = where(b(notz) gt 0. and a(notz) ge 0.,nq1)
    Q2 = where(b(notz) lt 0. and a(notz) ge 0.,nq2)
    Q3 = where(b(notz) lt 0. and a(notz) lt 0.,nq3)
    Q4 = where(b(notz) gt 0. and a(notz) lt 0.,nq4)
;   if (nq1 gt 0) then atn2(notz(q1)) = atn2(notz(q1))
    if (nq2 gt 0) then atn2(notz(q2)) = atn2(notz(q2)) + !pi
    if (nq3 gt 0) then atn2(notz(q3)) = atn2(notz(q3)) - !pi
;   if (nq4 gt 0) then atn2(notz(q4)) = atn2(notz(q4))
endif
return,atn2
end
;***
