function b3frombt,btfile
b3dir=getenv('ISCCPDIR')+'b3/'
f=strsplit(stripname(btfile),'.',/extr)
f(1) = 'B3'
f(2) = '?'
f(7) = strmid(f(7),0,3)+'?'
f(8) = '???'
fs = file_search(b3dir+f(4)+'/'+strjoin(f,'.'),count=nf)
itry=0
if (nf eq 0) then begin
; In some strange cases, there is no B3 file even though a BT file is
; present. In these cases, one should look for the first B3 file in
; that month as the closest B3 file. 

    while (nf eq 0) do begin
;check all day first
        if (itry eq 0) then f(7) = '????'
; then check all month        
        if (itry eq 1) then f(6) = '??'
        if (itry eq 2) then break
        fs = file_search(b3dir+f(4)+'/'+strjoin(f,'.'),count=nf)
        itry++
    endwhile
; If no B3 file is found that month ... well, I guess I'll cross that
; bridge later.
    if (nf eq 0) then begin
        return,''
;        Message,'No B3 file found for '+btfile
    endif
endif 

return,fs(nf-1)
end

;****I* idl/idl.b1read_calv2
; NAME
; b1read_calv2
; PURPOSE
; Provide ISCCP calibration for ISCCP B1 files. This routine gathers
; the ISCCP calibration from either BT or B3 files.
; USAGE
; See code for b1read.pro
; USES
; * stripname.pro
; * b3_channel_order.pro
; * btread.pro
; 
; USED BY
; * b1read.pro
; 
; NOTES
; Version 2 of b1read_cal.pro
;
; this update 
; ... uses the raw BT/B3 files (instead of KT files)
; ... searches B3 in addition to BT files
; ... properly searches for nearby files (staying within the month 1st)
; ... returns new variable with better description of version and
;     where it came from
;
; * Some channel orders are hardcoded because the B3 file corresponding
;   to the BT file is missing. (e.g., GOE-6 1986)
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; ???
; 
; HISTORY
; * 28Jun07 - Added this documentation.
; * 2010-08- KRK - Added more documentation.
; SOURCE

pro b1read_calv2,file,chan,nchan,ctable, $
                 version=version, $
                 calfile=calfile, $
                 quiet=quiet, $
                 debug=debug, $
                 table=table

iquiet = 0
if keyword_set(quiet) then iquiet=1
f = strsplit(stripname(file),'.',/extr)

if (f(3) eq 'MET-6' and f(4) eq '2000') then f(3) = 'MET-7'

if (f(3) eq 'MSG-1') then f(3) = 'MET-8'
if (f(3) eq 'MSG-2') then f(3) = 'MET-9'


if keyword_set(debug) then print,'FILE='+file

delmon=1

idev = 0

btdir = getenv('ISCCPDIR')+'bt/'
b3dir = getenv('ISCCPDIR')+'b3/'
nb1chan = n_elements(chan)

;Correct the # of channels when the Walsh Hammard transform is
;present (generally for AES files only).
badchan = where(strpos(chan,'WLSHM') ge 0)
if (badchan(0) ge 0) then nb1chan=nb1chan-1

oldchan = nb1chan
if (f(8) eq 'CSU' and nb1chan gt 5) then nb1chan = 5

if (f(3) eq 'GOES9') then f(3) = 'GOE-9'

;Why was the following included?? It caused some major problems when 
;checking the calibration of the data!!
;if (f(8) eq 'NOA') then nb1chan = 1
inoa = 0
if (f(8) eq 'NOA') then begin
    inoa = 1
endif

if (f(3) eq 'MET-8' or f(3) eq 'MET-9') then inoa = 1

; get time info from the filename
hhmm = f(7)
hh = hhmm / 100
mm = hhmm mod 100
jday = julday(f(5),f(6),f(4),hh,mm,0)
;round off to closest synoptic time
jday = round(jday * 8) / 8.d0
jday1 = jday
;get mon,day,year of rounded synoptic time
caldat,jday,mon,day,yer,hour
cyer = string(yer,form='(i4.4)')
cmon = string(mon,form='(i2.2)')
cday = string(day,form='(i2.2)')
chour= string(hour,form='(i2.2)')
f(1) = 'BT'
f(2) = '?'
f(4) = cyer
f(5) = cmon
f(6) = cday
f(7) = string(chour*100,format='(i4.4)')

;look for BT file at identical time
BTSearch1 = btdir + cyer+'/'+strjoin(f(0:7),'.')+'.???.??'
BTCalFile1 = file_search(BTSearch1,count=nbtfiles1)
if keyword_set(debug) then print,BTSearch1+' = '+string(nbtfiles1)+' files'
if (nbtfiles1 gt 0) then begin
    btf = btcalfile1(nbtfiles1-1)
    btread,btf,table,nchan=nbtchan
    if n_elements(table) eq 1 then goto,btfail1
    calfile=btf
    ;get the channel order
    b3f = b3frombt(calfile)
    if (strlen(b3f) eq 0) then begin
        if (f(3) eq 'GOE-6' and f(4) eq '1986') then begin
            order_b3 = ['VSCHN','IRWIN']
        endif else if (f(3) eq 'GOE-6' and f(4) eq '1988') then begin
            order_b3 = ['VSCHN','IRWIN']
        endif else if (f(3) eq 'GOE-6' and f(4) eq '1989') then begin
            order_b3 = ['VSCHN','IRWIN']
        endif else if (f(3) eq 'GOE-7' and f(4) eq '1988') then begin
            order_b3 = ['VSCHN','IRWIN']
        endif else if (f(3) eq 'GOE-7' and f(4) eq '1989') then begin
            order_b3 = ['VSCHN','IRWIN']
        endif else if (f(3) eq 'GOE-7' and f(4) eq '1990') then begin
            order_b3 = ['VSCHN','IRWIN']
        endif else if (f(3) eq 'GOE-7' and f(4) eq '1991') then begin
            order_b3 = ['VSCHN','IRWIN']
        endif else if (f(3) eq 'MET-3' and f(4) eq '1988') then begin
            order_b3 = ['VSCHN','IRWIN']
        endif else if (f(3) eq 'MET-2' and f(4) eq '1988') then begin
            order_b3 = ['VSCHN','IRWIN','IRWVP']
        endif else if (f(3) eq 'GOE-10' and f(4) eq '2002') then begin
            order_b3 = ['VSCHN','IRWIN','IRNIR','IRWVP','IRSPL']
        endif else begin
            print,'OOPS!!: No channel info'
            print,'BT file='+btcalfile1(nbtfiles1-1)
            print,'B3 file='+b3f
;            stop
        endelse
    endif else begin
        b3_channel_order,b3f,order_b3,b3f=b3f,/bt
    endelse
    ;if BT from identical time is found, then no need to check number
    ;the number of channels
    goto,chanord
endif
btfail1:
if (iquiet eq 0) then print,'Same Time BT failed'

;if none, look for B3 file at identical time
f(1) = 'B3'
B3Search1 = b3dir + cyer+'/'+strjoin(f(0:7),'.')+'.???.??'
B3CalFile1 = file_search(B3Search1,count=nb3files1)

if (idev) then nb3files1 = 0

if (nb3files1 ne 0) then begin
    b3f = b3calfile1(nb3files1-1)
    calfile=b3f
    b3_channel_order,b3f,order_b3,b3f=b3f,cal=cal
    nch = n_elements(cal.rpu_nom.phy_scl)
    table = fltarr(6,nch,256)
    table(0,*,*) = transpose(         cal.rpu_nom.phy_table) / $
                   rebin(reform(float(cal.rpu_nom.phy_scl),nch,1),nch,256)
    table(1,*,*) = transpose(         cal.rpu_nrm.phy_table) / $
                   rebin(reform(float(cal.rpu_nrm.phy_scl),nch,1),nch,256)
    table(2,*,*) = transpose(         cal.rpu_abs.phy_table) / $
                   rebin(reform(float(cal.rpu_abs.phy_scl),nch,1),nch,256)
    table(3,*,*) = transpose(         cal.apu_nom.phy_table) / $
                   rebin(reform(float(cal.apu_nom.phy_scl),nch,1),nch,256)
    table(4,*,*) = transpose(         cal.apu_nrm.phy_table) / $
                   rebin(reform(float(cal.apu_nrm.phy_scl),nch,1),nch,256)
    table(5,*,*) = transpose(         cal.apu_abs.phy_table) / $
                   rebin(reform(float(cal.apu_abs.phy_scl),nch,1),nch,256)
    ;if B3 from identical time is found, then no need to check number
    ;the number of channels
    goto,chanord
endif

if (iquiet eq 0) then print,'Same Time B3 failed'

;if none, look for BT/B3 files from the same day
f(1) = 'BT'
f(7) = '????'
BTSearch2 = btdir + cyer+'/'+strjoin(f(0:7),'.')+'.???.??'
BTCalFile2 = file_search(BTSearch2,count=nbtfiles2)

if (idev) then nbtfiles2 = 0

if (nbtfiles2 gt 0) then begin
    for i=0,nbtfiles2-1 do begin
        btread,btcalfile2(i),table,nchan=nbtchan
        if (n_elements(table) gt 1) then begin
            if (nb1chan le nbtchan) then break
            if (inoa) then break
        endif
    endfor
    if (i eq nbtfiles2) then goto,b3day
    calfile = btcalfile2(i)
    ;get the channel order
    b3f = b3frombt(calfile)
    if (strlen(b3f) eq 0) then begin
        if (f(3) eq 'GOE-6' and f(4) eq '1986') then $
          order_b3 = ['VSCHN','IRWIN']
    endif else begin
        b3_channel_order,b3f,order_b3,b3f=b3f,/bt
    endelse
    goto,chanord
endif

b3day:

if (iquiet eq 0) then print,'Same Day BT failed'

f(1) = 'B3'
f(7) = '????'
B3Search2 = b3dir + cyer+'/'+strjoin(f(0:7),'.')+'.???.??'
B3CalFile2 = file_search(B3Search2,count=nb3files2)

if (idev) then nb3files2 = 0

if (nb3files2 gt 0) then begin
    for i=0,nb3files2-1 do begin
        b3f = b3calfile2(i)
        b3_channel_order,b3f,order_b3,b3f=b3f,cal=cal
        nbtchan = n_elements(order_b3)
        if (nb1chan le nbtchan) then break
        if (inoa) then break
    endfor
    if (i eq nb3files2) then goto,btmonth
    calfile = b3calfile2(i)
    nch = n_elements(cal.rpu_nom.phy_scl)
    table = fltarr(6,nch,256)
    table(0,*,*) = transpose(         cal.rpu_nom.phy_table) / $
                   rebin(reform(float(cal.rpu_nom.phy_scl),nch,1),nch,256)
    table(1,*,*) = transpose(         cal.rpu_nrm.phy_table) / $
                   rebin(reform(float(cal.rpu_nrm.phy_scl),nch,1),nch,256)
    table(2,*,*) = transpose(         cal.rpu_abs.phy_table) / $
                   rebin(reform(float(cal.rpu_abs.phy_scl),nch,1),nch,256)
    table(3,*,*) = transpose(         cal.apu_nom.phy_table) / $
                   rebin(reform(float(cal.apu_nom.phy_scl),nch,1),nch,256)
    table(4,*,*) = transpose(         cal.apu_nrm.phy_table) / $
                   rebin(reform(float(cal.apu_nrm.phy_scl),nch,1),nch,256)
    table(5,*,*) = transpose(         cal.apu_abs.phy_table) / $
                   rebin(reform(float(cal.apu_abs.phy_scl),nch,1),nch,256)
    ;get the channel order
    goto,chanord
endif

btmonth:
ib3d=0
if (iquiet eq 0 and ib3d eq 0) then print,'Same Day B3 failed'
ib3d++

;if none, look for BT/B3 files from the same month
f(1) = 'BT'
f(6) = '??'
f(7) = '????'
BTSearch3 = btdir + cyer+'/'+strjoin(f(0:7),'.')+'.???.??'
BTCalFile3 = file_search(BTSearch3,count=nbtfiles3)
if (idev) then nbtfiles3 = 0
if (nbtfiles3 gt 0) then begin
    for i=0,nbtfiles3-1 do begin
        btread,btcalfile3(i),table,nchan=nbtchan
        if (n_elements(table) gt 1) then begin
            if (nb1chan le nbtchan) then break
            if (inoa) then break
        endif
    endfor
    if (i eq nbtfiles3) then goto,b3month
    calfile = btcalfile3(i)
    ;get the channel order
    b3f = b3frombt(calfile)
    if (strlen(b3f) eq 0) then goto,chanord
    b3_channel_order,b3f,order_b3,b3f=b3f,/bt
    goto,chanord
endif

if (iquiet eq 0) then print,'Same Month BT failed'

b3month:
f(1) = 'B3'
f(6) = '??'
f(7) = '????'
B3Search3 = b3dir + cyer+'/'+strjoin(f(0:7),'.')+'.???.??'
B3CalFile3 = file_search(B3Search3,count=nb3files3)
if (idev) then nb3files3 = 0
if (nb3files3 gt 0) then begin
    for i=0,nb3files3-1 do begin
        b3f = b3calfile3(i)
        b3_channel_order,b3f,order_b3,b3f=b3f,cal=cal
        nbtchan = n_elements(order_b3)
        if (nb1chan le nbtchan) then break
        if (inoa) then break
    endfor
    if (i eq nb3files3) then goto,btmonthp
    calfile = b3calfile3(i)
    nch = n_elements(cal.rpu_nom.phy_scl)
    table = fltarr(6,nch,256)
    table(0,*,*) = transpose(         cal.rpu_nom.phy_table) / $
                   rebin(reform(float(cal.rpu_nom.phy_scl),nch,1),nch,256)
    table(1,*,*) = transpose(         cal.rpu_nrm.phy_table) / $
                   rebin(reform(float(cal.rpu_nrm.phy_scl),nch,1),nch,256)
    table(2,*,*) = transpose(         cal.rpu_abs.phy_table) / $
                   rebin(reform(float(cal.rpu_abs.phy_scl),nch,1),nch,256)
    table(3,*,*) = transpose(         cal.apu_nom.phy_table) / $
                   rebin(reform(float(cal.apu_nom.phy_scl),nch,1),nch,256)
    table(4,*,*) = transpose(         cal.apu_nrm.phy_table) / $
                   rebin(reform(float(cal.apu_nrm.phy_scl),nch,1),nch,256)
    table(5,*,*) = transpose(         cal.apu_abs.phy_table) / $
                   rebin(reform(float(cal.apu_abs.phy_scl),nch,1),nch,256)
    ;get the channel order
    goto,chanord
endif
if (iquiet eq 0) then print,'Same Month B3 failed'

;check prior month BT/B3
btmonthp:
pmon = mon - delmon
pyer = yer
if (pmon lt 1) then begin
    pmon = pmon + 12
    pyer = yer - 1
endif
cyer = string(pyer,form='(i4.4)')
cmon = string(pmon,form='(i2.2)')
f(4) = cyer
f(5) = cmon
f(1) = 'BT'
f(6) = '??'
f(7) = '????'
BTSearch4 = btdir + cyer+'/'+strjoin(f(0:7),'.')+'.???.??'
BTCalFile4 = file_search(BTSearch4,count=nbtfiles4)
if (idev) then nbtfiles4 = 0
if (nbtfiles4 gt 0) then begin
    for i=0,nbtfiles4-1 do begin
        btread,btcalfile4(i),table,nchan=nbtchan
        if (n_elements(table) gt 1) then begin
            if (nb1chan le nbtchan) then break
            if (inoa) then break
        endif
    endfor
    if (i eq nbtfiles4) then goto,b3monthp
    calfile = btcalfile4(i)
    ;get the channel order
    b3f = b3frombt(calfile)
    if (strlen(b3f) eq 0) then begin
        if (f(3) eq 'MET-5' and f(4) eq '1998') then begin
            order_b3 = ['VSCHN','IRWIN','IRWVP']
        endif else begin
            print,'No file found: Error 1993725443413'
            print,calfile
            stop
        endelse 
    endif else begin
        b3_channel_order,b3f,order_b3,b3f=b3f,/bt
    endelse
    goto,chanord
endif
b3monthp:
f(1) = 'B3'
B3Search4 = b3dir + cyer+'/'+strjoin(f(0:7),'.')+'.???.??'
B3CalFile4= file_search(B3Search4,count=nb3files4)
if (idev) then nb3files4 = 0
if (nb3files4 gt 0) then begin
    for i=0,nb3files4-1 do begin
        b3f = b3calfile4(i)
        b3_channel_order,b3f,order_b3,b3f=b3f,cal=cal
        nbtchan = n_elements(order_b3)
        if (nb1chan le nbtchan) then break
        if (inoa) then break
    endfor
    if (i eq nb3files4) then goto,btmonthn
    calfile = b3calfile4(i)
    nch = n_elements(cal.rpu_nom.phy_scl)
    table = fltarr(6,nch,256)
    table(0,*,*) = transpose(         cal.rpu_nom.phy_table) / $
                   rebin(reform(float(cal.rpu_nom.phy_scl),nch,1),nch,256)
    table(1,*,*) = transpose(         cal.rpu_nrm.phy_table) / $
                   rebin(reform(float(cal.rpu_nrm.phy_scl),nch,1),nch,256)
    table(2,*,*) = transpose(         cal.rpu_abs.phy_table) / $
                   rebin(reform(float(cal.rpu_abs.phy_scl),nch,1),nch,256)
    table(3,*,*) = transpose(         cal.apu_nom.phy_table) / $
                   rebin(reform(float(cal.apu_nom.phy_scl),nch,1),nch,256)
    table(4,*,*) = transpose(         cal.apu_nrm.phy_table) / $
                   rebin(reform(float(cal.apu_nrm.phy_scl),nch,1),nch,256)
    table(5,*,*) = transpose(         cal.apu_abs.phy_table) / $
                   rebin(reform(float(cal.apu_abs.phy_scl),nch,1),nch,256)
    ;get the channel order
    goto,chanord
endif
btmonthn:
;check following month BT/B3
fmon = mon + delmon
fyer = yer
if (fmon gt 12) then begin
    fmon = fmon - 12
    fyer = yer + 1
endif
cyer = string(fyer,form='(i4.4)')
cmon = string(fmon,form='(i2.2)')
f(4) = cyer
f(5) = cmon
f(1) = 'BT'
f(6) = '??'
f(7) = '????'
BTSearch5 = btdir + cyer+'/'+strjoin(f(0:7),'.')+'.???.??'
BTCalFile5= file_search(BTSearch5,count=nbtfiles5)
if (idev) then nbtfiles5 = 0
if (nbtfiles5 gt 0) then begin
    for i=0,nbtfiles5-1 do begin
        btread,btcalfile5(i),table,nchan=nbtchan
        if (n_elements(table)  gt 1) then begin
            if (nb1chan le nbtchan) then break
            if (inoa) then break
        endif
    endfor
    if (i eq nbtfiles5) then goto,b3monthn
    calfile = btcalfile5(i)
    ;get the channel order
    b3f = b3frombt(calfile)
    b3_channel_order,b3f,order_b3,b3f=b3f,/bt
    goto,chanord
endif
b3monthn:
f(1) = 'B3'
B3Search5 = b3dir + cyer+'/'+strjoin(f(0:7),'.')+'.???.??'
B3CalFile5 = file_search(B3Search5,count=nb3files5)
if (idev) then nb3files5 = 0
if (nb3files5 gt 0) then begin
    for i=0,nb3files5-1 do begin
        b3f = b3calfile5(i)
        b3_channel_order,b3f,order_b3,b3f=b3f,cal=cal
        nbtchan = n_elements(order_b3)
        if (nb1chan le nbtchan) then break
        if (inoa) then break
    endfor
    if (i eq nb3files5) then goto,uhoh
    calfile = b3calfile5(i)
    nch = n_elements(cal.rpu_nom.phy_scl)
    table = fltarr(6,nch,256)
    table(0,*,*) = transpose(         cal.rpu_nom.phy_table) / $
                   rebin(reform(float(cal.rpu_nom.phy_scl),nch,1),nch,256)
    table(1,*,*) = transpose(         cal.rpu_nrm.phy_table) / $
                   rebin(reform(float(cal.rpu_nrm.phy_scl),nch,1),nch,256)
    table(2,*,*) = transpose(         cal.rpu_abs.phy_table) / $
                   rebin(reform(float(cal.rpu_abs.phy_scl),nch,1),nch,256)
    table(3,*,*) = transpose(         cal.apu_nom.phy_table) / $
                   rebin(reform(float(cal.apu_nom.phy_scl),nch,1),nch,256)
    table(4,*,*) = transpose(         cal.apu_nrm.phy_table) / $
                   rebin(reform(float(cal.apu_nrm.phy_scl),nch,1),nch,256)
    table(5,*,*) = transpose(         cal.apu_abs.phy_table) / $
                   rebin(reform(float(cal.apu_abs.phy_scl),nch,1),nch,256)
    ;get the channel order
    goto,chanord
endif
uhoh:
delmon=delmon + 1
if (delmon le 7) then goto,btmonthp
print,file

if (iquiet eq 0) then message,'No BT/B3 file found within +/- 7 month of the B1 file',/continue

goto,endd
tab = -1
chanord:
if (strlen(b3f) eq 0) then begin
    if (f(3) eq 'GOE-6' and f(4) eq '1986') then begin
                                         order_b3 = ['VSCHN','IRWIN']
    endif else if (f(3) eq 'MET-4' and f(4) eq '1989') then begin
                                         order_b3 = ['VSCHN','IRWIN','IRWVP']
    endif else if (f(3) eq 'GOE-6' and f(4) eq '1989') then begin
                                         order_b3 = ['VSCHN','IRWIN']
    endif else if (f(3) eq 'MET-5' and f(4) eq '1998') then begin
                                         order_b3 = ['VSCHN','IRWIN','IRWVP']
    endif else if (f(3) eq 'MET-7' and f(4) eq '1998') then begin
                                         order_b3 = ['VSCHN','IRWIN','IRWVP']
    endif else if (f(3) eq 'MET-7' and f(4) eq '2000') then begin
                                         order_b3 = ['VSCHN','IRWIN','IRWVP']
    endif else if (f(3) eq 'MET-5' and f(4) eq '1999') then begin
                                         order_b3 = ['VSCHN','IRWIN','IRWVP']
    endif else if (f(3) eq 'MET-7' and f(4) eq '1999') then begin
                                         order_b3 = ['VSCHN','IRWIN','IRWVP']
    endif else if (f(3) eq 'MET-7' and f(4) eq '2003') then begin
                                         order_b3 = ['VSCHN','IRWIN','IRWVP']
    endif else if (f(3) eq 'MET-7' and f(4) eq '2004') then begin
                                         order_b3 = ['VSCHN','IRWIN','IRWVP']
    endif else if (f(3) eq 'MET-2' and f(4) eq '1988') then begin
                                         order_b3 = ['VSCHN','IRWIN','IRWVP']
    endif else if (f(3) eq 'GOE-8' and f(4) eq '1996') then begin
                                         order_b3 = ['VSCHN','IRWIN','IRNIR','IRWVP','IRSPL']
    endif else if (f(3) eq 'GOE-9' and f(4) eq '1996') then begin
                                         order_b3 = ['VSCHN','IRWIN','IRNIR','IRWVP','IRSPL']
    endif else if (f(3) eq 'GOE-10' and f(4) eq '1999') then begin
                                         order_b3 = ['VSCHN','IRWIN','IRNIR','IRWVP','IRSPL']
    endif else if (f(3) eq 'GOE-10' and f(4) eq '2002') then begin
                                         order_b3 = ['VSCHN','IRWIN','IRNIR','IRWVP','IRSPL']
    endif else if (f(3) eq 'GOE-10' and f(4) eq '2003') then begin
                                         order_b3 = ['VSCHN','IRWIN','IRNIR','IRWVP','IRSPL']
    endif else if (f(3) eq 'GOE-12' and f(4) eq '2003') then begin
                                         order_b3 = ['VSCHN','IRWIN','IRNIR','IRWVP','IRSPL']
    endif else if (f(3) eq 'GOE-10' and f(4) eq '2004') then begin
                                         order_b3 = ['VSCHN','IRWIN','IRNIR','IRWVP','IRSPL']
    endif else if (f(3) eq 'GOE-12' and f(4) eq '2004') then begin
                                         order_b3 = ['VSCHN','IRWIN','IRNIR','IRWVP','IRSPL']
    endif else begin
        print,'Error in chanorder.'
        print,f
        stop
    endelse
endif

if (badchan(0) ge 0) then begin
    tab = fltarr(2,nb1chan+1,256)
endif else if (oldchan gt 5) then begin
    tab = fltarr(2,oldchan,256)
endif else begin
    tab = fltarr(2,oldchan,256)
endelse
;stop
for ich=0,nb1chan-1 do begin
    b3l = where(order_b3 eq chan(ich))
    if (b3l(0) ge 0) then begin
        if (chan(ich) eq 'IRWVP') then begin
;           ISCCP did a poor job of calibrating IRWVP when NOAA
;           switched to N-15. So I will use the nominal and
;           recalibratee the IRWVP myself.
            tab(0,ich,*) = table(0,b3l,*)
            tab(1,ich,*) = table(3,b3l,*)
        endif else begin
            tab(0,ich,*) = table(2,b3l,*)
            tab(1,ich,*) = table(5,b3l,*)
        endelse
    endif
endfor
checktable:
;Sometime I need to implement a test to check the calibration 
;table. In some instances, the table has one or two bad points.
;
;i=indgen(256)
;for ich=0,nb1chan-1 do begin
;    for j=0,1 do begin
;        t = reform(tab(j,ich,*))
;        dt = t[i+1] - t[i]
;        dt2=dt[i]-dt[i-1]
;        dtt=t[i+1] + t[i-1] - 2*t[i]
;        cont = abs(t[i]-t[i+1]) + abs(t[i]-t[i-1])
;        stop
;    endfor
;endfor

ctable = tab

endd:
end

;***
