function itor,num,n,m
nb=n_elements(num)
if (nb ne n) then stop

minus = bittest(num(0),7)
num(0) = num(0) + 128b * (minus eq 1)
number = long64(0)
for j=0,n-1 do begin
    number = number + long64(num(n-j-1))*long64(2)^(j*8)
endfor

itor = double(number)
for j=0,m-1 do begin
    itor = itor / 10.0d0 
endfor
itor = itor * (-1.0d0)^(minus)
if (n le 4) then itor = float(itor)
return,itor
end
;+++++++++++++++++++++++++++++++++++++++++
;****I* idl/idl.b1read_cma_fy2
; NAME
; b1read_cma_fy2
; 
; PURPOSE
; To read B1 files from CMA. The user should use b1read instead of
; using this routine directly.
; 
; USES
; * b1u_ij2ll.pro
; * bittest.pro
; * blank.pro
; * cma_bestfit.pro
; * geo_struct.pro
; * reset.pro
; * stripname.pro
; * yymmdd_2_yyjjj.pro
; 
; USED BY
; * b1read.pro
; 
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; * Dec. 05
; 
; HISTORY
; *   Jun07 - Changes since last commit: compression, cmacal, 
; * 28Jun07 - Added documentation
; 
; SOURCE

pro b1read_cma_fy2,file=file, $
                   b1u=b1u,$    ;images=images
                   images=images, $ ;return image information in the images variable
                   LINhdr=LINhdr,$ ;LINE HEADERs
                   B1VER=B1VER,$ ;returns revision info
                   FHDR=FHDR,$    ;returns file header
                   cmacal=cmacal, $
                   compress=compress, $
                   debug=debug

fhdr=0b
b1ver = '$Id: b1read_cma_fy2.pro 22762 2010-09-16 12:14:14Z kknapp $'

linhdr = {CHID:0b, $            ;Channel ID: I=IR1, J=IR2,
;                                W=WVP, K=IR4(NIR), V=VIS
          seq:0, $              ;sequence # of each line
          scan:0,$              ;scan number
          err:0b,$              ;error flag: 0=normal, 1=error
          wedge:0,$             ;western edge of earth
          eedge:0,$             ;eastern edge of earth
          beta:0l,$             ;angle between solar and earth viewed
;                               from the satellite
          pixc:0,$              ;effective pixel count
          year:0,$              ;year
          mon:0,$               ;month
          dom:0,$               ;day of month
          hour:0,$              ;hour
          min:0,$               ;minute
          sec:0,$               ;second
          hsec:0,$              ;hundredth's of a second (1/100 sec)
          blank:bytarr(10)}
scan={hdr:linhdr, $
      data:intarr(1146), $
      spare:bytarr(8)}
datrec = replicate(scan,5)
;datrec = {scan:replicate(scan,5)}

blkhdr = {recnum:0,$            ;record number
          blklen:0l,$           ;block length (bytes)
          datlin:0,$            ;line number of the data recorded in block
          recsiz:0,$            ;record size
          recsiz2:0,$           ;size of record information part for each record
          res:bytarr(8)}


filhdr = {satid:bytarr(8),$     ;SAtellite name (e.g., 'FY2-C')
          TimDat:bytarr(6),$    ;Time of observation (MJD) R*6.8
          block_count:0,$       ;number of data blocks in the data file
          chanid:bytarr(2),$    ;I=IR, V=VIS, IV=IR / VIS
          sline:0,$             ;starting line
          eline:0,$             ;ending line
          nlines:0,$            ;# of effective lines
          spix:0,$              ;starting pixel
          epix:0,$              ;ending pixel
          linres:0,$            ;line sampling rate
          pixres:0,$            ;pixel sampling rate
          res:0l,$              ;reserved
          epoch:  bytarr(6),$   ;epoch time (MJD)
          semima: bytarr(6),$   ;semimajor axis (km)
          eccen:  bytarr(6),$   ;eccentricity
          incl:   bytarr(6),$   ;orbital inclination(degrees)
          ascen:  bytarr(6),$   ;long. of ascending node
          perigee:bytarr(6),$   ;argument of perigee
          meana:  bytarr(6),$   ;mean anomaly
          ssp_lon:bytarr(6),$   ;SSP longitude
          ssp_lat:bytarr(6),$   ;SSP latitude
          epoch2: bytarr(6),$    ;epoch time (MJD) for attitude data
          angle1: bytarr(6),$   ;
          rate1:  bytarr(6),$   ;
          angle2: bytarr(6),$   ;
          rate2:  bytarr(6),$   ;
          spinra: bytarr(6),$   ;
          VISSR_x:bytarr(4),$   ;
          VISSR_y:bytarr(4),$   ;
          VISSR_z:bytarr(4),$   ;
          linang_vis:bytarr(4),$;
          linang_ir: bytarr(4),$ ;
          pixang_vis:bytarr(4),$;
          pixang_ir: bytarr(4),$ ;
          cal_vis1:lonarr(64),$ ;
          cal_vis2:lonarr(64),$ ;
          cal_vis3:lonarr(64),$ ;
          cal_vis4:lonarr(64),$ ;
          cal_ir1_t:lonarr(1024),$ ;
          cal_ir1_e:lonarr(1024),$ ;
          cal_ir2_t:lonarr(1024),$ ;
          cal_ir2_e:lonarr(1024),$ ;
          cal_wvp_t:lonarr(1024),$ ;
          cal_wvp_e:lonarr(1024),$ ;
          cal_ir4_t:lonarr(1024),$ ;
          cal_ir4_e:lonarr(1024),$ ;
          res2:0l,$
          ssplat:0l, $
          ssplon:0l, $
          sspalt:0l, $
          ssplin:0l, $
          ssppix:0l, $
          earthrad:0l, $
          grid:intarr(2,25,25), $
          res3:bytarr(76) $
         }
close,/all

;file  = '/raid/kknapp/data/isccp/b1/CMA/b1all/ISCCP.B1.1.FY2-C.2005.09.11.0800.CMA'

icomp = 0
if keyword_set(compress) then icomp=1

openr,lun,file,/swap_end,comp=icomp,/get_lun
readu,lun,filhdr
point_lun,-lun,fred 
on_ioerror,read_fail
while not(eof(lun)) do begin
    readu,lun,blkhdr
    ;check block length
    if ((long(blkhdr.recsiz)*long(blkhdr.datlin)+20l) ne $
         blkhdr.blklen) then stop
    for i=0,blkhdr.datlin-1 do begin
        readu,lun,datrec
        if (n_elements(scans) eq 0) then begin
            scans = reform(datrec,5,1)
        endif else begin
            scans = [[scans],[datrec]]
        endelse
    endfor
endwhile

goto,read_ok

;Reading the file FAILED
read_fail:
;For now, do nothing, just skip to the end

goto,problemread

read_ok:
close,lun
free_lun,lun
on_ioerror,null


;Assign calibration data to cmacal
cmacal = fltarr(2,5,1024)
;Chan order=IRWIN, IRSPL, IRWVP, IRNIR, VSCHN
;           IR1    IR2    WV     IR4    VIS
;           I      J      W      K      V
cmacal(0,0,*)    = filhdr.cal_ir1_e/1e6 ;(w/cm2/sr/um)
cmacal(0,1,*)    = filhdr.cal_ir2_e/1e6
cmacal(0,2,*)    = filhdr.cal_wvp_e/1e6
cmacal(0,3,*)    = filhdr.cal_ir4_e/1e6
cmacal(0,4,0:63) = -1

cmacal(1,0,*) = filhdr.cal_ir1_t/1e3
cmacal(1,1,*) = filhdr.cal_ir2_t/1e3
cmacal(1,2,*) = filhdr.cal_wvp_t/1e3
cmacal(1,3,*) = filhdr.cal_ir4_t/1e3
cmacal(1,4,0:63) = filhdr.cal_vis1/1e6

mjd = itor(filhdr.timdat,6,8)
ssp_lon = itor(filhdr.ssp_lon,6,6)
ssp_lat = itor(filhdr.ssp_lat,6,6)

caldat,mjd+2400000.5d0,month,day,year,hour,minute,second

imgdat = (year-1900)*10000l + month*100l + day
imgtim = hour*10000l + minute*100l + floor(second)

s=size(scans.data)
nlin = s(3)
nchan  = s(2)
npix = s(1)

nx = npix
ny = nlin

images = transpose(scans.data,[1,0,2])


common grid,gridi,gridj
common STR,timstr_,orbstr_,attstr_,imgstr_

; set up the structures
@geo_struct.pro

;reset global variables
reset

;KEPLER
semima = itor(filhdr.semima,6,8)
incl   = itor(filhdr.incl,6,8)
eccen  = itor(filhdr.eccen,6,10)
meana  = itor(filhdr.meana,6,8)
rasc   = itor(filhdr.ascen,6,8)
perigee= itor(filhdr.perigee,6,8)
;convert MJD to time/date/msec
epoch = itor(filhdr.epoch,6,8)
caldat,epoch+2400000.5d0,month,day,year,hour,minute,second
yymmdd = (year-1900)*10000l + month*100l + day
hhmmss = hour*10000l + minute*100l + floor(second)
;edate:YYJJJ,etime:HHMMSS,msec = 0.001 per millisec
edate  = yymmdd_2_yyjjj(yymmdd)
etime  = hhmmss
msec   = second mod 1.

;ATTITUDE
roll = +itor(filhdr.vissr_x,4,10)
pitch= -itor(filhdr.vissr_z,4,10)
yaw  = -itor(filhdr.vissr_y,4,10)
spra1 = -ssp_lon
spra2 = -ssp_lon
spdc1 = 90. + ssp_lat
spdc2 = 90. + ssp_lat

;IMAGE
cen_lin=filhdr.ssplin
cen_pix=filhdr.ssppix
linres= filhdr.linres
pixres= filhdr.pixres
sline=  filhdr.sline
spix=   filhdr.spix
linang= itor(filhdr.linang_ir,4,8) * 180. / !pi
pixang= -itor(filhdr.pixang_ir,4,10)* 180. / !pi

;frdate:YYJJJ, frtime:time of day in SECONDS
caldat,mjd+2400000.5d0,month,day,year,hour,minute,second
yymmdd = (year-1900)*10000l + month*100l + day
time = hour*60.*60. + minute*60. + second
frdate = yymmdd_2_yyjjj(yymmdd)
frtime = time

spper = 60./itor(filhdr.spinra,6,8)

pma    = 0.
rma    = 0.
;GREEN
gra1 = 0.
gra2 = 0.
gt1  = 0.
gt2  = 13.d0*3600.d0

orbit.ep_date = edate
orbit.ep_time = etime
orbit.ep_msec = msec
orbit.semima = semima
orbit.incl = incl
orbit.eccen = eccen
orbit.meana = meana
orbit.rgtasc = rasc
orbit.perigee = perigee
orbit.sper = spper
ORBstr = orbit

imginfo.frdate = frdate
imginfo.frtime = frtime
imginfo.cen_lin = cen_lin
imginfo.cen_pix = cen_pix
imginfo.linres = linres
imginfo.pixres = pixres
imginfo.nlin = ny ;header.nlines
imginfo.npix = 1100 ;header.npixels
imginfo.sline = sline
imginfo.spix = spix
imginfo.linang = -linang
imginfo.pixang = pixang
IMGstr = imginfo

att.roll = roll
att.pitch = pitch
att.yaw = yaw
att.rma = 0.d0
att.pma = 0.d0
ATTstr = att

sattim.d = gt2-gt1
sattim.gt1 = gt1
sattim.gt2 = gt2
sattim.gra1 = gra1
sattim.gra2 = gra2
sattim.spra1 = spra1            ;-999.d0
sattim.spra2 = spra2            ;-999.d0
sattim.spdc1 = spdc1            ;-999.d0
sattim.spdc2 = spdc2            ;-999.d0
sattim.cheb_x = -999.d0 ;ch_x
sattim.cheb_y = -999.d0 ;ch_y
sattim.cheb_z = -999.d0 ;ch_z
sattim.cheb_a = -999.d0
sattim.cheb_b = -999.d0
sattim.cheb_c = -999.d0
TIMstr = sattim

pt = filhdr.grid

cma_bestfit,file=file,ORB=ORBstr,IMG=IMGstr, $
            ATT=ATTstr,PT=PT,TIMstr=TIMstr, $
            bias=fitbias,fstdev=fitstdev,cost=fitcost, $
            /rerun,debug=debug
if keyword_set(debug) then print,'A ',fitcost

cma_bestfit,file=file,ORB=ORBstr,IMG=IMGstr, $
            ATT=ATTstr,PT=PT,TIMstr=TIMstr, $
            bias=fitbias,fstdev=fitstdev,cost=fitcost, $
            debug=debug
if keyword_set(debug) then print,'B ',fitcost

if keyword_set(debug) then stop

nx = imgstr.npix
ny = imgstr.nlin
ii = [nx/2,nx/2]
jj = [1,ny-1]
b1u_ij2ll,ii,jj,lat,lon, $
          b1TIM=TIMstr,b1ATT=ATTstr, $
          b1ORB=ORBstr,b1IMG=IMGstr, $
          glat=glat,glon=glon,grad=grad

;Direct conversion to b1u data
;---------------------------------------------------
;\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/
;
; BEGIN B1U array definitions
;
MISSING = -9999l
REVinf = strarr(6)
IMGinf = lonarr(24)
SATinf = strarr(34)
NAVinf = lonarr(200)
PREinf = intarr(nchan,20,IMGstr.nlin)

;REVISION INFORMATION
REVinf(*) = 'Missing'
REVinf(0) = stripname(file)
REVinf(3) = b1ver

;IMAGE INFORMATION
IMGinf(*) = MISSING
IMGinf(0) = frdate
IMGinf(1) = imgtim
IMGinf(2) = MISSING             ;areadir.sss
IMGinf(3) = nlin                ;areadir.nline
IMGinf(4) = npix                ;areadir.nelem
IMGinf(5) = nchan
IMGinf(6) = 2                   ;areadir.bytes
IMGinf(7) = 1                   ;N-S orientation
IMGinf(8) = 0                   ;E-W orientation
IMGinf(9) = MISSING             ;Vis averaging: -999=MISSING
IMGinf(10) = MISSING
IMGinf(11) = MISSING
IMGinf(12) = 0                  ;no image byte offset
IMGinf(13) = 0                  ;neither Imager or Sounder
IMGinf(14) = 1024

i=0
f = strsplit(stripname(file),'.',/extr)
s = strsplit(f(3),'-',/extr)
SATinf(*) = 'Missing'

SATinf(i++) = 'FY-2C'
SATinf(i++) = 'VISSR'
SATinf(i++) = 'IRWIN'
SATinf(i++) = 'IRSPL'
SATinf(i++) = 'IRWVP'
SATinf(i++) = 'IRNIR'
if (nchan lt 5) then begin
    print,'Not enough channels.'
    print,'Need to adjust algorithm for < 5 channels'
    stop
endif
SATinf(i++) = 'VSCHN'
;from FY-2C spectral responses
; FWHM
SATinf(i++) = '10.3-11.5 um'
SATinf(i++) = '11.3-12.7 um'
SATinf(i++) = '6.27-7.55 um'
SATinf(i++) = '3.53-4.04 um'
SATinf(i++) = '0.58-0.91 um'


;---------------------------
; Navigation information - NEW B1U NAV ARRAY
NAVinf(*) = MISSING
;IMGDAT
NAVinf(0) = IMGstr.sline
NAVinf(1) = IMGstr.spix
NAVinf(2) = IMGstr.cen_lin * 10
NAVinf(3) = IMGstr.cen_pix * 10
NAVinf(4) = IMGstr.linres
NAVinf(5) = IMGstr.pixres
NAVinf(6) = IMGstr.linang*1.e9
NAVinf(7) = IMGstr.pixang*1.e9
NAVinf(8) = spper*1.e6            ;Seconds per revolution*10^6
;KEPSAT#1
NAVinf(9) = 16                  ;Kepler from fit to test grid
NAVinf(10) = ORBstr.ep_date
NAVinf(11) = ORBstr.ep_time
NAVinf(12) = ORBstr.ep_msec
NAVinf(13) = ORBstr.semima *1.e2
NAVinf(14) = ORBstr.incl   *1.e3
NAVinf(15) = ORBstr.eccen  *1.e6
NAVinf(16) = ORBstr.meana  *1.e3
NAVinf(17) = ORBstr.rgtasc *1.e3
NAVinf(18) = ORBstr.perigee*1.e3

;ATTFIT
NAVinf(42) = ATTstr.roll  * 1.e6
NAVinf(43) = ATTstr.pitch * 1.e6
NAVinf(44) = ATTstr.yaw   * 1.e6
NAVinf(45) = TIMstr.spra1 * 1.e6
NAVinf(46) = TIMstr.spdc1 * 1.e6
NAVinf(47) = TIMstr.spra2 * 1.e6
NAVinf(48) = TIMstr.spdc2 * 1.e6

;FITINFO
NAVinf(52:56) = [fitcost*10l, $
                 reverse(fitbias)*1000l, $
                 reverse(fitstdev)*1000l]
;LOCDAT
NAVinf(131) = 1
NAVinf(132) = glat(0)*1e6
NAVinf(133) = glon(0)*1e6
NAVinf(134) = grad(0)*1e3
NAVinf(137) = ny-1
NAVinf(138) = glat(1)*1e6
NAVinf(139) = glon(1)*1e6
NAVinf(140) = grad(1)*1e3

;MISC
NAVinf(143) = 0                 ;GOES-NEXT Flag=0
NAVinf(147) = IMGstr.frdate
NAVinf(148) = IMGstr.frtime * 1000
NAVinf(149) = IMGstr.nlin
NAVinf(150) = IMGstr.npix

latstart = 60.
lonstart = 45.
dlat = -5.
dlon = 5.

TGRinf = lonarr(6+2*25*25)
i=0
TGRinf(i++) = 25                ;nrows
TGRinf(i++) = 25                ;ncols
TGRinf(i++) = latstart*10
TGRinf(i++) = lonstart*10
TGRinf(i++) = dlat*10
TGRinf(i++) = dlon*10
for k=0,24 do begin
    for j=0,24 do begin
        TGRinf(i++) = pt(0,j,k) ;TGRstr.line(j,k)
        TGRinf(i++) = pt(1,j,k) ;TGRstr.pixl(j,k)
    endfor
endfor

;PREinf = intarr(nchan,20,areadir.nline)
i=0
yymmdd = ((scans.hdr.year mod 100)*10000l + $
           scans.hdr.mon * 100l + $
           scans.hdr.dom)
doy = (yymmdd_2_yyjjj(yymmdd)) mod 1000l
PREinf(*,i++,*) = scans.hdr.seq ;relative scanline
PREinf(*,i++,*) = scans.hdr.scan ;absolute scan line
PREinf(*,i++,*) = MISSING
PREinf(*,i++,*) = scans.hdr.year ;year
PREinf(*,i++,*) = doy           ;doy
PREinf(*,i++,*) = scans.hdr.hour*100+ $
                  scans.hdr.min ;HHMM
PREinf(*,i++,*) = scans.hdr.sec ;Sec
PREinf(*,i++,*) = scans.hdr.hsec*10 ;MSec
PREinf(*,i++,*) = scans.hdr.eedge ;eastern edge of earth
PREinf(*,i++,*) = scans.hdr.wedge ;western edge of earth
PREinf(*,i++,*) = scans.hdr.chid
PREinf(*,i++,*) = scans.hdr.err ;validity code
PREinf(*,i++,*) = total(images,2) mod 256 ;checksum

b1u = {rev:REVinf,$
       img:IMGinf,$
       sat:SATinf,$
       nav:NAVinf,$
       tgr:TGRinf,$
       pre:PREinf,$
       image:images}      

;
; END B1U array definitions
;---------------------------------------------------
goto,endd
problemread:
images = -1
print,'Problem Reading file'
endd:

end
;***
