pro parse_header_gms,input,header,asctab
;+
; NAME:
;      parse_header_gms
;
;
; PURPOSE:
;      Used to parse the ASCTAB header in a JMA file
;      Only called by B1READ_JMA_GMS
;
; CATEGORY:
;     b1read_jma_gms
;
; MODIFICATION HISTORY:
;     Created 01/08/2004 by Ken Knapp NOAA/NCDC
;
;-

satid    = strjoin(asctab[input(0:7)])
datatime = strjoin(asctab[input(8:32)])
chid     = strjoin(asctab[input(33:36)])
proctime = strjoin(asctab[input(37:61)])
sline    = strjoin(asctab[input(62:66)])
eline    = strjoin(asctab[input(67:71)])
spixel   = strjoin(asctab[input(72:76)])
epixel   = strjoin(asctab[input(77:81)])
lineres  = strjoin(asctab[input(82:86)])
pixlres  = strjoin(asctab[input(87:91)])
nlines   = strjoin(asctab[input(92:96)])
npixels  = strjoin(asctab[input(97:101)])
etimeo   = strjoin(asctab[input(112:136)])
semima   = strjoin(asctab[input(137:161)])
eccen    = strjoin(asctab[input(162:186)])
incl     = strjoin(asctab[input(187:211)])
rasc     = strjoin(asctab[input(212:236)])
perigee  = strjoin(asctab[input(237:261)])
manom    = strjoin(asctab[input(262:286)])
ssp_lon  = strjoin(asctab[input(287:311)])
ssp_lat  = strjoin(asctab[input(312:336)])
etimea   = strjoin(asctab[input(337:361)])
axis_xy  = strjoin(asctab[input(362:386)])
xy_acc   = strjoin(asctab[input(387:411)])
axis_yz  = strjoin(asctab[input(412:436)])
yz_acc   = strjoin(asctab[input(437:461)])
spinra   = strjoin(asctab[input(462:476)]) ;rpm
vmisal_x = strjoin(asctab[input(477:491)])
vmisal_y = strjoin(asctab[input(492:506)])
vmisal_z = strjoin(asctab[input(507:521)])
lineang  = strjoin(asctab[input(522:536)])
pixelang = strjoin(asctab[input(537:551)])

header = {satid:' ',$
          time:0.d0,$           ;start time of obs. (MJD)
          chid:' ',$
          proctime:0.d0,$
          sline:0,$
          eline:0,$
          spixel:0,$
          epixel:0,$
          lineres:0,$
          pixelres:0,$
          nlines:0,$
          npixels:0,$
          epochtime1:0.d0,$
          semima:0.d0,$
          eccen:0.d0,$
          incl:0.d0,$
          rasc:0.d0,$
          perigee:0.d0,$
          manom:0.d0,$
          satlat:0.d0,$
          satlon:0.d0,$
          epochtime2:0.d0,$
          axis_xy:0.d0,$
          xy_acc:0.d0,$
          axis_yz:0.d0,$
          yz_acc:0.d0,$
          sper:0.d0,$           ;spin rate in rpm
          misal_x:0.d0,$
          misal_y:0.d0,$
          misal_z:0.d0,$
          lineang:0.d0,$
          pixelang:0.d0}

header.satid      = satid
header.time       = double(strjoin(strsplit(datatime,' ',/extr)))
header.chid       = chid
header.proctime   = double(strjoin(strsplit(proctime,' ',/extr)))
header.sline      = fix(sline)
header.eline      = fix(eline)
header.spixel     = fix(spixel)
header.epixel     = fix(epixel)
header.lineres    = fix(lineres)
header.pixelres   = fix(pixlres)
header.nlines     = fix(nlines)
header.npixels    = fix(npixels)
header.epochtime1 = double(strjoin(strsplit(etimeo,' ',/extr)))
header.semima     = double(strjoin(strsplit(semima,' ',/extr)))
header.eccen      = double(strjoin(strsplit(eccen,' ',/extr)))
header.incl       = double(strjoin(strsplit(incl,' ',/extr)))
header.rasc       = double(strjoin(strsplit(rasc,' ',/extr)))
header.perigee    = double(strjoin(strsplit(perigee,' ',/extr)))
header.manom      = double(strjoin(strsplit(manom,' ',/extr)))
header.satlat     = double(strjoin(strsplit(ssp_lat,' ',/extr)))
header.satlon     = double(strjoin(strsplit(ssp_lon,' ',/extr)))
header.epochtime2 = double(strjoin(strsplit(etimea,' ',/extr)))
header.axis_xy    = double(strjoin(strsplit(axis_xy,' ',/extr)))
header.xy_acc     = double(strjoin(strsplit(xy_acc,' ',/extr)))
header.axis_yz    = double(strjoin(strsplit(axis_yz,' ',/extr)))
header.yz_acc     = double(strjoin(strsplit(yz_acc,' ',/extr)))
header.sper       = double(strjoin(strsplit(spinra,' ',/extr)))
header.misal_x    = double(strjoin(strsplit(vmisal_x,' ',/extr)))
header.misal_y    = double(strjoin(strsplit(vmisal_y,' ',/extr)))
header.misal_z    = double(strjoin(strsplit(vmisal_z,' ',/extr)))
header.lineang    = double(strjoin(strsplit(lineang,' ',/extr)))
header.pixelang   = double(strjoin(strsplit(pixelang,' ',/extr)))
end

;****I* idl/idl.b1read_jma_gms
;+
; NAME:
;    b1read_jma_gms
;
; PURPOSE:
;    To read ISCCP-B1 files created at JMA
;
; CATEGORY:
;    ISCCP
;
; CALLING SEQUENCE:
;    This routine is called from b1read. The user should use that read
;    routine to read any and all B1 data.
;
; INPUTS:
;    file = filename of B1 data [if not provided, then the
;    dialog_pickfile will prompt user to select a file]
;
; KEYWORD PARAMETERS:
;    All keywords are outputs
;
; OUTPUTS:
;    NAVstr: Navigation info
;    lnhdr:  Line headers
;    IMAGES: The image file. The return is BYTARR[NCHAN,NX,NY]
;    
;      Structures common to all B1 read routines:
;    GEOstr: Used to navigate the data using geo_ll2ij/geo_ij2ll
;    ORBstr: Keplarian orbital information
;    IMGstr: Image information [size, resolution, ...]
;    ATTstr: Attitude info [roll, pitch, yaw, ...]
;    TIMstr: Timing data [greenwich right asc, chebyshev polynomials, ...]
;    CHNstr: Channel information [names, number, ...]
;    LINhdr: Line header information
;    TGRstr: Test grid data [used to test navigation in old data]
;    B1VER : Return version information
;    FHDR:   Return a byte array of the entire B1 file header
;
; COMMON BLOCKS:
;    These common blocks were used initially, but were replaced by the
;    structures above.
;               KEPLER - keplarian orbit elements
;               ATT - Satellite attitude parameters
;               IMAGE - Image coordinate constants
;               GREEN - greenwich right ascensions/times
;
; RESTRICTIONS:
;    External Routine:
;        reset.pro  ... performs housekeeping tasks
;        geo_struct ... defines structures
;        plot_tle   ... get TLE nav info
;        latlon2ij  ... perform navigation calculations
;
; USES
; * asctab.pro
; * b1u_ij2ll.pro
; * ftime.pro
; * geo_struct.pro
; * get_file_size.pro
; * gha.pro
; * itime.pro
; * jma_bestfit.pro
; * plot_tle.pro
; * reset.pro
; * str2num.pro
; * stripname.pro
; * yymmdd_2_yyjjj.pro
;
; USED BY
; * b1read.pro
;
; MODIFICATION HISTORY:
;       $Id: b1read_jma_gms.pro 22770 2010-09-16 12:25:15Z kknapp $
;       Created 01/08/2004 - Ken Knapp NOAA/NCDC
;       Modified03/02/2004 - Ken Knapp, NCDC
;                            Added structures which will replace
;                            common blocks.
;               03/11/2004   Imported to CVS
;               03/15/2004   Attempting to return LINhdr info in new keyword
;               03/30/2004   Added B1VER
;                            Removed common blocks
;               04/23/2004   Adding B1U functionality
;
;-
; SOURCE 

pro b1read_jma_gms,file=file,$ ;images=images,area=area,
                    lnhdr=lnhdr,$
                    images=images, $ ;return image information in the images variable
                    NAVstr=NAVstr,$ ;navigation header
                    GEOstr=GEOstr,$ ;use with geo_ij2ll and geoll2ij
                    ORBstr=ORBstr,$ ;keplarian orbital info
                    IMGstr=IMGstr,$ ;image size and scan geometry
                    ATTstr=ATTstr,$ ;satellite attitude info
                    TIMstr=TIMstr,$ ;temporal data
                    CHNstr=CHNstr,$ ;channel information
                    LINhdr=LINhdr,$ ;LINE HEADER info
                    TGRstr=TGRstr,$ ;the test grid data
                    B1VER=B1VER,$ ;returns revision info
                    FHDR=FHDR,$ ;returns file header
                   b1u=b1u, $
                   compress=compress, $
                   debug=debug, $
                   gmscal=gmscal
common grid,gridi,gridj
common STR,timstr_,orbstr_,attstr_,imgstr_

b1ver = '$Id: b1read_jma_gms.pro 22770 2010-09-16 12:25:15Z kknapp $'

; set up the structures
@geo_struct.pro

;reset global variables
reset

asctab = [ ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',$
           ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',$
           ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',$
           ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',$
           ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',$
           ' ',' ',' ',' ',' ','.','<','(','+','|','&',' ',' ',' ',$
           ' ',' ',' ',' ',' ',' ','!','$','*',')',';','^','-','/',$
           ' ',' ',' ',' ',' ',' ',' ',' ',' ',',','%','_','>','?',$
           ' ',' ',' ',' ',' ',' ',' ',' ',' ','`',':','#','@','\',$
           '=','"',' ','a','b','c','d','e','f','g','h','i',' ',' ',$
           ' ',' ',' ',' ',' ','j','k','l','m','n','o','p','q','r',$
           ' ',' ',' ',' ',' ',' ',' ','~','s','t','u','v','w','x',$
           'y','z',' ',' ',' ','[',' ',' ',' ',' ',' ',' ',' ',' ',$
           ' ',' ',' ',' ',' ',' ',' ',']',' ',' ','{','A','B','C',$
           'D','E','F','G','H','I',' ',' ',' ',' ',' ',' ','}','J',$
           'K','L','M','N','O','P','Q','R',' ',' ',' ',' ',' ',' ',$
           '\\',' ','S','T','U','V','W','X','Y','Z',' ',' ',' ',' ',$
           ' ',' ','0','1','2','3','4','5','6','7','8','9',' ',' ',$
           ' ',' ',' ',' ']

; Get a file name if none provided
if not(keyword_set(file)) then begin
    file = dialog_pickfile(path='~/data/ISCCP-test',filter='*GMS*JMA*')
endif
if not(file_test(file)) then begin
    print,'Non-existent file: '+file
    goto,endd
endif

;file_info(file)
filesize=get_file_size(file,comp=icomp)

f = strsplit(file,'.',/extr)

if (f(3) eq 'GMS-5' or f(3) eq 'GOES9' or f(3) eq 'GOE-9') then begin
    lenhead = 21200
    igms5 = 1
endif else begin
    igms5 = 0
    lenhead = 10000
endelse

fhdr = bytarr(lenhead)
rhdr = bytarr(24)

;TEST for ASCII vs EBDIC
ascii = 0
asctest = bytarr(8)

icomp=0
if keyword_set(compress) then icomp=1

openr,unit,file,/get_lun,comp=icomp
readu,unit,asctest
close,unit
free_lun,unit
if (asctest(7) eq 32b) then ascii = 1 $;32 = SPACE in ASCII
else if (asctest(7) eq 64) then ascii = 0 else stop

if (ascii) then begin
    asctab = strarr(256)
    for i=0,255 do begin
        asctab[i] = string(byte(i))
    endfor
endif

;on_ioerror,problemread
openr,unit,file,/get_lun,comp=icomp

;read file header
readu,unit,fhdr

;parse out the header
parse_header_gms,fhdr,header,asctab

navstr=header
;GET CHANNEL INFO
flag_ir = 0
flag_vis = 0
flag_wv = 0
flag_ir2 = 0
nchan = 0
chid = header.chid
i = strpos(chid,'I')
if (i ge 0) then begin
    flag_ir = 1
    nchan = nchan + 1
endif
i = strpos(chid,'V')
if (i ge 0) then begin
    flag_vis = 1
    nchan = nchan + 1
endif
i = strpos(chid,'J')
if (i ge 0) then begin
    flag_ir2 = 1
    nchan = nchan + 1
endif
i = strpos(chid,'W')
if (i ge 0) then begin
    flag_wv = 1
    nchan = nchan + 1
endif
chninfo.nchan = nchan
;loop over records
idone = 0
point_lun,-unit,lasthere
nextscan = 0
while (idone eq 0) do begin
    readu,unit,rhdr
    totrhdr = total(rhdr)
    scan1 = long(strjoin(asctab[rhdr(1:4)]))

    if (nextscan ne 0 and $
        nextscan ne scan1) then begin
        ;if the scanline info doesn't match the data
        ;from the last header, then stop reading
        goto,outofit
    endif
    
    if (totrhdr ne 0) then begin
        if (rhdr(0) eq 64b) then begin
            scan1 = long(strjoin(asctab(rhdr(0:4))))
            reclen= long(strjoin(asctab(rhdr(5:10))))
            nscan = long(strjoin(asctab(rhdr(11:13))))
            scnlen= long(strjoin(asctab(rhdr(14:18))))
            scnhdrlen=long(strjoin(asctab(rhdr(19:21))))
            nchan = long(strjoin(asctab(rhdr(22:23))))
            if (scan1 gt 9900) then message,'scan1 > 9900:'+asctab(rhdr(0:4))
        endif else begin
            scan1 = long(strjoin(string(rhdr(0:4))))
            reclen= long(strjoin(string(rhdr(5:10))))
            nscan = long(strjoin(string(rhdr(11:13))))
            scnlen= long(strjoin(string(rhdr(14:18))))
            scnhdrlen=long(strjoin(string(rhdr(19:21))))
            nchan = long(strjoin(string(rhdr(22:23))))
            if (scan1 gt 9900) then message,'Scan1 > 9900:'+string(rhdr(0:4))
        endelse
        nextscan=scan1 + nscan
        
        scan = {hdr:bytarr(scnhdrlen),line:bytarr(1100),filler:bytarr(8)}

; THE GOES9 FILES HAVE NSCAN=21 ... BUT THAT IS WRONG, IT SHOULD BE 28
        if (nscan eq 21 and (ascii or $
                             f(3) eq 'GOES9' or $
                             f(3) eq 'GOE-9')) then nscan = 28
        input = replicate(scan,nscan)
        readu,unit,input
        point_lun,-unit,here

; THE FOLLOWING DOES NOT WORK FOR GOES9
        if (not(ascii or $
                f(3) eq 'GOES9' or $
                f(3) eq 'GOE-9')) then point_lun,unit,lasthere+reclen
        lasthere=here

        chid = asctab[input.hdr(0)]
                                ;I = IR Channel 10.5 micron
        match = where(chid eq 'I',nmatch)
        if (nmatch gt 0) then begin
            if (n_elements(ch1) eq 0) then ch1 = input(match) else $
              ch1 = [ch1,input(match)]
            flag_ir = 1
        endif
                                ;J = IR Channel (11.5 micron ... this is a guess)
        match = where(chid eq 'J',nmatch)
        if (nmatch gt 0) then begin
            if (n_elements(ch2) eq 0) then ch2 = input(match) else $
              ch2 = [ch2,input(match)]
            flag_ir2 = 1
        endif
                                ;W = IR Water Vapor Channel
        match = where(chid eq 'W',nmatch)
        if (nmatch gt 0) then begin
            if (n_elements(ch3) eq 0) then ch3 = input(match) else $
              ch3 = [ch3,input(match)]
            flag_wv = 1
        endif
                                ;V = Vis Channel 0.7 micron
        match = where(chid eq 'V',nmatch)
        if (nmatch gt 0) then begin
            if (n_elements(ch4) eq 0) then ch4 = input(match) else $
              ch4 = [ch4,input(match)]
            flag_vis = 1
        endif
    endif
    idone = (totrhdr eq 0) or eof(unit) or here eq filesize
endwhile
outofit:

close,unit
free_lun,unit
on_ioerror,null
nchan = flag_vis + flag_ir + flag_ir2 + flag_wv
chninfo.nchan = nchan
IRL = (size(ch1.line))[2]
if (flag_ir) then images = ch1.line

if (flag_vis)then begin
    nn = (size(ch4.line))[2]
    if (nn eq irl) then  images = [[[images]],[[ch4.line]]] $
      else begin
        file_move,file,getenv('ISCCPDIR')+'b1/failed-read/'+stripname(file), $
                  /overwrite,/allow_same
        images = -1
        message,'Error in visible imagery',/info
        return
    endelse
endif
if (flag_ir2)then begin
    nn = (size(ch2.line))[2]
    if (nn eq irl) then  images = [[[images]],[[ch2.line]]] $
      else begin
        file_move,file,getenv('ISCCPDIR')+'b1/failed-read/'+stripname(file), $
                  /overwrite,/allow_same
        images = -1
        message,'Error in IR imagery',/info
        return
    endelse
endif
if (flag_wv) then begin
    nn = (size(ch3.line))[2]
    if (nn eq irl) then  images = [[[images]],[[ch3.line]]] $
      else begin
        file_move,file,getenv('ISCCPDIR')+'b1/failed-read/'+stripname(file), $
                  /overwrite,/allow_same
        images = -1
        message,'Error in water vapor imagery',/info
        return
    endelse
endif
s= size(images)

if (s(3) eq 1) then images = reform(images,1,s(1),s(2))
if (s(0) gt 2) then images = transpose(images,[2,0,1])
lnhdr = ch1.hdr

s=size(lnhdr)
ny = s(2)
;set up the line headers
LineHeader = {kind:0b,$         ;Data type: I:IR1, J:IR2, W:WV, V:VIS
              seq:0l,$          ;sequence number of each line for each channel
              scnnum:0l,$       ;original scan line number
              val:0b,$          ;validity code: 0=normal, 1=error
              WEdge:0l,$        ;pixel of Earth's western edge
              EEdge:0l,$        ;pixel of Earth's eastern edge
              beta:0.0,$        ;beta angle
              time:intarr(7),$  ;time: YY,MM,DD,HH,mm,SS,msec
              space:0,$         ;Reserved: space code
              res:0l,$
              chksum:0l}
linhdr= replicate(LineHeader,nchan,ny)

; ASSIGN VALUES TO THE LINE HEADERS AS NEEDED
ich = 0
if (flag_ir) then begin
;CHANNEL ONE
    linhdr(ich,*).kind =                                ch1.hdr(0,*)
    linhdr(ich,*).seq  =  reform(str2num(strjoin(asctab[ch1.hdr(1:4)]),/i4),1,ny)
    linhdr(ich,*).scnnum= reform(str2num(strjoin(asctab[ch1.hdr(5:8)]),/i4),1,ny)
    linhdr(ich,*).val =                                 ch1.hdr(9,*)
    linhdr(ich,*).WEdge = reform(str2num(strjoin(asctab[ch1.hdr(10:13)]),/i4),1,ny)
    linhdr(ich,*).EEdge = reform(str2num(strjoin(asctab[ch1.hdr(14:17)]),/i4),1,ny)
    linhdr(ich,*).beta=   reform(str2num(strjoin(asctab[ch1.hdr(18:32)]),/r4),1,ny)
    linhdr(ich,*).time(0)=reform(str2num(strjoin(asctab[ch1.hdr(33:36)]),/i2),1,ny)
    linhdr(ich,*).time(1)=reform(str2num(strjoin(asctab[ch1.hdr(37:38)]),/i2),1,ny)
    linhdr(ich,*).time(2)=reform(str2num(strjoin(asctab[ch1.hdr(39:40)]),/i2),1,ny)
    linhdr(ich,*).time(3)=reform(str2num(strjoin(asctab[ch1.hdr(41:42)]),/i2),1,ny)
    linhdr(ich,*).time(4)=reform(str2num(strjoin(asctab[ch1.hdr(43:44)]),/i2),1,ny)
    linhdr(ich,*).time(5)=reform(str2num(strjoin(asctab[ch1.hdr(45:46)]),/i2),1,ny)
    linhdr(ich,*).time(6)=reform(str2num(strjoin(asctab[ch1.hdr(47:49)]),/i2),1,ny)
    linhdr(ich,*).space  =                          fix(ch1.hdr(50:51),0,1,ny)
    linhdr(ich,*).res    =             swap_endian(long(ch1.filler(0:3),0,1,ny))
    linhdr(ich,*).chksum =             swap_endian(long(ch1.filler(4:7),0,1,ny))
    ich=ich+1
endif
if (flag_vis) then begin
; CHANNEL TWO
    linhdr(ich,*).kind =                            ch4.hdr(0,*)
    linhdr(ich,*).seq  =  reform(str2num(strjoin(asctab[ch4.hdr(1:4)]),/i4),1,ny)
    linhdr(ich,*).scnnum= reform(str2num(strjoin(asctab[ch4.hdr(5:8)]),/i4),1,ny)
    linhdr(ich,*).val =                             ch4.hdr(9,*)
    linhdr(ich,*).WEdge = reform(str2num(strjoin(asctab[ch4.hdr(10:13)]),/i4),1,ny)
    linhdr(ich,*).EEdge = reform(str2num(strjoin(asctab[ch4.hdr(14:17)]),/i4),1,ny)
    linhdr(ich,*).beta=   reform(str2num(strjoin(asctab[ch4.hdr(18:32)]),/r4),1,ny)
    linhdr(ich,*).time(0)=reform(str2num(strjoin(asctab[ch4.hdr(33:36)]),/i2),1,ny)
    linhdr(ich,*).time(1)=reform(str2num(strjoin(asctab[ch4.hdr(37:38)]),/i2),1,ny)
    linhdr(ich,*).time(2)=reform(str2num(strjoin(asctab[ch4.hdr(39:40)]),/i2),1,ny)
    linhdr(ich,*).time(3)=reform(str2num(strjoin(asctab[ch4.hdr(41:42)]),/i2),1,ny)
    linhdr(ich,*).time(4)=reform(str2num(strjoin(asctab[ch4.hdr(43:44)]),/i2),1,ny)
    linhdr(ich,*).time(5)=reform(str2num(strjoin(asctab[ch4.hdr(45:46)]),/i2),1,ny)
    linhdr(ich,*).time(6)=reform(str2num(strjoin(asctab[ch4.hdr(47:49)]),/i2),1,ny)
    linhdr(ich,*).space  =                          fix(ch4.hdr(50:51),0,1,ny) 
    linhdr(ich,*).res    =             swap_endian(long(ch4.filler(0:3),0,1,ny))
    linhdr(ich,*).chksum =             swap_endian(long(ch4.filler(4:7),0,1,ny))
    ich=ich+1
endif
if (flag_ir2) then begin
    linhdr(ich,*).kind =                                ch2.hdr(0,*)
    linhdr(ich,*).seq  =  reform(str2num(strjoin(asctab[ch2.hdr(1:4)]),/i4),1,ny)
    linhdr(ich,*).scnnum= reform(str2num(strjoin(asctab[ch2.hdr(5:8)]),/i4),1,ny)
    linhdr(ich,*).val =                                 ch2.hdr(9,*)
    linhdr(ich,*).WEdge = reform(str2num(strjoin(asctab[ch2.hdr(10:13)]),/i4),1,ny)
    linhdr(ich,*).EEdge = reform(str2num(strjoin(asctab[ch2.hdr(14:17)]),/i4),1,ny)
    linhdr(ich,*).beta=   reform(str2num(strjoin(asctab[ch2.hdr(18:32)]),/r4),1,ny)
    linhdr(ich,*).time(0)=reform(str2num(strjoin(asctab[ch2.hdr(33:36)]),/i2),1,ny)
    linhdr(ich,*).time(1)=reform(str2num(strjoin(asctab[ch2.hdr(37:38)]),/i2),1,ny)
    linhdr(ich,*).time(2)=reform(str2num(strjoin(asctab[ch2.hdr(39:40)]),/i2),1,ny)
    linhdr(ich,*).time(3)=reform(str2num(strjoin(asctab[ch2.hdr(41:42)]),/i2),1,ny)
    linhdr(ich,*).time(4)=reform(str2num(strjoin(asctab[ch2.hdr(43:44)]),/i2),1,ny)
    linhdr(ich,*).time(5)=reform(str2num(strjoin(asctab[ch2.hdr(45:46)]),/i2),1,ny)
    linhdr(ich,*).time(6)=reform(str2num(strjoin(asctab[ch2.hdr(47:49)]),/i2),1,ny)
    linhdr(ich,*).space  =                          fix(ch2.hdr(50:51),0,1,ny) 
    linhdr(ich,*).res    =             swap_endian(long(ch2.filler(0:3),0,1,ny))
    linhdr(ich,*).chksum =             swap_endian(long(ch2.filler(4:7),0,1,ny))
    ich=ich+1
endif
if (flag_wv) then begin
    linhdr(ich,*).kind =                                ch3.hdr(0,*)
    linhdr(ich,*).seq  =  reform(str2num(strjoin(asctab[ch3.hdr(1:4)]),/i4),1,ny)
    linhdr(ich,*).scnnum= reform(str2num(strjoin(asctab[ch3.hdr(5:8)]),/i4),1,ny)
    linhdr(ich,*).val =                                 ch3.hdr(9,*)
    linhdr(ich,*).WEdge = reform(str2num(strjoin(asctab[ch3.hdr(10:13)]),/i4),1,ny)
    linhdr(ich,*).EEdge = reform(str2num(strjoin(asctab[ch3.hdr(14:17)]),/i4),1,ny)
    linhdr(ich,*).beta  = reform(str2num(strjoin(asctab[ch3.hdr(18:32)]),/r4),1,ny)
    linhdr(ich,*).time(0)=reform(str2num(strjoin(asctab[ch3.hdr(33:36)]),/i2),1,ny)
    linhdr(ich,*).time(1)=reform(str2num(strjoin(asctab[ch3.hdr(37:38)]),/i2),1,ny)
    linhdr(ich,*).time(2)=reform(str2num(strjoin(asctab[ch3.hdr(39:40)]),/i2),1,ny)
    linhdr(ich,*).time(3)=reform(str2num(strjoin(asctab[ch3.hdr(41:42)]),/i2),1,ny)
    linhdr(ich,*).time(4)=reform(str2num(strjoin(asctab[ch3.hdr(43:44)]),/i2),1,ny)
    linhdr(ich,*).time(5)=reform(str2num(strjoin(asctab[ch3.hdr(45:46)]),/i2),1,ny)
    linhdr(ich,*).time(6)=reform(str2num(strjoin(asctab[ch3.hdr(47:49)]),/i2),1,ny)
    linhdr(ich,*).space  =                          fix(ch3.hdr(50:51),0,1,ny)
    linhdr(ich,*).res    =             swap_endian(long(ch3.filler(0:3),0,1,ny))
    linhdr(ich,*).chksum =             swap_endian(long(ch3.filler(4:7),0,1,ny))
    ich=ich+1
endif

;
; SET ORBIT/ATTITUDE PARAMETERS
;

; this sets up the pt values ... which are line/pixel pairings for
; known lat/lons.
if (igms5) then begin
    t=dblarr(11*11*2)
    sheader = strjoin(asctab[fhdr(19451:21144)])
    reads,sheader,t,format='(242f7.2)'
endif else begin

    t = dblarr(2*11*11)
    sheader = strjoin(asctab[fhdr(8187:8187+1694-1)])
    ;reform sheader into 242 ascii values
    sheader = string(reform(byte(sheader),7,242))
    t = str2num(sheader,/r4)
;    reads,sheader,t,format='(242f7.2)'
;    pt2 = reform(t,2,11,11)

endelse
pt = reform(t,2,11,11)
rad = 180. / !pi

;KEPLER
semima = header.semima
incl   = header.incl
eccen  = header.eccen
meana  = header.manom
rasc   = header.rasc
perigee= header.perigee
;convert MJD to time/date/msec
caldat,header.epochtime1+2400000.5d0,month,day,year,hour,minute,second
yymmdd = (year-1900)*10000l + month*100l + day
hhmmss = hour*10000l + minute*100l + floor(second)
;edate:YYJJJ,etime:HHMMSS,msec = 0.001 per millisec
edate  = yymmdd_2_yyjjj(yymmdd)
etime  = hhmmss
msec   = second mod 1.

; GET CORRECT KEPLER ELEMENTS FOR 05/22/03 to 05/5/04 (inclusive)
; The Kepler elements were wrong for these times, so need to input the
; correct fvalues from files sent from JMA
iupdate = 0
if ((f(4) eq '2003' and (f(5) ge 6 or (f(5) eq '05' and f(6) ge 22))) or $
    (f(4) eq '2004' and (f(5) le 4 or (f(5) eq '05' and f(6) le  5)))) then begin
    iupdate=1
    kdir = getenv('ISCCPDIR')+'b1kep/'
    ind = long(f(6))*10000 + long(f(7))
    kepfile = kdir + 'OrbitParam'+f(4)+f(5)+'.bin'
    ele = dblarr(9)
    kdate = 0l
    openr,lun,kepfile,/get,/swap_end
    while (kdate ne ind and not(eof(lun))) do begin
        readu,lun,kdate
        readu,lun,ele
    endwhile
    close,lun
    free_lun,lun
    if (kdate eq ind) then begin
        mjdepoch=ele(0)         ;Epoch time (MJD)
        semima = ele(1)
        eccen  = ele(2)
        incl   = ele(3)
        rasc   = ele(4)
        perigee= ele(5)
        meana  = ele(6)
        caldat,mjdepoch+2400000.5d0, $
               month,day,year,hour,minute,second
        yymmdd = (year-1900)*10000l + month*100l + day
        hhmmss = hour*10000l + minute*100l + floor(second)
;       edate:YYJJJ,etime:HHMMSS,msec = 0.001 per millisec
        edate  = yymmdd_2_yyjjj(yymmdd)
        etime  = hhmmss
        msec   = second mod 1.
        newdate = edate
        newtime = etime
        newmsec = msec
    endif else iupdate = 0
endif


;ATT
roll  = +header.misal_x * rad
pitch = -header.misal_z * rad
yaw   = -header.misal_y * rad

spra1 = -header.satlon ;+ header.axis_yz*rad
spra2 = -header.satlon ;+ header.axis_yz*rad
spdc1 = 90.+header.satlat   - header.axis_yz*rad ;-180.
spdc2 = 90.+header.satlat   - header.axis_yz*rad ;-180.



;IMAGE
cen_lin=(header.sline + header.eline) / 2.   ;;t(822)
cen_pix=(header.spixel + header.epixel) / 2. ;;t(823)
linres = header.lineres
pixres = header.pixelres
sline  = header.sline  ;* linres
spix   = header.spixel ;* pixres
linang =  header.lineang * rad ;* float(linres)
pixang = -header.pixelang* rad ;* float(pixres)


;frdate:YYJJJ, frtime:time of day in SECONDS
caldat,header.time+2400000.5d0,month,day,year,hour,minute,second
yymmdd = (year-1900)*10000l + month*100l + day
time = hour*60.*60. + minute*60. + second
frdate = yymmdd_2_yyjjj(yymmdd)
frtime = time

spper  = 60. / header.sper
if (abs(header.sper) lt 1.e-5) then spper = 0.6

pma    = 0.
rma    = 0.
;GREEN
gra1 = 0.
gra2 = 0.
gt1  = 0.
gt2  = 13.d0*3600.d0


;
;Assign variables to structures
;

;All values are directly from the common block, so this if can be
;copied to each B1READ routine
orbit.ep_date = edate
orbit.ep_time = etime
orbit.ep_msec = msec
orbit.semima = semima
orbit.incl = incl
orbit.eccen = eccen
orbit.meana = meana
orbit.rgtasc = rasc
orbit.perigee = perigee
orbit.sper = spper
ORBstr = orbit

if (f(3) eq 'GOES9' or f(3) eq 'GOE-9') then begin ;or ascii) then begin
    st = 'goes-9'
    year = fix(f(4))
    jday = yymmdd_2_yyjjj((year-1900)*10000l + f(5)*100+f(6))-((year-1900)*1000l)
    plot_tle,sat=st,year=year,jday=jday,ORBstr=TLEORBstr ;,/out
endif

;All values are directly from the common block, so this if can be
;copied to each B1READ routine
imginfo.frdate = frdate
imginfo.frtime = frtime
imginfo.cen_lin = cen_lin
imginfo.cen_pix = cen_pix
imginfo.linres = linres
imginfo.pixres = pixres
imginfo.nlin = ny ;header.nlines
imginfo.npix = 1100 ;header.npixels
imginfo.sline = sline
imginfo.spix = spix
imginfo.linang = -linang
imginfo.pixang = pixang
IMGstr = imginfo

att.roll = roll
att.pitch = pitch
att.yaw = yaw
att.rma = 0.d0
att.pma = 0.d0
ATTstr = att

sattim.d = gt2-gt1
sattim.gt1 = gt1
sattim.gt2 = gt2
sattim.gra1 = gra1
sattim.gra2 = gra2
sattim.spra1 = spra1            ;-999.d0
sattim.spra2 = spra2            ;-999.d0
sattim.spdc1 = spdc1            ;-999.d0
sattim.spdc2 = spdc2            ;-999.d0
sattim.cheb_x = -999.d0 ;ch_x
sattim.cheb_y = -999.d0 ;ch_y
sattim.cheb_z = -999.d0 ;ch_z
sattim.cheb_a = -999.d0
sattim.cheb_b = -999.d0
sattim.cheb_c = -999.d0
TIMstr = sattim

ij = strpos(header.chid,'I')
if (ij ge 0) then chninfo.ch_name(ij) = 'IRWIN'
ij = strpos(header.chid,'V')
if (ij ge 0) then chninfo.ch_name(ij) = 'VSCHN'
ij = strpos(header.chid,'W')
if (ij ge 0) then chninfo.ch_name(ij) = 'IRWVP'
ij = strpos(header.chid,'J')
if (ij ge 0) then chninfo.ch_name(ij) = 'IRSPL'
chninfo.ch_info = 'none'
if (f(3) eq 'GOES9' or f(3) eq 'GOE-9' or (ascii and nchan gt 2)) then begin
    chninfo.ch_name(2) = 'IRSPL'
    chninfo.ch_name(3) = 'IRWVP'
endif
if (nchan eq 4) then chninfo.ch_name(0:3) = ['IRWIN','VSCHN','IRSPL','IRWVP']
if (nchan eq 3) then chninfo.ch_name(0:2) = ['IRWIN','IRSPL','IRWVP']
CHNstr = chninfo





;
; CALCULATE THE "CORRECT" Orbit & Attitude based on the test grid
; provided.

jma_bestfit,file=file,ORB=ORBstr,IMG=IMGstr, $
            ATT=ATTstr,PT=PT,TIMstr=TIMstr, $
            bias=fitbias,fstdev=fitstdev,cost=fitcost,b1kfile=b1kfile
;print,'A ',fitcost

jma_bestfit,file=file,ORB=ORBstr,IMG=IMGstr, $
            ATT=ATTstr,PT=PT,TIMstr=TIMstr, $
            bias=fitbias,fstdev=fitstdev,cost=fitcost,b1kfile=b1kfile, $
            /rerun
;print,'B ',fitcost

if (fitcost gt 1.e6) then begin
;----------------------------------------------------------------
;TRY ASSIGNING THE ORBITAL PARAMETERS BASED SOLELY ON THE SUB SAT
;POINTS
; This is based on what we do for METEOSAT, .... which has zero
; inclination, so not sure what to do here... will use lat as
; inclination for now.
    RECLON = header.satlon
    RECLAT = header.satlat
    print,'REC:',RECLAT,RECLON
    jday = frdate mod 1000
    angle = gha(julday(1,jday,year,ftime(etime),0,msec/1000.),/degree)
    ; set up kepler elements
    semima = 42166.264d0
    eccen = 0.d0
    incl =  RECLAT
    meana = (RECLON + angle) mod 360.
    perigee = 0.d0 
    rgtasc = 0.d0 
    orbit.ep_date = frdate      ;edate
    orbit.ep_time = etime       ;etime
    orbit.ep_msec = msec        ;msec
    orbit.semima = semima       ;semima
    orbit.incl = incl           ;incl
    orbit.eccen = eccen         ;eccen
    orbit.meana = meana         ;meana
    orbit.rgtasc = rgtasc       ;rasc
    orbit.perigee = perigee     ;perigee
    orbit.sper = 0.6            ;spper
    ORBstr = orbit
    file_delete,b1kfile 
    jma_bestfit,file=file,ORB=ORBstr,IMG=IMGstr, $
                ATT=ATTstr,PT=PT,TIMstr=TIMstr, $
                bias=fitbias,fstdev=fitstdev,cost=fitcost,b1kfile=b1kfile
print,'C ',fitcost
    openw,13,getenv('ISCCPDIR')+'b1/new-JMA-files',/append
    printf,13,file
    close,13
endif

timstr_=timstr
orbstr_=orbstr
attstr_=attstr
imgstr_=imgstr


if (f(3) eq 'GOES9' or f(3) eq 'GOE-9') then begin
    latstart= 50.
    lonstart = 105.
endif else begin
    latstart= 50.
    lonstart = 90.
endelse

inlat = 0.0d0
inlon = 150.d0

ATTstr.spra = TIMstr.spra1
ATTstr.spdc = TIMstr.spdc1

TGRID.n    = 11
TGRID.slat = latstart ;50.   ;NAVstr.latn / 10.
TGRID.slon = lonstart ;90.   ;NAVstr.lonw / 10.
TGRID.dlat = -10.  ;NAVstr.dlat / 10.
TGRID.dlon =  10.  ;NAVstr.dlon / 10.
TGRID.pixl(0:TGRID.N-1,0:TGRID.N-1) = pt(1,*,*)*100  ;NAVstr.lnn
TGRID.line(0:TGRID.N-1,0:TGRID.N-1) = pt(0,*,*)*100  ;NAVstr.ltn
TGRstr=TGRID

nx = imgstr.npix
ny = imgstr.nlin
ii = [nx/2,nx/2]
jj = [1,ny-1]
b1u_ij2ll,ii,jj,lat,lon, $
          b1TIM=TIMstr,b1ATT=ATTstr, $
          b1ORB=ORBstr,b1IMG=IMGstr, $
          glat=glat,glon=glon,grad=grad

;get calibration information where possible
if (igms5) then begin
    gmscal = fltarr(2,nchan,256)
    irloc = where(chninfo.ch_name eq 'IRWIN')
    if (irloc(0) ge 0 and irloc(0) lt nchan) then begin
        section = fhdr(552:2343)
        if (ascii) then $
          IRtempTable = string(section) else $
          IRtempTable = strjoin(asctab(section))
        input = fltarr(256)
        reads,IRtempTable,input,format='(256f7.3)'
        gmscal(1,irloc(0),*) = input
        section = fhdr(2344:6183)
        if (ascii) then $
          IRradTable = string(section) else $
          IRradTable = strjoin(asctab(section))
        reads,IRradTable,input,format='(256f15.8)'
        gmscal(0,irloc(0),*) = input
    endif
    irloc = where(chninfo.ch_name eq 'IRSPL')
    if (irloc(0) ge 0 and irloc(0) lt nchan) then begin
        section = fhdr(6184:7975)
        if (ascii) then $
          IRtempTable = string(section) else $
          IRtempTable = strjoin(asctab(section))
        input = fltarr(256)
        reads,IRtempTable,input,format='(256f7.3)'
        gmscal(1,irloc(0),*) = input
        section = fhdr(7976:11815)
        if (ascii) then $
          IRradTable = string(section) else $
          IRradTable = strjoin(asctab(section))
        reads,IRradTable,input,format='(256f15.8)'
        gmscal(0,irloc(0),*) = input
    endif
    irloc = where(chninfo.ch_name eq 'IRWVP')
    if (irloc(0) ge 0 and irloc(0) lt nchan) then begin
        section = fhdr(11816:13607)
        if (ascii) then $
          IRtempTable = string(section) else $
          IRtempTable = strjoin(asctab(section))
        input = fltarr(256)
        reads,IRtempTable,input,format='(256f7.3)'
        gmscal(1,irloc(0),*) = input
        section = fhdr(13608:17447)
        if (ascii) then $
          IRradTable = string(section) else $
          IRradTable = strjoin(asctab(section))
        reads,IRradTable,input,format='(256f15.8)'
        gmscal(0,irloc(0),*) = input
    endif
    irloc = where(chninfo.ch_name eq 'VSCHN')
    if (irloc(0) ge 0 and irloc(0) lt nchan) then begin
        section = fhdr(17448:19239)
        if (ascii) then $
          VISalb = string(section) else $
          VISalb = strjoin(asctab(section))
        gmscal(1,irloc(0),*) = VISalb
    endif
endif else begin
    gmscal = fltarr(2,nchan,256)
    irloc = where(chninfo.ch_name eq 'IRWIN')
    if (irloc(0) ge 0 and irloc(0) lt nchan) then begin
        section = fhdr(552:2343)
        if (ascii) then $
          IRtempTable = string(section) else $
          IRtempTable = strjoin(asctab(section))
        input = fltarr(256)
        reads,IRtempTable,input,format='(256f7.3)'
        gmscal(1,irloc(0),*) = input
        section = fhdr(2344:6183)
        if (ascii) then $
          IRradTable = string(section) else $
          IRradTable = strjoin(asctab(section))
        reads,IRradTable,input,format='(256f15.8)'
        gmscal(0,irloc(0),*) = input
    endif
    irloc = where(chninfo.ch_name eq 'VSCHN')
    if (irloc(0) ge 0 and irloc(0) lt nchan) then begin
        section = fhdr(6184:7975)
        if (ascii) then $
          VISalb = string(section) else $
          VISalb = strjoin(asctab(section))
        gmscal(1,irloc(0),*) = VISalb
    endif
endelse


if keyword_set(debug) then stop
;---------------------------------------------------
;\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/
;
; BEGIN B1U array definitions
;
MISSING = -9999l
REVinf = strarr(6)
IMGinf = lonarr(24)
SATinf = strarr(34)
NAVinf = lonarr(200)
PREinf = intarr(nchan,20,IMGstr.nlin)

;REVISION INFORMATION
REVinf(*) = 'Missing'
REVinf(0) = stripname(file)
REVinf(3) = b1ver

;IMAGE INFORMATION
IMGinf(*) = MISSING
IMGinf(0) = IMGstr.frdate
IMGinf(1) = itime(IMGstr.frtime/3600.)
IMGinf(2) = MISSING             ;areadir.sss
IMGinf(3) = IMGstr.nlin         ;areadir.nline
IMGinf(4) = IMGstr.npix         ;areadir.nelem
IMGinf(5) = nchan
IMGinf(6) = 1                   ;areadir.bytes
IMGinf(7) = 1                   ;N-S orientation
IMGinf(8) = 0                   ;E-W orientation
IMGinf(9) = MISSING             ;Vis averaging: -999=MISSING
IMGinf(10) = MISSING
IMGinf(11) = MISSING
IMGinf(12) = 0                  ;no image byte offset
IMGinf(13) = 0                  ;neither Imager or Sounder
;Satellite information

i=0
f = strsplit(stripname(file),'.',/extr)
s = strsplit(f(3),'-',/extr)
SATinf(*) = 'Missing'

if (n_elements(s) eq 1 or s(0) eq 'GOE') then begin
    SATinf(i++) = 'GOES-9'
    SATinf(i++) = 'Imager'
endif else begin    
    sn = fix(s(1))
    SATinf(i++) = 'GMS-'+s(1)
    SATinf(i++) = 'VISSR'
endelse
for j=0,nchan-1 do begin
    SATinf(i++) = chninfo.ch_name(j)
endfor

;  GMS channel info will be FWHM bounds ... estimated from the
;  ISCCP web page
gmsinfo = [['0.49-0.71 um','10.4-11.8 um'], $
           ['0.50-0.72 um','10.3-11.3 um'], $
           ['0.51-0.75 um','10.3-12.6 um'], $
           ['0.49-0.73 um','10.3-11.3 um']]

gms5info = ['0.51-0.99 um', $
            '10.2-11.4 um', $
            '10.9-12.1 um', $
            '6.70-7.16 um']
goes9info = ['0.52-0.74 um', $
             '10.2-11.2 um', $
             '11.6-12.5 um', $
             '6.50-7.00 um']
for j=0,nchan-1 do begin
    if (igms5) then begin
        if (chninfo.ch_name(j) eq 'VSCHN') then tmp = gms5info(0)
        if (chninfo.ch_name(j) eq 'IRWIN') then tmp = gms5info(1)
        if (chninfo.ch_name(j) eq 'IRSPL') then tmp = gms5info(2)
        if (chninfo.ch_name(j) eq 'IRWVP') then tmp = gms5info(3)
    endif else if (f(3) eq 'GOES9' or f(3) eq 'GOE-9' or ascii) then begin
        if (chninfo.ch_name(j) eq 'VSCHN') then tmp = goes9info(0)
        if (chninfo.ch_name(j) eq 'IRWIN') then tmp = goes9info(1)
        if (chninfo.ch_name(j) eq 'IRSPL') then tmp = goes9info(2)
        if (chninfo.ch_name(j) eq 'IRWVP') then tmp = goes9info(3)
    endif else begin
        if (chninfo.ch_name(j) eq 'VSCHN') then tmp = gmsinfo(0,sn-1)
        if (chninfo.ch_name(j) eq 'IRWIN') then tmp = gmsinfo(1,sn-1)
    endelse
    SATinf(i++) = tmp
endfor

;---------------------------
; Navigation information - NEW B1U NAV ARRAY
;IMGDAT
NAVinf(0) = IMGstr.sline
NAVinf(1) = IMGstr.spix
NAVinf(2) = IMGstr.cen_lin * 10
NAVinf(3) = IMGstr.cen_pix * 10
NAVinf(4) = IMGstr.linres
NAVinf(5) = IMGstr.pixres
NAVinf(6) = IMGstr.linang*1.e9
NAVinf(7) = IMGstr.pixang*1.e9
NAVinf(8) = spper*1.e6            ;Seconds per revolution*10^6
;KEPSAT#1
NAVinf(9) = 16                  ;Kepler from fit to test grid
NAVinf(10) = ORBstr.ep_date
NAVinf(11) = ORBstr.ep_time
NAVinf(12) = ORBstr.ep_msec
NAVinf(13) = ORBstr.semima *1.e2
NAVinf(14) = ORBstr.incl   *1.e3
NAVinf(15) = ORBstr.eccen  *1.e6
NAVinf(16) = ORBstr.meana  *1.e3
NAVinf(17) = ORBstr.rgtasc *1.e3
NAVinf(18) = ORBstr.perigee*1.e3
;KEPSAT#2
;convert MJD to time/date/msec
caldat,header.epochtime1+2400000.5d0,month,day,year,hour,minute,second
yymmdd = (year-1900)*10000l + month*100l + day
hhmmss = hour*10000l + minute*100l + floor(second)
;edate:YYJJJ,etime:HHMMSS,msec = 0.001 per millisec
edate  = yymmdd_2_yyjjj(yymmdd)
etime  = hhmmss
msec   = second mod 1.
NAVinf(19) = 10                 ;Kepler from file header
NAVinf(20) = edate
NAVinf(21) = etime
NAVinf(22) = msec*1.e3
NAVinf(23) = header.semima *1.e2
NAVinf(24) = header.incl   *1.e3
NAVinf(25) = header.eccen  *1.e6
NAVinf(26) = header.manom  *1.e3
NAVinf(27) = header.rasc   *1.e3
NAVinf(28) = header.perigee*1.e3
;KEPSAT#3
NAVinf(29:38) = MISSING
if (iupdate) then begin
    NAVinf(29) = 17             ;Kepler from corrected JMA
    NAVinf(30) = newdate
    NAVinf(31) = newtime
    NAVinf(32) = newmsec
    NAVinf(33) = ele(1) *1.e2
    NAVinf(34) = ele(3) *1.e3
    NAVinf(35) = ele(2) *1.e6
    NAVinf(36) = ele(6) *1.e3
    NAVinf(37) = ele(4) *1.e3
    NAVinf(38) = ele(5) *1.e3
endif
;ATTDAT
NAVinf(39:41) = MISSING

;ATTFIT
NAVinf(42) = ATTstr.roll  * 1.e6
NAVinf(43) = ATTstr.pitch * 1.e6
NAVinf(44) = ATTstr.yaw   * 1.e6
NAVinf(45) = TIMstr.spra1 * 1.e6
NAVinf(46) = TIMstr.spdc1 * 1.e6
NAVinf(47) = TIMstr.spra2 * 1.e6
NAVinf(48) = TIMstr.spdc2 * 1.e6

;RECINFO
NAVinf(49:51) = MISSING

;FITINFO
NAVinf(52:56) = [fitcost*10l, $
                 reverse(fitbias)*1000l, $
                 reverse(fitstdev)*1000l]

;POSDAT
NAVinf(57:129) = MISSING

;LOCDAT
NAVinf(130:142) = MISSING
NAVinf(131) = 1
NAVinf(132) = glat(0)*1e6
NAVinf(133) = glon(0)*1e6
NAVinf(134) = grad(0)*1e3
NAVinf(137) = ny-1
NAVinf(138) = glat(1)*1e6
NAVinf(139) = glon(1)*1e6
NAVinf(140) = grad(1)*1e3


;MISC
NAVinf(143) = 0                 ;GOES-NEXT Flag=0
NAVinf(144) = MISSING
NAVinf(145) = MISSING
NAVinf(146) = MISSING
NAVinf(147) = IMGstr.frdate
NAVinf(148) = IMGstr.frtime * 1000
NAVinf(149) = IMGstr.nlin
NAVinf(150) = IMGstr.npix
NAVinf(151:199) = MISSING

;---------------------------------------------------------
; TEST GRID
if (f(3) eq 'GOES9' or f(3) eq 'GOE-9' or ascii) then begin
    latstart= 50.
    lonstart = 105.
endif else begin
    latstart= 50.
    lonstart = 90.
endelse

dlat = -10.
dlon = 10.
TGRinf = lonarr(6+2*11*11)
i=0
TGRinf(i++) = 11                ;nrows
TGRinf(i++) = 11                ;ncols
TGRinf(i++) = latstart*10
TGRinf(i++) = lonstart*10
TGRinf(i++) = dlat*10
TGRinf(i++) = dlon*10
for k=0,10 do begin
    for j=0,10 do begin
        TGRinf(i++) = TGRstr.line(j,k)
        TGRinf(i++) = TGRstr.pixl(j,k)
    endfor
endfor

; Line prefix information
;PREinf = intarr(nchan,20,areadir.nline)
i=0
yymmdd = ((linhdr.time(0) mod 100)*10000l + $
          linhdr.time(1) * 100l + $
          linhdr.time(2))
jday = (yymmdd_2_yyjjj(yymmdd)) mod 1000l

PREinf(*,i++,*) = linhdr.seq    ;relative scanline
PREinf(*,i++,*) = linhdr.SCNNUM ;absolute scan line
PREinf(*,i++,*) = MISSING
PREinf(*,i++,*) = linhdr.time(0) ;year
PREinf(*,i++,*) = jday          ;doy
PREinf(*,i++,*) = (linhdr.time(3)*100+ $
                   linhdr.time(4)) ;HHMM
PREinf(*,i++,*) = linhdr.time(5) ;Sec
PREinf(*,i++,*) = linhdr.time(6) ;MSec
PREinf(*,i++,*) = linhdr.eedge ;eastern edge of earth
PREinf(*,i++,*) = linhdr.wedge ;western edge of earth
PREinf(*,i++,*) = linhdr.kind   ;detector ID
PREinf(*,i++,*) = MISSING       ;validity code
PREinf(*,i++,*) = total(images,2) mod 256 ;checksum
PREinf(*,i++,*) = linhdr.chksum
PREinf(*,i++,*) = linhdr.res
PREinf(*,i++,*) = linhdr.space

b1u = {rev:REVinf,$
       img:IMGinf,$
       sat:SATinf,$
       nav:NAVinf,$
       tgr:TGRinf,$
       pre:PREinf,$
       image:images}      

;
; END B1U array definitions
;---------------------------------------------------
goto,endd
problemread:
images = -1
print,'Problem Reading file'
endd:
end

;***
