;****I* idl/idl.b1u_dn2cal
; NAME
; b1u_dn2cal
; 
; PURPOSE
; Provide calibrated imagery of B1U files.
; 
; USAGE
; INPUT
; OUTPUT
; RESULT
; EXAMPLE
; USES
; * lr.pro
; * mean.pro
; * stdev.pro
; 
; USED BY
; * b1u_2_grisat.pro
; 
; NOTES
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; Tue Jun 26 09:51:47 EDT 2007
; 
; HISTORY
; SOURCE

function b1u_dn2cal,channels,b1u, $
  image=image, $
  qc=qc, $
  ichan=ichan, $
  debug=debug, $
  scale=scale, $
  offset=offset, $
  frac=frac

off = -999
scale = -999

channel=strtrim(channels,2)
nc = n_elements(channel)

iir=intarr(nc)
for i=0,nc-1 do begin
    j=where(strtrim(b1u.sat,2) eq channel(i))-2
    iir(i) = j(0)
endfor
ichan = iir
images = (b1u.image(iir,*,*))

btemp = images * 0.0

nch = b1u.img(5)
nb = 256
if (b1u.img(14) eq 1024) then nb = 1024

cver     = b1u.cal(0)
caltable = reform(b1u.cal(1:2*nch*nb),2,nch,nb)/1000.

b1ufile=strtrim(b1u.rev(0),2)
f=strsplit(b1ufile,'.',/extr)
sat=f(3)
spc = f(8)
satellite = strmid(sat,0,3)

;correct the FY visible calibration
if (cver eq (-8) and satellite eq 'FY2') then begin
    ivs = where(strtrim(channel,2) eq 'VSCHN',nvs)
    if (nvs gt 0 and nb eq 1024) then begin
        if (total(Abs(caltable(1,ivs(0),64:1023))) lt 1.) then begin
            caltable(1,ivs(0),*) = congrid(reform(caltable(1,ivs(0),0:63)),1024,/interp)
        endif
    endif
endif

;skip this stuff if an IR channel isn't requested
if (nc eq 1 and channel(0) eq 'VSCHN') then goto,calall


off = nb-1                      ;255
scale = -1.

b1ufile=strtrim(b1u.rev(0),2)
f=strsplit(b1ufile,'.',/extr)
sat=f(3)
spc = f(8)
satellite = strmid(sat,0,3)

if keyword_set(debug) then print,'CVER=',cver,' SAT=',f(3)

if (cver gt -20 and cver lt -1) then begin
    off = 0.
    scale = 1.
    goto,calall    
endif

;determine orientation of IRWIN
ij = where(strtrim(b1u.sat,2) eq 'IRWIN')-2

if (ij lt 0) then begin
    scale=-999
    offset=-999
    frac = replicate(-1.,3)
    return,-100
endif

image = reform(b1u.image(ij,*,*))



filename=b1u.rev(0)
f=strsplit(filename,'.',/extr)
;=============================================================
; The following no longer applies for MSG as MSG now has ISCCP
; calibration, so sometimes needs to be inverted.
;
;if (f(3) eq 'MSG-1' or f(3) eq 'MSG-2') then begin
;    off = 0.
;    scale = 1.
;    goto,calall
;endif
;=============================================================
;calculate the fraction of bad points
s=size(image)
fracbad = total(image gt 1000) / float(s(1)*s(2))
;fracbad was added b/c some 0900 files are NHemiX during the eclipse
;season so have the same problem as the 800 and 1000 files)
if (f(3) eq 'GOE-11' and $
    (((fix(f(4)) ge 2006) and (fix(f(7)) eq 1000 or $
                              fix(f(7)) eq 800)) or $
     fracbad gt 0.3)) then begin
    off = 0.
    scale = 1.
    off = nb-1
    scale = -1.
    goto,calall
endif
if (f(3) eq 'GOE-12' and f(8) eq 'NOA' and fracbad gt 0.3) then begin
    off = 0.
    scale = 1.
    off = nb-1
    scale = -1.
    goto,calall
endif


if (strmid(f(3),0,3) eq 'MTS') then begin
    off = 0.
    scale = 1.
    goto,calall
endif
;this is included for the one month in 1982 which has a B3 calibration
;table
if (f(3) eq 'GMS-2' and fix(f(4)) eq 1982 and fix(f(5)) eq 12) then begin
    off = nb-1
    scale=-1.
    goto,calall
endif
if (f(3) eq 'GMS-2' and fix(f(4)) lt 1983) then begin
    off = 0.
    scale=1.
    goto,calall
endif
if (f(3) eq 'GMS-1' and fix(f(4)) lt 1983) then begin
    off = 0.
    scale=1.
    goto,calall
endif
if (strmid(f(3),0,3) eq 'MET') then begin
    off = nb-1                  ;255
    scale = -1.
    goto,calall
endif



if (satellite eq 'GOE') then begin ; or satellite eq 'GMS') then begin
    ;for GOES satellites ... the space values are never zeroed out,
    ;so if we can determine what space is, then we know the orientation
    ;first try to get center
    irimg=image
    s=size(irimg)
    nx=s(1)
    ny=s(2)

    ;THE FOLLOWING CODE WAS DIRECTLY COPIED FROM B1U_FINALCENTER
    ;IF ANY CHANGES ARE MADE, IT SHOULD ALSO BE MADE IN THAT
    ;ROUTINE AS WELL
    ;
    ;------------ BEGIN SNIPETTE
    nny = (ny-41)
    
    x1=10
startover:
    mn = mean( irimg(x1:x1+20,10:10+nny))
    sdd= stdev(irimg(x1:x1+20,10:10+nny))
;if the sector isn't smooth, then
    if (sdd gt 10) then begin
        sp=fltarr(nny)
        sd=fltarr(nny)
        scan=fltarr(nny)
        sd(*) = 100.
                                ;get the mean & stdev of many blocks along the scan edge
        for i=0,nny-1 do begin
                                ;this check makes sure that the scanline is ok, only
                                ;storing the mean if the Earth is observed in the scan
            if (stdev(irimg(0:nx-1,10+i+2)) gt 0.) then begin
                sp(i) = mean( irimg(x1:x1+10,10+i:10+i+3))
                sd(i) = stdev(irimg(x1:x1+10,10+i:10+i+3))
                ;The following appears wrong. It sums over a column
                ;instead of along the scan line
                ;scan(i) = total(irimg(30:40,*))
                ;This sums the scan lines
                scan(i) = total(irimg(*,10+i:10+i+3))
            endif
        endfor
        
                                ;find median of blocks with stdev < 10
        
        s=sort(sd)
        ok = where(sd(s) lt 10 and scan(s) gt 0.,nok)
        
        if (nok gt 0) then begin
                                ;first, get the lowest 20% of the standard deviations
            sd_ = sd(s(ok(0:nok*0.2)))
            sp_ = sp(s(ok(0:nok*0.2)))
            nnok = nok*0.2
                                ;then of that, grab the darkest points
            ss = sort(sp_)
            mn = sp_(ss(nnok*0.15)) ; <-- 0.2 * 0.15 = 3rd percentile            
        endif else begin
            if (x1 lt 35) then begin
                x1=x1+10
                goto,startover
            endif else begin
                if (stdev(sp) lt 1) then begin
                    mn = mean(sp)
                endif else begin
               ;if all points are bad then ...???
;                    window,xs=900,ys=900
;                    tvscl,irimg
;                    stop
                endelse
            endelse
        endelse
    endif
    space = mn

    if keyword_set(debug) then print,'SPACE=',space
    if (space gt 200) then begin
        off = 0
        scale = 1
        goto,calall
    endif else begin
        off = nb-1              ;255
        scale = -1.
        goto,calall
    endelse 
endif

temp = caltable(image*0+1,image*0+ij(0),off + scale * image)
mx = max(caltable(1,ij,*))
check = where(temp gt 200 and temp lt (mx-1),ncheck)
if (ncheck lt 50000 or $
    (total(image eq 0 or image eq (nb-1))/n_elements(image)) gt 0.85) then begin
    scale=-998
    offset=-998
    frac = replicate(-1.,3)
    return,-100
endif
md = median(temp(check))
mom=moment(temp(check))
mn = mom(0)
sk = mom(2)
if (md gt mn and sk lt 0) then begin
    ;return if the stats check out OK
    if keyword_set(debug) then     print,'Median=',md,' > Mean=',mn,' and skew=',sk,' < 0.'
    goto,calall
endif else if (md lt mn and sk gt 0) then begin
    ;reverse the calibration table if the stats are opposite
    off = 0
    scale = 1
    temp = caltable(image*0+1,image*0+ij(0),off + scale * image)
    check = where(temp gt 200 and temp lt (mx-1),ncheck)
    if (ncheck eq 0) then begin
        ;if there are no valid values ... then I guess return no values??
        ;primarily occurs for ISCCP.B1U.a.GMS-3.1988.03.23.1543.JMA.gz
        Message,'Bad calibration table. Skipping file',/info
        btemp=0
        frac = replicate(-1.,3)
        goto,skipfile
    endif
    md = median(temp(check))
    mom=moment(temp(check))
    mn = mom(0)
    sk = mom(2)
    if (md gt mn and sk lt 0) then begin
        ;return if the stats are OK
        if keyword_set(debug) then print,'Median=',md,' > Mean=',mn,' and skew=',sk,' < 0.'
        goto,calall
    endif else begin
        ;ERROR out if stats are bad
        print,'mean',mn
        print,'median',md
        print,'moment',mom
stop
off=nb-1                        ;255
        scal=-1
        goto,calall
        stop
    endelse
endif else begin
    ;ERROR out if the stats are bad
    print,'mean',mn
    print,'median',md
    print,'moment',mom
stop
    off=nb-1                        ;255
    scal=-1
    goto,calall
    stop
endelse
calall:

for i=0,nc-1 do begin
    if (iir(i) ge 0) then begin
        img = reform(images(i,*,*))
        if (strtrim(channel(i),2) eq 'VSCHN') then begin
            if ((sat eq 'GOE-7' and spc eq 'AES') or $
                (sat eq 'GOE-7' and spc eq 'NOA')) then begin
;                stop
                temp = caltable(img*0+1,img*0+iir(i),img*2)
            endif else begin
                temp = caltable(img*0+1,img*0+iir(i),img)
            endelse
        endif else begin
            temp = caltable(img*0+1,img*0+iir(i),off+scale*img)
        endelse
                                ;calibrate, but set missing values to zero
        btemp(i,*,*) = temp * (img gt 0 and img lt (nb-1))
                                ;if QC is available, check it out.
        if keyword_set(qc) then  btemp(i,*,*) = temp * (qc(iir(i),*,*) eq 0)
    endif else begin
        btemp(i,*,*) = -1.
    endelse      
endfor
image=images
if keyword_set(debug) then print,'OFF:',off,' SCALE:',scale
offset=off

if arg_present(frac) then begin
    ivs = where(strtrim(channel,2) eq 'VSCHN',nvs)
    if (nvs gt 0) then begin
        ch = iir(ivs(0))
        frac = mean(caltable(0,ch(0),100:255)/caltable(1,ch(0),100:255))
        rad=reform(caltable(0,ch(0),*))
        ref=reform(caltable(1,ch(0),*))
        ok = where(rad gt 0.,nok)
        if (nok gt 10) then begin
            dn = findgen(256)
            strad = lr(dn(ok),rad(ok))
            stref = lr(dn(ok),ref(ok))
            stdn = lr(ref(ok),dn(ok))
            frac = [strad(2) / stref(2), stdn(2:3)]
        endif else begin
            frac = replicate(-1.,3)
        endelse
    endif else begin
        frac = replicate(-1.,3)
    endelse
endif


skipfile:
;if keyword_set(debug) then stop
return,btemp
end
;***
