pro set_up_old_structures,b1u,IMGstr,ATTstr,ORBstr,TIMstr,pt

; set up the structures
@geo_struct.pro

orbit.ep_date = b1u.nav(10)
orbit.ep_time = b1u.nav(11)
orbit.ep_msec = b1u.nav(12)/1d3
orbit.semima =  b1u.nav(13)/1d2
orbit.incl =    b1u.nav(14)/1d3
orbit.eccen =   b1u.nav(15)/1d6
orbit.meana =   b1u.nav(16)/1d3
orbit.rgtasc =  b1u.nav(17)/1d3
orbit.perigee = b1u.nav(18)/1d3
orbit.sper =    b1u.nav(8)/1d6
ORBstr = orbit

imginfo.frdate = b1u.img(0)
imginfo.frtime = ftime(b1u.img(1))*3600.
imginfo.cen_lin = b1u.nav(2)/10.
imginfo.cen_pix = b1u.nav(3)/10.
imginfo.linres =  b1u.nav(4)
imginfo.pixres =  b1u.nav(5)
imginfo.nlin =    b1u.img(3);header.nlines
imginfo.npix =    b1u.img(4) ;header.npixels
imginfo.sline =   b1u.nav(0)
imginfo.spix =    b1u.nav(1)
imginfo.linang =  b1u.nav(6)/1d9
imginfo.pixang =  b1u.nav(7)/1d9
IMGstr = imginfo

gt1  = 0.
gt2  = 13.d0*3600.d0
gra1 = 0.
gra2 = 0.

sattim.d = gt2-gt1
sattim.gt1 = gt1
sattim.gt2 = gt2
sattim.gra1 = gra1
sattim.gra2 = gra2
sattim.spra1 = b1u.nav(45)/1d6           ;-999.d0
sattim.spra2 = b1u.nav(47)/1d6            ;-999.d0
sattim.spdc1 = b1u.nav(46)/1d6            ;-999.d0
sattim.spdc2 = b1u.nav(48)/1d6            ;-999.d0
sattim.cheb_x = -999.d0 ;ch_x
sattim.cheb_y = -999.d0 ;ch_y
sattim.cheb_z = -999.d0 ;ch_z
sattim.cheb_a = -999.d0
sattim.cheb_b = -999.d0
sattim.cheb_c = -999.d0
TIMstr = sattim

att.roll = b1u.nav(42)/1d6
att.pitch = b1u.nav(43)/1d6
att.yaw = b1u.nav(44)/1d6
att.rma = 0.d0
att.pma = 0.d0
ATTstr = att

;get pt from b1u.tgr
pt = intarr(2,b1u.tgr(0),b1u.tgr(1))
t=0
for i=0,b1u.tgr(0)-1 do begin
    for j=0,b1u.tgr(1)-1 do begin
        pt(*,j,i) = [b1u.tgr(6+t),b1u.tgr(7+t)]/100.
        t=t+2
    endfor
endfor

end


;****I* idl/idl.b1u_get_center
; NAME
; b1u_get_center
;
; PURPOSE
; This routine is used by idl.b1u_finalcenter to determine the 
; correct center of the image, which is used to make navigation
; more accurate.
;
; AUTHOR
; Ken Knapp - Feb. 2006
; 
; NOTES
; - The routine has been tested and the latest version in CVS (1.3)
;   finds the center of the image and has been tested on all B1U
;   files. The next step is to develop the navigation algorithm.
; - This updated version is different from previous in that 
;   a) binom is a cubic equation
;   b) it uses idl.b1u_ll2ij and idl.b1u_ij2ll to get a correction
;      for the central scan number
;
; USES
; * b1u_ij2ll.pro
; * b1u_ll2ij.pro
; * ftime.pro
; * mean.pro
; * stdev.pro
; 
; USED BY
; * b1_2_b1u.pro
;
; 
; SOURCE
pro b1u_get_center,irimg,nav,center, $
                   debug=debug, $
                   b1u=b1u, $
                   meta=meta, $
                   file=file

if (b1u.img(14) lt 257) then begin
    divider=200
    dntop = 254
endif else if (b1u.img(14) eq 1024) then begin
    divider=800
    dntop = 1022
endif else begin
    print,'Unknown number of bins:',b1u.img(14)
    stop
endelse 


binom = fltarr(4)
binom(*) = -1.
midscan=-1
x = 0
pix_min = -1
pix_max = -1
corr = 0
;loadct,0,/sil
s=size(irimg)
nx=s(1)
ny=s(2)
;if keyword_set(debug) then print,nx,ny
;initial attempt to determine whether space is high values or low
;values
;this doesn't work when a significant number of points are missing (0)
;and the space values are 254. It sets space to the low end. Will try
;a new technique by looking at the upper corner, which is nearly?
;always space.
;hifrac = total(irimg gt 250) / float(nx)/float(ny)
;lofrac = total(irimg lt  50) / float(nx)/float(ny)
;if (hifrac gt lofrac) then $
;  earthmask = irimg lt 250 else $
;  earthmask = irimg gt 50


;Using a fixed location ran into too many problems picking up bad
;lines/pixels... am trying to move around
; also, will consentrate on one scanline to avoid potentially bad scanlines
;space = mean(irimg(10:15,10:15))

;This doesn't work when the first portion of the image is missing, so
;it sees space as zero.
;for dj=5,50 do begin
;    mn = mean(irimg(10:20,dj:dj+2))
;    sd = stdev(irimg(10:20,dj:dj+2))
;    if (sd lt 10) then break
;endfor
;space=mn

;get an estimate of space from a long sample

nny = (ny-41)

x1=10
startover:
mn = mean( irimg(x1:x1+20,10:10+nny))
sdd= stdev(irimg(x1:x1+20,10:10+nny))
;print,x1,mn,sdd
;if the sector isn't smooth, then
if (sdd gt 10) then begin
    sp=fltarr(nny)
    sd=fltarr(nny)
    scan=fltarr(nny)
    sd(*) = 100.
    ;get the mean & stdev of many blocks along the scan edge
    for i=0,nny-1 do begin
        ;this check makes sure that the scanline is ok, only
        ;storing the mean if the Earth is observed in the scan
        if (stdev(irimg(0:nx-1,10+i+2)) gt 0.) then begin
            sp(i) = mean( irimg(x1:x1+10,10+i:10+i+3))
            sd(i) = stdev(irimg(x1:x1+10,10+i:10+i+3))
            scan(i) = total(irimg(30:40,*))
        endif
    endfor

    ;find median of blocks with stdev < 10

;This worked until a 1995 file was mostly in Southern Hemisphere.
    ;ok=where(sd lt 10,nok)
;so I am trying a lower value ... not sure if it will always work
;... since some sensors are more noisy
;    ok=where(sd lt 5,nok)  <----- even a limit of 5 didn't work.
;
;NEW Attempt: Try usingthe 1st percentile of the std dev to grab as space
    s=sort(sd)
    ok = where(sd(s) lt 10 and scan(s) gt 0.,nok)
    if (nok gt 0) then begin
        ;first, get the lowest 20% of the standard deviations
        sd_ = sd(s(ok(0:nok*0.2)))
        sp_ = sp(s(ok(0:nok*0.2)))
        nnok = nok*0.2
        ;then of that, grab the darkest points
        ss = sort(sp_)
        mn = sp_(ss(nnok*0.15)) ; <-- 0.2 * 0.15 = 3rd percentile

        ; OLD ATTEMPTS TO DETRMINE SPACE
        ;mn = sp(s(ok(nok*0.05)))
        ;mn = median(sp(ok))
    endif else begin
        if (x1 lt 35) then begin
            x1=x1+10
            goto,startover
        endif else begin
            if (stdev(sp) lt 1) then begin
                mn = mean(sp)
            endif else begin
               ;if all points are bad then ...???
                window,xs=900,ys=900
                tvscl,irimg
                stop
            endelse
        endelse
    endelse
endif
space = mn

if (b1u.img(14) lt 257) then begin
    if (space gt divider) then earthmask = irimg lt (space-4) else $
                               earthmask = irimg gt (space+10)
endif else if (b1u.img(14) eq 1024) then begin
    if (space gt divider) then earthmask = irimg lt (space-24) else $
                               earthmask = irimg gt (space+24)
endif else begin

endelse

em = where( earthmask eq 1 and $
            irimg gt 2 and $
            irimg lt dntop,nem)

if (nem gt 0) then begin
    mom=moment(irimg(em))
    md = median(irimg(em))
    mom(1) = sqrt(mom(1))
    ;meta={moment:mom,median:md,space:space}
    meta=[mom,md,space]
endif else begin
    meta = replicate(-1,10)
endelse

center = fltarr(ny)
limb=intarr(2,ny)
for j=0,ny-1 do begin
    ;can only find center if the image center is a valid IR obs
    if (earthmask(nx/2,j) eq 1) then begin
        i=nx/2
        ;the initial test of just one pixel was pretty sensitive to
        ;salt-n-pepper noise, so adding test of next pixel too
        for i=nx/2,nx-2 do begin
            if (earthmask(i,j) eq 0 and earthmask(i+1,j) eq 0) then break
        endfor
        i2 = i-1
        i=nx/2
        for i=nx/2,2,-1 do begin
            if (earthmask(i,j) eq 0 and earthmask(i-1,j) eq 0) then break
        endfor
        i1 = i+1
        if (i1 gt 3 and i2 lt (nx-4)) then begin
            limb(0,j) = i1
            limb(1,j) = i2
            center(j) = (i2+i1)/2.
        endif
    endif
endfor

;variable center is in image coordinates
;convert to offset from satellite coordinates

if (b1u.img(13)) then begin
;    pixres = 
;    spix =    
    message,'Not set up to process GOES-Next (GVAR) files'
endif else begin
    pixres = b1u.nav(5)
    spix =   b1u.nav(1)
    centerorig = b1u.nav(3)/10.
    centerimg = (centerorig - spix) / float(pixres)
endelse

center = center * pixres + spix - centerorig


y=indgen(ny)
if keyword_set(debug) then begin
    window,xs=1000,ys=1000
    tv,hist_equal(congrid(irimg,1000,1000))>3,/ord
    color,'violet',0
    plot,center+centerimg,y,xmarg=[0,0],ymarg=[0,0], $
         xs=1,ys=1,yrange=[ny-1,0],xrange=[0,nx-1], $
         /noerase,col=0
    oplot,limb(0,*),y,col=0
    oplot,limb(1,*),y,col=0
    loadct,0,/sil
    window,2
    plot,center,min=-1000,/yno,psym=3
endif



nz=where(center gt (-500),nnz)

;fit the new center to a binomial
x=findgen(ny)-ny/2.
if (nnz gt 100) then begin
    binom = poly_fit(x(nz),center(nz),3,sigma=sigma,yerr=yerr,yfit=yfit)
    if keyword_set(debug) then begin
        color,'blue',1
        color,'green',2
        oplot,nz,yfit,col=1
    endif
    c=0
again:
    yfit_ = binom(0)+x*binom(1)+x*x*binom(2)+x*x*x*binom(3)
    ok = where(abs(yfit_-center) lt (4.*yerr),nok)
    if (nok gt 10) then begin
        binom = poly_fit(x(ok),center(ok),3,sigma=sigma,yfit=yfit,yerr=yerr)
        if keyword_set(debug) then oplot,x(ok)+ny/2.,yfit(ok),col=1
        yerr=yerr>0.2
    endif
    c=c+1
    if (c lt 5) then goto,again
    pix_min = min(center(ok))
    pix_max = max(center(ok))
    cen_line = binom

    if keyword_set(debug) then oplot,x(ok)+ny/2.,yfit,col=2
    if keyword_set(debug) then begin
        print,'Final'
        print,reform(binom)
        print,reform(binom)/sigma
        print,yerr
    endif
endif else begin
    goto,returning
endelse

;use the limb information to get an estimate on the center scan number
width=reform(limb(1,*)-limb(0,*))
width=0.*width
width(ok) = limb(1,ok) - limb(0,ok)
wmx = max(width(ok),mxloc)
mid=ok(mxloc)

; If the maximum width is less than 100 pixels, then something is
; wrong and this file should not be processed.
; It may be
; 1. a bad image or
; 2. these tests need to be altered for a new style of image.

if (wmx lt 100) then begin
    print,'ROUTINE: b1u_get_center.pro'
    print,'Maximum width of Earth''s disc is too small:'
    print,'Max Width=',wmx
    print,'total(width gt 0):',total(width gt 0)
    goto,returning
endif


;for most files, ny works, but when the bottom of the scan is missing
;and filled in with zeros, then we need to use the last valid scan
;frac = mid / float(ny)

;find last possible valid scan
for j=ny-1,0,-1 do begin
    if (total(irimg(*,j)) gt 0) then break
endfor
lastv = j
frac = mid / float(lastv)


iplots=0


if iplots then window,4
di=1
if (iplots) then di=3
;search for center point if max isn't near beginning or end of image
if (frac gt 0.05 and frac lt 0.95) then begin
    sch = ((ny-mid)>mid)
    score = lonarr(2*sch+1)
    score2 = lonarr(2*sch+1)
    score(*) = nx
    y1=width
    revw=reverse(width)
;PROBLEM: When there are two portions (upper/lower) of image with
;center values, then score sometimes finds a bad center.
; e.g., /home/raid/kknapp/data/isccp/b1/1983/ISCCP.B1.0.GOE-5.1983.12.21.2100.UWS
; solution: increase the score by (np-nok). If nok is large, then this
; offset is less, if nok is small (e.g., 1) then the offset is large
; HOWEVER, this ruins things in a really noisy image, so only turn it
; on if width is missing near the center of the image
    imiss=0
    for i=-200,200 do begin
        if (total(center(ny/2-150:ny/2+150)) eq 0.) then begin
            ;if this happens, then the algorithm can't find the image
            ;center in the middle of the scan, so we need to adjust
            ;the score by (np-nok)
            imiss=1
            break
        endif
    endfor

;PROBLEM: When a portion of the scan is missing, and less than half
;the scan has a known width, then you can't calculate a new center.
; e.g., /home/raid/kknapp/data/isccp/b1/1988/ISCCP.B1.3.GOE-6.1988.08.30.1514.CSU
; Before this change, a new center as calculated ... so we need to not
; recalculate a center when less than half the width is known.

;is less than half the width known?
ind=indgen(ny-1)
dw = width(ind+1)-width(ind)
sdw = smooth(dw,51)
widthok = total(sdw gt   0.1  and sdw lt   4.5 ) gt 10 and $
          total(sdw lt (-0.1) and sdw gt (-4.5)) gt 10
if (widthok eq 0) then goto,returning

    np = total(y1 gt 10)
    for is=-sch,-1,di do begin
        y2 = shift(revw,is)
        y2(ny-1+is:ny-1) = 0
        use = where(y1 gt 10 and y2 gt 10,nok)
        if (nok gt 10) then begin
            score(is+sch) = total(abs(y1(use)-y2(use)))/float(nok) 
            score2(is+sch) = (np-nok);*imiss
        endif
        if (iplots) then begin
            plot,width
            oplot,y2
            oplot,score,psym=1
            ;if (score(is+sch) lt 70) then wait,1
            wait,0.1
        endif
        ;if (is eq (-32)) then stop
    endfor
    for is=0,sch,di do begin
        y2 = shift(revw,is)
        y2(0:is) = 0
        use = where(y1 gt 10 and y2 gt 10,nok)
        if (nok gt 10) then begin
            score(is+sch) = total(abs(y1(use)-y2(use)))/float(nok)
            score2(is+sch) = (np-nok);*imiss
        endif
        if (iplots) then begin
            plot,width
            oplot,y2
            oplot,score,psym=1
            ;if (score(is+sch) lt 70) then wait,1
            wait,0.1
        endif
    endfor
    score1=score+score2*imiss*0.1
    mn=min(score1,minloc)
    del = (score1(minloc+1) - score1(minloc)) > (score1(minloc-1) - score1(minloc))
    if (del lt 100) then begin
        midscan=ny/2.+(minloc-sch)/2.
        ;Correct the center pixel based on the midscan value
        ;this assumes the mid scan transects the subsatellite point
        ;get subsatellite point lat/lon
        b1u_ij2ll,nx/2.,ny/2.,lat,lon,nav=nav,glat=glat,glon=glon
        b1u_ll2ij,glat,glon,isub,jsub,nav=nav
        ;adjust center line
        if (n_elements(nav) gt 4) then begin
            corr = (midscan - jsub)*10.*nav(4)
        endif else begin
            corr = (midscan - jsub)*10.*nav.b1uarea.line_res
        endelse
        if keyword_set(debug) then begin 
            print,'Jsub',jsub
            print,'center',midscan,corr
        endif
        ;b1u.nav(2) = b1u.nav(2) + round(center.cscan - jsub)*10.*b1u.nav(4)

        if keyword_set(debug) then begin 
            wset,0
            color,'magenta',5
            plot,center+centerimg,y,xmarg=[0,0],ymarg=[0,0], $
                 xs=1,ys=1,yrange=[ny-1,0],xrange=[0,nx-1], $
                 /noerase,col=0,/nodata,xticklen=0.,yticklen=0.
            oplot,center+centerimg,y,col=5
            oplot,[0,nx],[midscan,midscan],col=2
            oplot,yfit_+centerimg,x+ny/2.,col=2
        endif
    endif
    if keyword_set(debug) then begin 
        oplot,[20,20],[10,310],col=1
        oplot,[10,10],[10,310],col=1
        oplot,[10,20],[10,10],col=1
        oplot,[10,20],[310,310],col=1

    endif
endif

returning:
center = {cline_eq:binom, $
          cline: x, $
          cline_range:[pix_min,pix_max], $
          cen_lin_corr:corr}

return

;need to rerun the jma_bestfit.pro if the file is a JMA file
;need to rerun the cma_bestfit.pro if the file is from FY-2C

set_up_old_structures,b1u,IMGstr,ATTstr,ORBstr,TIMstr,pt

;call JMA bestfit
;first, try to reproduce the currently retieved value.
;       this will verify the structures created by
;       set_up_old_structures are correct. 
; Next, pass the new center value and see if the bias and
;       RMS are reduced. If not, figure out why not!!!!!!

closewin

latj = 15.
lonj = 120.
b1u_ll2ij,latj,lonj,igoodc,jgoodc,nav=b1u.nav,center=center ;,/debug
b1u_ll2ij,latj,lonj,igood,jgood,nav=b1u.nav ;,/debug

;print
print,'Answer',igood(0),jgood(0)

set_up_old_structures,b1u,IMGstr,ATTstr,ORBstr,TIMstr,pt

b1u_ll2ij,latj,lonj,itest,jtest, $
          b1tim=TIMstr, $
          b1orb=ORBstr, $
          b1att=ATTstr, $
          b1img=IMGstr,center=center          ;,/debug
print,'Test  ',itest(0),jtest(0)

if (abs(itest-igood) gt 1 or $
    abs(jtest-jgood) gt 1) then stop

print,'RENAVIGATE ------------------------------------'

jma_bestfit,file=file, $
            imgstr=imgstr,attstr=attstr,orbstr=orbstr, $
            timstr=timstr,pt=pt,/debug,bias=bias, $
            fstdev=fstdev,cost=cost,/rerun,center=center
print,'AFTER=============================='
print,'Bias=',bias
print,'Std =',fstdev
print,'Cost=',cost

stop

end


;***
