function gatt,k0,rec,wa,te,iout=iout
;+
; NAME:
;    GATT
;
; CATEGORY:
;    ISCCP NAVIGATION
;
; MODIFICATION HISTORY:
;    Created 1/23/2004 Ken Knapp NOAA/NCDC
;                      based on code developed in FORTRAN
;                      for GVAR navigation
;
;-
if (keyword_set(iout)) then iout = 1

;constant component
k = k0-1
att = rec(k+2)
;compute the exponential terms
if (te ge 0.0d0 and rec(k+1) gt 0) then begin
    att = att + rec(k)*exp(-te/rec(k+1))
endif

;extract # of sinusoids
ir = rec(k+3)
i = fix(ir)

;calculate sinusoids

for l=1,i do begin
    att = att + rec(k+2*l+2)*cos(wa*l+rec(K+2*L+3))
endfor
;pointer to the number of monomial sinusoids
k=k+34

;exact number of monomial sinusoids
ir=rec(k)
i=fix(ir)

;compute monomial sinusoids
;
; PROBLEM: This routine uses EQUIVALENCE FOR J==JR and M==MR
;
for l=1,i do begin
    ll=k+5*l
    ;order of sinusoid
    jr=rec(ll-4)
    ;order of monomial sinusoid
    mr = rec(ll-3)
    att = att + rec(ll-2) * ((wa*rec(ll))**M) * cos(j*wa+rec(ll-1))
if (keyword_set(iout)) then print,'mr,m',mr,m
endfor
return,att
end
;=================================================================
;=================================================================
function scpx,instr,pix,iout=iout
;+
; NAME:
;    SCPX
;
; PURPOSE:
;    THIS FUNCTION CONVERTS FRACTIONAL PIXEL NUMBER TO SCAN ANGLE
;    IN RADIANS.
;
; CATEGORY:
;    ISCCP NAVIGATION
;
; RESTRICTIONS:
;    require INSTCO.PRO
;
;
; MODIFICATION HISTORY:
;    Created 6/2/2004 Ken Knapp NOAA/NCDC
;                     based on code developed in FORTRAN
;                     for GVAR navigation
;
;-
if (keyword_set(iout)) then iout = 1
@instco.pro
SCPX=((PIX*1.0D0)-1.0D0)*(SCNPX(INSTR-1)*1.0D0) $
     -(SCNMAX(INSTR-1)*1.0D0)
RETURN,scpx
END
;=================================================================
;=================================================================
function evln,instr,rline,iout=iout
;+
; NAME:
;    EVLN
;
; PURPOSE:
;    THIS FUNCTION CONVERTS FRACTIONAL LINE NUMBER TO ELEVATION ANGLE
;    IN RADIANS.
;
; CATEGORY:
;    ISCCP NAVIGATION
;
; RESTRICTIONS:
;    require INSTCO.PRO
;
;
; MODIFICATION HISTORY:
;    Created 1/23/2004 Ken Knapp NOAA/NCDC
;                      based on code developed in FORTRAN
;                      for GVAR navigation
;
;-
if (keyword_set(iout)) then iout = 1

@instco.pro

IF (INSTR EQ 1) THEN begin
    EVLN=ELVMAX(INSTR-1)*1.0D0 - (RLINE-4.5d0)*(ELVLN(INSTR-1)*1.0D0)
endif else begin
    EVLN=ELVMAX(INSTR-1)*1.0D0 - (RLINE-2.5D0)*(ELVLN(INSTR-1)*1.0D0)
Endelse
RETURN,evln
END
;=================================================================
;=================================================================
pro evsc2l,instr,elev,scan,rl,rp,iout=iout
;+
; NAME:
;    EVSC2L
;
; PURPOSE:
;    THIS SUBROUTINE CONVERTS ELEVATION AND SCAN ANGLES
;    TO THE FRACTIONAL LINE AND PIXEL NUMBERS.
;
; CATEGORY:
;    ISCPP Navigation
;
; RESTRICTIONS:
;    uses ELCOMM.PRO and ELCONS.PRO
;
; MODIFICATION HISTORY:
;    Created 1/23/2004 Ken Knapp NOAA/NCDC
;                      based on code developed in FORTRAN
;                      for GVAR navigation
;
;-
if (keyword_set(iout)) then iout = 1

;instr: INSTRUMENT CODE (1-IMAGER, 2-SOUNDER)                   
@instco.pro

np = n_elements(elev)
rl = dblarr(np)
rp = dblarr(np)
rl(*) = -1.d0
rp(*) = -1.d0
good = where(elev ne 99999.d0,ngood)
if (keyword_set(iout)) then begin
    print,'EVSC2l'
    print,SCNMAX(INSTR-1),SCAN,SCNPX(INSTR-1)
endif

RL(good)=(ELVMAX(INSTR-1)-ELEV(good))/ELVLN(INSTR-1)

IF (INSTR EQ 1) THEN begin
    RL=RL+4.5D0
endif ELSE begin 
    RL=RL+2.5D0
ENDelse
;C     COMPUTE FRACTIONAL PIXEL NUMBER

RP(good) = (SCNMAX(INSTR-1)+SCAN(good))/SCNPX(INSTR-1)+1.0D0

end
;=================================================================
;=================================================================
pro lpoint,ALPHA0,ZETA0,RLAT,RLON,IERR,iout=iout
;THIS SUBROUTINE CONVERTS THE INSTRUMENT ELEVATION AND SCAN
;ANGLES TO THE RELATED GEOGRAPHIC LATITUDE AND LONGITUDE.
if (keyword_set(iout)) then iout = 1

np = n_elements(alpha0)

@elcons.pro
@elcomm.pro
@instco.pro

ierr=1
instr = 1                       ;GOES Imager is HARD-CODED HERE

; ADD THE NEW SECOND ORDER ORIGIN OFFSET CORRECTION
EWNOM = 2.5D0 * INCMAX(INSTR) * SCNINC(INSTR)
DOFF = SCNMAX(INSTR-1) - EWNOM
ALPHA = ALPHA0 - ALPHA0 * ZETA0 * DOFF
ZETA = ZETA0 + 0.5 * ALPHA0 * ALPHA0 * DOFF
;COMPUTES TRIGONOMETRIC FUNCTIONS OF THE SCAN AND ELEVATION
;ANGLES CORRECTED FOR THE ROLL AND PITCH MISALIGNMENTS
CA=COS(ALPHA)
SA=SIN(ALPHA)
CZ=COS(ZETA)
DA=ALPHA-PMA*SA*(1.0D0/CZ+TAN(ZETA))-RMA*(1.0D0-CA/CZ)
DZ=ZETA+RMA*SA
;CORRECTED SCAN ANGLE
CZ=COS(DZ)
;COMPUTES POINTING VECTOR IN INSTRUMENT COORDINATES
G = dblarr(3,np)
G(1-1,*)=    SIN(DZ)
G(2-1,*)=-CZ*SIN(DA)
G(3-1,*)= CZ*COS(DA)
;TRANSFORMS THE POINTING VECTOR TO EARTH FIXED COORDINATES
;G1 = dblarr(3,np)
;G1(1-1,*)=BT(1-1,1-1)*G(1-1,*)+BT(1-1,2-1)*G(2-1,*)+BT(1-1,3-1)*G(3-1,*)
;G1(2-1,*)=BT(2-1,1-1)*G(1-1,*)+BT(2-1,2-1)*G(2-1,*)+BT(2-1,3-1)*G(3-1,*)
;G1(3-1,*)=BT(3-1,1-1)*G(1-1,*)+BT(3-1,2-1)*G(2-1,*)+BT(3-1,3-1)*G(3-1,*)

G1 = BT # G

;COMPUTES COEFFICIENTS AND SOLVES A QUADRATIC EQUATION TO
;FIND THE INTERSECT OF THE POINTING VECTOR WITH THE EARTH
;SURFACE
Q1=G1(1-1,*)^2+G1(2-1,*)^2+AEBE2*G1(3-1,*)^2
Q2=XS(1-1)*G1(1-1,*)+XS(2-1)*G1(2-1,*)+AEBE2*XS(3-1)*G1(3-1,*)
D=Q2*Q2-Q1*Q3

;IF (ABS(D) LT 1.D-9) then D=0.0D0
low = where(ABS(D) LT 1.D-9,nlow)
if (nlow gt 0) then d(low) = 0.d0

rlat = replicate(-999d, np)
rlon = rlat
ierr = replicate(1, np) 

good = where(d gt 0,ngood)

;IF THE DISCIMINANTE OF THE EQUATION, D, IS NEGATIVE, THE
;INSTRUMENT POINTS OFF THE EARTH
;IF (D LT 0.0D0) THEN begin
;    RLAT=-999.0D0
;    RLON=-999.0D0
;ENDIF else begin
if (ngood gt 0) then begin
    D=SQRT(D(good))                                                                
    ;SLANT DISTANCE FROM THE SATELLITE TO THE EARTH POINT
    H=-(Q2(good)+D)/Q1(good)
    ;CARTESIAN COORDINATES OF THE EARTH POINT
    U = dblarr(3,ngood)
    U(1-1,*)=XS(1-1)+H*G1(1-1,good)
    U(2-1,*)=XS(2-1)+H*G1(2-1,good)
    U(3-1,*)=XS(3-1)+H*G1(3-1,good)
    ;SINUS OF GEOCENTRIC LATITUDE                                              
    D1=U(3-1,*)/SQRT(U(1-1,*)^2+U(2-1,*)^2+U(3-1,*)^2)
    ;GEOGRAPHIC (GEODETIC) COORDINATES OF THE POINT                            
    RLAT(good)=ATAN(AEBE2*D1/SQRT(1.0D0-D1*D1))
    RLON(good)=ATAN2(U(2-1,*),U(1-1,*))
    IERR(good)=0
endif
end
;=================================================================
;=================================================================
pro sndelo,CYEW,INCEW,CYNS,INCNS,SVEW,SVNS,DOFF,GEO,iout=iout
;+
; NAME:
;    SNDELO
;
; PURPOSE:
;    CONVERT THE MIRROR POSITION, GIVEN IN CYCLES AND INCREMENTS, TO           
;    ELEVATION AND SCAN ANGLES                                                 
;
; CATEGORY:
;    ISCCP Navigation
;
; RESTRICTIONS:
;    Uses INSTCO.PRO
;
; MODIFICATION HISTORY:
;    Created 1/23/2004 Ken Knapp NOAA/NCDC
;                      based on code developed in FORTRAN
;                      for GVAR navigation
;-
if (keyword_set(iout)) then iout = 1

@instco.pro

;CONVERT THE MIRROR POSITION, GIVEN IN CYCLES AND INCREMENTS, TO           
;ELEVATION AND SCAN ANGLES                                                 
E=((CYNS-9)*INCMAX(2-1)+INCNS)*ELVINC(2-1)-ELVMAX(2-1)
S=(CYEW*INCMAX(2-1)+INCEW)*SCNINC(2-1)-SCNMAX(2-1)

;CORRECT ELEVATION AND SCAN ANGLES FOR SERVO ERRORS OBTAINING THE          
;TRUE MIRROR POINTING                                                      
E=E+SVNS                                                                  
S=S+SVEW
SINE=SIN(E)
COSE=COS(E)
H=-2.0D0*SCNPX(2-1)

;COMPUTE EARTH LOCATIONS FOR FOUR DETECTORS
for i=1,4 do begin ;DO 10 I=1,4
    ;COMPUTE POSITIONAL OFFSETS OF I-TH DETECTOR
    DE=(2.5-I)*ELVLN(2-1)+DOFF(I-1,2-1)
    DS=H+DOFF(I-1,1-1)
    ;COMPUTE ELEVATION AND SCAN ANGLES RELATED TO I-TH DETECTOR                
    ;AND CORRECT THEM FOR THE DETECTOR POSITIONAL OFFSETS                      
    ;EV=E+DOFF(I,2)
    ;SC=S+DOFF(I,1)
    ;CONVERT POSITIONAL OFFSETS TO ANGULAR OFFSETS AND
    ;CORRECT ELEVATION AND SCAN ANGLES
    EV = E + DE*COSE - DS*SINE
    SC = S + DE*SINE + DS*COSE
    ;TRANSFORM DETECTOR'S POINTING ANGLES TO GEOGRAPHIC COORDINATES            
    ;OF THE CORRESPONDING POINT ON THE EARTH SURFACE.                          
    ;NOTE:  IF A DETECTOR LOOKS OFF THE EARTH, THE RELATED LATITUDE            
    ;AND LONGITUDE ARE SET TO 999999.                                   
    LPOINT,EV,SC,GEO(I-1,1-1),GEO(I-1,2-1),IER
    H=-H
endfor                          ;  10 CONTINUE
end
;=================================================================
;=================================================================
function time50,i,iout=iout
;+
; NAME:
;    TIME50
;
; PURPOSE:
;    TIME50 ACCEPTS TWO WORDS CONTAINING DATE AND TIME
;    AND RETURNS TIME EXPRESSED AS DOUBLE PRECISION MINUTES FROM
;    1950 JAN. 1.0
;
; CATEGORY:
;    ISCCP Navigation
;
; MODIFICATION HISTORY:
;    Created 1/23/2004 Ken Knapp NOAA/NCDC
;                      based on code developed in FORTRAN
;                      for GVAR navigation
;-
if (keyword_set(iout)) then iout = 1

;CONVERT INPUT YEAR, DAY OF YEAR, HOUR AND MINUTE
;TO INTEGER VALUES.
NY =I(1-1)/10000
IAA=I(1-1)-(NY*10000)
ND=(I(1-1)-(NY*10000))*.1
IAB=(IAA-(ND*10))*10
NBC=I(2-1)/10000000.
IAC=I(2-1)-(NBC*10000000)
NH=IAB+NBC
DEF=I(2-1)-IAC
NM=IAC*.00001
S=(I(2-1)-(DEF+(NM*100000)))*.001D0

;HERE WE CONVERT INTEGER YEAR AND DAY OF YEAR TO NUMBER OF
;DAYS FROM 0 HOUR UT, 1950 JAN. 1.0
;THIS CONVERTION IS BASED ON AN ALGORITHM BY FLIEGEL AND VAN
;FLANDERN, COMM. OF ACM, VOL.11, NO. 10, OCT. 1968 (P.657)
J=ND+1461l*(NY+4799l)/4l-3l*((NY+4899l)/100l)/4l-2465022l
;COMPUTE TIME IN MINUTES FROM JANUARY 1.0, 1950                            
TIME50=J*1440.D0+NH*60.D0+NM+S/60.D0
return,time50
end
;=================================================================
;=================================================================
function timex,NY,ND,NH,NM,S,iout=iout
;+
; NAME:
;    TIMEX
;
; PURPOSE:
;    TIMEX ACCEPTS WORDS CONTAINING DATE AND TIME INFORMATION
;    AND RETURNS TIME EXPRESSED AS DOUBLE PRECISION MINUTES FROM
;    1950 JAN. 1.0
;
; CATEGORY:
;    ISCCP Navigation
;
; MODIFICATION HISTORY:
;    Created 1/23/2004 Ken Knapp NOAA/NCDC
;                      based on code developed in FORTRAN
;                      for GVAR navigation
;-
if (keyword_set(iout)) then iout = 1

J=ND+1461l*(NY+4799l)/4l-3l*((NY+4899l)/100l)/4l-2465022l
;COMPUTE ACTUAL TIME IN MINUTES FROM JANUARY 1.0, 1950                     
TIMEX=J*1440.D0+NH*60.D0+NM+S/60.D0
return,timex
end
;=================================================================
;=================================================================
pro inst2er,r,p,y,a,at,iout=iout
;+
; NAME:
;    INST2ER
;
; PURPOSE:
;    INST2ER ACCEPTS THE SINGLE PRECISION ROLL, PITCH AND YAW ANGLES
;    OF AN INSTRUMENT AND RETURNS THE DOUBLE PRECISION INSTRUMENT TO
;    EARTH COORDINATES TRANSFORMATION MATRIX.
;
; CATEGORY:
;    ISCCP Navigation
;
; MODIFICATION HISTORY:
;    Created 1/23/2004 Ken Knapp NOAA/NCDC
;                      based on code developed in FORTRAN
;                      for GVAR navigation
;-
if (keyword_set(iout)) then iout = 1

;WE COMPUTE INSTRUMENT TO BODY COORDINATES TRANSFORMATION
;MATRIX BY USING A SMALL ANGLE APPROXIMATION OF TRIGONOMETRIC
;FUNCTIONS OF THE ROLL, PITCH AND YAW.
if (keyword_set(iout)) then iout = 1
rpy = dblarr(3,3)

RPY(1-1,1-1)=1.0D0-0.50D0*(P*P+Y*Y)
RPY(1-1,2-1)=-Y
RPY(1-1,3-1)=P
RPY(2-1,1-1)=Y+P*R
RPY(2-1,2-1)=1.0D0-0.50D0*(Y*Y+R*R)
RPY(2-1,3-1)=-R
RPY(3-1,1-1)=-P+R*Y
RPY(3-1,2-1)=R+P*Y
RPY(3-1,3-1)=1.0D0-0.50D0*(P*P+R*R)

;MULTIPLICATION OF MATRICES A AND RPY
;at = dblarr(3,3)
;for i = 0,2 do begin ;DO 20 I=1,3
;    for j = 0,2 do begin ;DO 10 J=1,3
;        AT(I,J)=A(I,1-1)*RPY(1-1,J)+A(I,2-1)*RPY(2-1,J)+A(I,3-1)*RPY(3-1,J)
;    endfor                      ;10 CONTINUE
;endfor                          ;20 CONTINUE
at = a # rpy
if keyword_set(iout) then begin
;    print,'R-P-Y'
;    print,rpy
endif


end
;=================================================================
;=================================================================
pro gpoint,rlat,rlon,alf,gam,ierr,iout=iout
;+
; NAME:
;    GPOINT
;
; PURPOSE:
;     this subroutine converts geographic lat/lon to the related elevation
;     and scan angles.
;
; CATEGORY:
;    ISCCP Navigation
;
; MODIFICATION HISTORY:
;    Created 1/23/2004 Ken Knapp NOAA/NCDC
;                      based on code developed in FORTRAN
;                      for GVAR navigation
;-
if (keyword_set(iout)) then iout = 1
np = n_elements(rlat)
@elcons.pro
@elcomm.pro
@instco.pro
;COMPUTES SINUS OF GEOGRAPHIC (GEODETIC) LATITUDE
SING=SIN(RLAT)
W1=AEBE4*SING*SING

;SINUS OF THE GEOCENTRIC LATITUDE
SLAT=((0.375D0*W1-0.5D0)*W1+1.0D0)*SING/AEBE2

;COMPUTES LOCAL EARTH RADIUS AT SPECIFIED POINT
W2=SLAT*SLAT
W1=AEBE3*W2
W1=(0.375D0*W1-0.5D0)*W1+1.D0

;COMPUTES CARTESIAN COORDINATES OF THE POINT
u = dblarr(3,np)
U(3-1,*)=SLAT*W1                                                              
W2=W1*SQRT(1.0D0-W2)
U(1-1,*)=W2*COS(RLON)
U(2-1,*)=W2*SIN(RLON)
if (keyword_set(iout)) then begin
;print,'Point on Earth'
;print,u
endif
;POINTING VECTOR FROM SATELLITE TO THE EARTH POINT
f = dblarr(3,np)
F(1-1,*)=U(1-1,*)-XS(1-1)
F(2-1,*)=U(2-1,*)-XS(2-1)  
F(3-1,*)=U(3-1,*)-XS(3-1)  
W2=U(1-1,*)*F(1-1,*)+U(2-1,*)*F(2-1,*)+U(3-1,*)*F(3-1,*)*AEBE2
if (keyword_set(iout)) then begin
    print,'Pointing Vector Earth coord.'
    print,F/sqrt(total(f^2))
endif
;VERIFIES VISIBILITY OF THE POINT

vis = where(w2 le 0.d0,nvis,comp=invis,ncomp=ninvis)
ft = dblarr(3,np)
ierr = intarr(np)
alf = dblarr(np)
gam = dblarr(np)

;IF (W2 GT 0.0D0) THEN begin
;    ;INVISIBLE POINT ON THE EARTH                    
;    IERR=1
;    ALF=99999.0D0
;    GAM=99999.0D0
;ENDIF else begin
if (ninvis gt 0) then begin
    alf(invis) = 99999.d0
    gam(invis) = 99999.d0
    ierr(invis) = 1
endif
if (nvis gt 0) then begin
    FT(1-1,vis)=BT(1-1,1-1)*F(1-1,vis)+BT(2-1,1-1)*F(2-1,vis)+BT(3-1,1-1)*F(3-1,vis)
    FT(2-1,vis)=BT(1-1,2-1)*F(1-1,vis)+BT(2-1,2-1)*F(2-1,vis)+BT(3-1,2-1)*F(3-1,vis)
    FT(3-1,vis)=BT(1-1,3-1)*F(1-1,vis)+BT(2-1,3-1)*F(2-1,vis)+BT(3-1,3-1)*F(3-1,vis)
    if (keyword_set(iout)) then begin
        print,'Pointing vector in instr. coord.'
        print,ft/sqrt(total(ft^2))
    endif
    ;CONVERTS POINTING VECTOR TO INSTRUMENT COORDINATES
;    CONVERTS POINTING VECTOR TO SCAN AND ELEVATION ANGLES AND
;     CORRECTS FOR THE ROLL AND PITCH MISALIGNMENTS
    GAM(vis) =  ATAN(FT(1-1,vis)/SQRT(FT(2-1,vis)^2+FT(3-1,vis)^2))
    ALF(vis) = -ATAN(FT(2-1,vis)/FT(3-1,vis))
    W1=SIN(ALF(vis))
    W2=COS(GAM(vis))
    ALF1=ALF(vis)+RMA*(1.0D0-COS(ALF(vis))/W2) + PMA*W1*(1.0D0/W2+TAN(GAM(vis)))
    GAM(vis)=GAM(vis)-RMA*W1
    if (keyword_set(iout)) then begin
        print,'misalignment correction'
        print,RMA*(1.0D0-COS(ALF)/W2)+PMA*W1*(1.0D0/W2+TAN(GAM)),-RMA*W1
    endif
    ;OPTICAL AXIS CORRECTION
    EWNOM = 2.5d0*incmax(0)*scninc(0)
    DOFF= SCNMAX(0) - EWNOM
    if (keyword_set(iout)) then begin
; OPTICAL AXIS CORRECTION NOW MATCHES THE GEO_LL2IJ OUTPUT
;        print,'optical axis correction'
;        print,alf1,gam
;        print,scnmax(0),ewnom
;        print,ALF1*GAM*DOFF,- 0.5*ALF1*ALF1*DOFF
;        stop
    endif
    ALF(vis) = ALF1 + ALF1*GAM(vis)*DOFF
    GAM(vis) = GAM(vis) - 0.5*ALF1*ALF1*DOFF
    if (keyword_set(iout)) then begin
        print,'scan,elev'
        print,alf,gam
    endif

    IERR(vis) = 0
endif

end
;=================================================================
;=================================================================
pro setcon,INSTR,NS_NAD_CY,NS_NAD_INC,EW_NAD_CY,EW_NAD_INC,iout=iout
;+
; NAME:
;    SETCON
;
; PURPOSE:
;     this subroutine sets constants used in navigation
;
; CATEGORY:
;    ISCCP Navigation
;
; MODIFICATION HISTORY:
;    Created 1/23/2004 Ken Knapp NOAA/NCDC
;                      based on code developed in FORTRAN
;                      for GVAR navigation
;-
if (keyword_set(iout)) then iout = 1

@elcons.pro
@elcomm.pro
@instco.pro

;INCMAX(1)=6136
;INCMAX(2)=2805
incmax = [6136,2805]
;ELVINC(1)=8.0D-6
;ELVINC(2)=17.5D-6
elvinc = [8.0D-6,17.5D-6]
;SCNINC(1)=16.D-6
;SCNINC(2)=35.D-6
scninc = [16.D-6,35.D-6]
;ELVLN(1)=28.D-6
;ELVLN(2)=280.D-6
elvln = [28.D-6,280.D-6]
;SCNPX(1)=16.D-6
;SCNPX(2)=280.D-6
scnpx = [16.D-6,280.D-6]

if (NS_NAD_CY eq 0 and $
    NS_NAD_INC eq 0 and $
    EW_NAD_CY eq 0 and $
    EW_NAD_INC eq 0) then begin
    NS_NAD_CY  = 4
    NS_NAD_INC = 3068
    EW_NAD_CY  = 2
    EW_NAD_INC = 3068
endif


;RECOMPUTE ELEVATION AND SCAN BIASES BASED ON USER INPUTS OF               
;CYCLES & INCREMENTS OBTAINED FROM GVAR
;ELVMAX(INSTR) = (NS_NAD_CY*INCMAX(INSTR)+NS_NAD_INC)*ELVINC(INSTR)
elvmax = dblarr(2)
scnmax = dblarr(2)
IF(INSTR EQ 1) THEN begin
    ELVMAX(INSTR-1)=(NS_NAD_CY*INCMAX(INSTR-1)+NS_NAD_INC)*ELVINC(INSTR-1)
endif ELSE begin
    ELVMAX(INSTR-1)=((9-NS_NAD_CY)*INCMAX(INSTR-1)-NS_NAD_INC) $
      *ELVINC(INSTR-1)
ENDelse

SCNMAX(INSTR-1) = (EW_NAD_CY*INCMAX(INSTR-1)+EW_NAD_INC)*SCNINC(INSTR-1)
end
;=================================================================
;=================================================================
pro lmodel,t,tu,rec,imc,rlat,rlon,grad=grad, $
           rpyaw=rpyaw,spnax=spnax,iout=iout
;+
; NAME:
;    SETCON
;
; CATEGORY:
;    ISCCP Navigation
;
; PURPOSE:
;    THIS SUBROUTINE COMPUTES THE POSITION OF THE SATELLITE AND THE            
;    ATTITUDE OF THE IMAGER OR SOUNDER.  THE CALCULATIONS ARE BASED            
;    ON THE OATS ORBIT AND ATTITUDE MODEL REPRESENTED BY THE O&A               
;    PARAMETER SET IN GVAR BLOCK 0.                                            
;
;    LMODEL ACCEPTS AN INPUT DOUBLE PRECISION TIME IN MINUTES FROM
;    1950, JAN.1.0 AND AN INPUT SET OF O&A PARAMETERS AND COMPUTES
;    POSITION OF THE SATELLITE, THE ATTITUDE ANGLES AND ATTITUDE
;    MISALIGNMENTS AND THE INSTRUMENT TO EARTH FIXED COORDINATES
;    TRANSFORMATION MATRIX.
;
; INPUTS:                                                              
;    TIME, EPOCH TIME, O&A PARAMETER SET, IMC STATUS.
;
; OUTPUTS:
;    THE SPACECRAFT POSITION VECTOR IN EARTH FIXED COORDINATES
;    THE GEOMETRIC ROLL, PITCH, YAW ANGLES AND THE ROLL,
;    PITCH MISALIGNMENTS FOR EITHER THE IMAGER OR THE SOUNDER
;    THE EARTH FIXED TO INSTRUMENT FRAME TRANSFORMATION MATRIX
;    GEOGRAPHIC LATITUDE AND LONGITUDE AT SUBSATELLITE POINT.      
;
; MODIFICATION HISTORY:
;    Created 1/23/2004 Ken Knapp NOAA/NCDC
;                      based on code developed in FORTRAN
;                      for GVAR navigation
;-

if (keyword_set(iout)) then iout = 1

@elcons.pro
@elcomm.pro

;ASSIGN REFERENCE VALUES TO THE SUBSATELLITE LONGITUDE AND
;LATITUDE, THE RADIAL DISTANCE AND THE ORBIT YAW.

LAM=REC(5-1) 
DR=REC(6-1)
PHI=REC(7-1)
PSI=REC(8-1)

;ASSIGN REFERENCE VALUES TO THE ATTITUDES AND MISALIGNMENTS
ROLL =REC(9-1)
PITCH=REC(10-1)
YAW  =REC(11-1)
RMA  =0.0D0
PMA  =0.0D0

;IF IMC IS OFF, COMPUTE CHANGES IN THE SATELLITE ORBIT
IF (IMC NE 0) THEN begin
;    print,'INSIDE'
    ;SET REFERENCE RADIAL DISTANCE, LATITUDE AND ORBIT YAW TO ZERO
    DR=0.0D0
    PHI=0.0D0
    PSI=0.0D0
    ;COMPUTE TIME SINCE EPOCH (IN MINUTES)                                     
    TS=T-TU                                                                   
    ;COMPUTES ORBIT ANGLE AND THE RELATED TRIGONOMETRIC FUNCTIONS.             
    ;EARTH ROTATIONAL RATE=.729115E-4 (RAD/S)                                  
    W=0.729115D-4*60.0D0*TS                                                   
    SW=SIN(W)
    CW=COS(W)
    SW1=SIN(0.927D0*W)
    CW1=COS(0.927D0*W)
    S2W=SIN(2.0D0*W)
    C2W=COS(2.0D0*W)
    SW3=SIN(1.9268D0*W)
    CW3=COS(1.9268D0*W)
    ;COMPUTES CHANGE IN THE IMC LONGITUDE FROM THE REFERENCE
    LAM=LAM+REC(18-1)+(REC(19-1)+REC(20-1)*W)*W $
      +(REC(27-1)*SW1+REC(28-1)*CW1+REC(21-1)*SW+REC(22-1)*CW $
        +REC(23-1)*S2W+REC(24-1)*C2W + REC(25-1)*SW3+REC(26-1)*CW3 $
        +W*(REC(29-1)*SW+REC(30-1)*CW))*2.0D0
    ;COMPUTES CHANGE IN RADIAL DISTANCE FROM THE REFERENCE (KM)
    DR=DR + REC(31-1) + REC(32-1)*CW+REC(33-1)*SW $
      +REC(34-1)*C2W+REC(35-1)*S2W + REC(36-1)*CW3+REC(37-1)*SW3 $
      +REC(38-1)*CW1+REC(39-1)*SW1 + W*(REC(40-1)*CW+REC(41-1)*SW)
    ;COMPUTES THE SINE OF THE CHANGE IN THE GEOCENTRIC LATITUDE                
    DLAT=REC(42-1) + REC(43-1)*CW+REC(44-1)*SW $
      +REC(45-1)*C2W+REC(46-1)*S2W $
      +W*(REC(47-1)*CW+REC(48-1)*SW) $
      +REC(49-1)*CW1+REC(50-1)*SW1
    ;COMPUTES GEOCENTRIC LATITUDE BY USING AN EXPANSION FOR ARCSINE
    PHI=PHI+DLAT*(1.0D0+DLAT*DLAT/6.0D0)
    ;COMPUTES SINE OF THE CHANGE IN THE ORBIT YAW                              
    DYAW=REC(51-1) + REC(52-1)*SW+REC(53-1)*CW $
      +REC(54-1)*S2W+REC(55-1)*C2W $
      +W*(REC(56-1)*SW+REC(57-1)*CW) $
      +REC(58-1)*SW1+REC(59-1)*CW1                                          
    ;COMPUTES THE ORBIT YAW BY USING AN EXPANSION FOR ARCSINE.                 
    PSI=PSI+DYAW*(1.0D0+DYAW*DYAW/6.0D0)
    ;CALCULATION OF CHANGES IN THE SATELLITE ORBIT ENDS HERE                   
ENDIF                                                                    

;CONVERSION OF THE IMC LONGITUDE AND ORBIT YAW TO THE SUBSATELLITE         
;LONGITUDE AND THE ORBIT INCLINATION (REF: GOES-PCC-TM-2473, INPUTS        
;REQUIRED FOR EARTH LOCATION AND GRIDDING BY SPS,  JUNE 6, 1988)           
SLAT=SIN(PHI)
SYAW=SIN(PSI)
SI=SLAT^2+SYAW^2                                                        
CI=SQRT(1.0D0-SI)
SI=SQRT(SI)
IF (SLAT EQ 0.0D0 AND SYAW  EQ  0.0D0) THEN begin
    U=0.0D0
endif ELSE begin
    U=ATAN2(SLAT,SYAW)
ENDelse
SU=SIN(U)
CU=COS(U)
;COMPUTES LONGITUDE OF THE ASCENDING NODE
ASC=LAM - U
SA=SIN(ASC)
CA=COS(ASC)
;COMPUTES THE SUBSATELLITE GEOGRAPHIC LATITUDE                             
RLAT=ATAN(AEBE2*TAN(PHI))
;COMPUTES THE SUBSATELLITE LONGITUDE                                       
RLON=ASC+ATAN2(CI*SU,CU)
;COMPUTES THE SPACECRAFT TO EARTH FIXED COORDINATES TRANSFORMATION         
;MATRIX:                                                                   
;(VECTOR IN ECEF COORDINATES) = B * (VECTOR IN S/C COORDINATES)
b = dblarr(3,3)
B(1-1,1-1)=-CA*SU-SA*CU*CI                                                    
B(2-1,1-1)=-SA*SU+CA*CU*CI                                                    
B(3-1,1-1)= CU*SI
B(1-1,2-1)=-SA*SI                                                             
B(2-1,2-1)= CA*SI                                                             
B(3-1,2-1)=-CI                                                                
B(1-1,3-1)=-CA*CU+SA*SU*CI                                                    
B(2-1,3-1)=-SA*CU-CA*SU*CI                                                    
B(3-1,3-1)=-SLAT                                                              
;COMPUTES THE NORMALIZED SPACECRAFT POSITION VECTOR IN EARTH FIXED
;COORDINATES - XS.
R=(NOMORB+DR)/AE

;COMPUTE GRAD AND SPNAX USED BY GEO_IJ2LL AND GEO_LL2IJ
grad = NOMORB+DR

;THESE ARE EFFECTIVE SPIN AXIS RIGHT ASCENSION AND DECLINATION USED TO
;NAVIGATE THE IMAGERY
nspdc = asin(b(2,1))
nspra = atan2(b(2,0),b(2,2))
spnax = [nspra, nspdc]

xs = dblarr(3)
XS(1-1)=-B(1-1,3-1)*R
XS(2-1)=-B(2-1,3-1)*R
XS(3-1)=-B(3-1,3-1)*R
if (keyword_set(iout)) then begin
; SATELLITE POSITIONS MATCH TO WITHIN 50 METERS
;print,'Satellite position'
;print,xs
;stop
print,'matrix b'
print,b
endif
;PRECOMPUTES Q3 (USED IN LPOINT)                                           
Q3=XS(1-1)^2+XS(2-1)^2+AEBE2*XS(3-1)^2-1.0D0
;COMPUTES THE ATTITUDES AND MISALIGNMENTS IF IMC IS OFF                    
IF (IMC NE 0) THEN begin
;    print,'INSIDE TOO'
    ;COMPUTES THE SOLAR ORBIT ANGLE                                            
    WA=REC(60-1)*TS     
    ;COMPUTES THE DIFFERENCE BETWEEN CURRENT TIME, TS, AND THE
    ;EXPONENTIAL TIME, REC(61). NOTE THAT BOTH TIMES ARE SINCE EPOCH.
    TE=TS-REC(61-1)                                                          
    ;COMPUTES ROLL + ROLL MISALIGNMENT                                         
    ROLL=ROLL+GATT(62,REC,WA,TE)                                           
    ;COMPUTES PITCH + PITCH MISALIGNMENT                                       
    PITCH=PITCH+GATT(117,REC,WA,TE)                                        
    ;COMPUTES YAW                                                              
    YAW=YAW+GATT(172,REC,WA,TE)                                            
    ;COMPUTES ROLL MISALIGNMENT                                                
    RMA=GATT(227,REC,WA,TE)                                                
    ;COMPUTES PITCH MISALIGNMENT                                               
    PMA=GATT(282,REC,WA,TE)                                                
    ;APPLY THE SPCECRAFT COMPENSATION
    ROLL =ROLL +REC(15-1)
    PITCH=PITCH+REC(16-1)
    YAW  =YAW  +REC(17-1)
ENDIF
;COMPUTES THE INSTRUMENT TO EARTH FIXED COORDINATES TRANSFORMATION
;MATRIX - BT

;print,'ROLL=PITCH=YAW=0.d0'
;roll=0.d0 & pitch=0.d0 & yaw=0.d0
INST2ER,ROLL,PITCH,YAW,B,BT,iout=iout
if (keyword_set(iout)) then begin
    print,'BT'
    print,bt
endif
rpyaw = [roll,pitch,yaw]/rad

end
;=================================================================
;=================================================================
pro navtorec,NAV,rec,iout=iout
;+
; NAME:
;    NAVTOREC
;
; PURPOSE:
;    THIS ROUTINE CONVERTS THE NAVIGATION STRUCTURE TO THE
;    REC ARRAY
;
; CATEGORY:
;    ISCCP NAVIGATION
;
; MODIFICATION HISTORY:
;    Created 1/23/2004 Ken Knapp NOAA/NCDC
;
;-
if (keyword_set(iout)) then iout = 1
rec = dblarr(337)
;UNSURE OF SOME OF THE SCALING
rec(5) = NAV.ref_long * 1.d-7
rec(6) = NAV.ref_rad_dist * 1.d-7
rec(7) = NAV.ref_lat * 1.d-7
rec(8) = NAV.ref_orb_yaw * 1.d-7
rec(9) = NAV.ref_att_roll * 1.d-7
rec(10) = NAV.ref_att_pitch * 1.d-7
rec(11) = NAV.ref_att_yaw * 1.d-7
;rec(12) = NAV.   epoch
;rec(13) = NAV.   epoch
rec(14) = NAV.start_time*1.d-2
rec(15) = NAV.imc_corr_roll * 1.d-7
rec(16) = NAV.imc_corr_pitch * 1.d-7
rec(17) = NAV.imc_corr_yaw * 1.d-7
rec(18:30) = NAV.ref_long_change * 1.d-7
rec(31:41) = NAV.ref_rad_dist_change * 1.d-7
rec(42:50) = NAV.sine_lat * 1.d-7
rec(51:59) = NAV.sine_orb_yaw * 1.d-7
rec(60) = NAV.solar_rate * 1.d-7
rec(61) = NAV.exp_start_time*1.d-2
st = 62
rec(st) = NAV.roll_att.exp_mag* 1.d-7
rec(st+1) = NAV.roll_att.exp_time_const*1.d-2
rec(st+2) = NAV.roll_att.mean_att_ang_const* 1.d-7
rec(st+3) = NAV.roll_att.num_sinu_per_angle
ind = st+ 4 + lindgen(15)*2
rec(ind) = NAV.roll_att.sinusoid(*).mag_sinu* 1.d-7
rec(ind+1) = NAV.roll_att.sinusoid(*).phase_ang_sinu* 1.d-7
rec(st+34) =  NAV.roll_att.NUM_MONO_SINU
ind = st + 35 + lindgen(4)*5
rec(ind) = NAV.roll_att.monomial(*).order_appl_sinu
rec(ind+1)=NAV.roll_att.monomial(*).ORDER_MONO_SINU
rec(ind+2)=NAV.roll_att.monomial(*).MAG_MONO_SINU* 1.d-7
rec(ind+3)=NAV.roll_att.monomial(*).PHASE_ANG_SINU* 1.d-7
rec(ind+4)=NAV.roll_att.monomial(*).ANG_FROM_EPOCH* 1.d-7
st = 117
rec(st) = NAV.pitch_att.exp_mag* 1.d-7
rec(st+1) = NAV.pitch_att.exp_time_const*1.d-2
rec(st+2) = NAV.pitch_att.mean_att_ang_const* 1.d-7
rec(st+3) = NAV.pitch_att.num_sinu_per_angle
ind = st+ 4 + lindgen(15)*2
rec(ind) = NAV.pitch_att.sinusoid(*).mag_sinu* 1.d-7
rec(ind+1) = NAV.pitch_att.sinusoid(*).phase_ang_sinu* 1.d-7
rec(st+34) =  NAV.pitch_att.NUM_MONO_SINU
ind = st + 35 + lindgen(4)*5
rec(ind) = NAV.pitch_att.monomial(*).order_appl_sinu
rec(ind+1)=NAV.pitch_att.monomial(*).ORDER_MONO_SINU
rec(ind+2)=NAV.pitch_att.monomial(*).MAG_MONO_SINU* 1.d-7
rec(ind+3)=NAV.pitch_att.monomial(*).PHASE_ANG_SINU* 1.d-7
rec(ind+4)=NAV.pitch_att.monomial(*).ANG_FROM_EPOCH* 1.d-7
st = 172
rec(st) = NAV.yaw_att.exp_mag* 1.d-7
rec(st+1) = NAV.yaw_att.exp_time_const*1.d-2
rec(st+2) = NAV.yaw_att.mean_att_ang_const* 1.d-7
rec(st+3) = NAV.yaw_att.num_sinu_per_angle
ind = st+ 4 + lindgen(15)*2
rec(ind) = NAV.yaw_att.sinusoid(*).mag_sinu* 1.d-7
rec(ind+1) = NAV.yaw_att.sinusoid(*).phase_ang_sinu* 1.d-7
rec(st+34) =  NAV.yaw_att.NUM_MONO_SINU
ind = st + 35 + lindgen(4)*5
rec(ind) = NAV.yaw_att.monomial(*).order_appl_sinu
rec(ind+1)=NAV.yaw_att.monomial(*).ORDER_MONO_SINU
rec(ind+2)=NAV.yaw_att.monomial(*).MAG_MONO_SINU* 1.d-7
rec(ind+3)=NAV.yaw_att.monomial(*).PHASE_ANG_SINU* 1.d-7
rec(ind+4)=NAV.yaw_att.monomial(*).ANG_FROM_EPOCH* 1.d-7
st = 172+55
rec(st) = NAV.roll_misalgn.exp_mag* 1.d-7
rec(st+1) = NAV.roll_misalgn.exp_time_const*1.d-2
rec(st+2) = NAV.roll_misalgn.mean_att_ang_const* 1.d-7
rec(st+3) = NAV.roll_misalgn.num_sinu_per_angle
ind = st+ 4 + lindgen(15)*2
rec(ind) = NAV.roll_misalgn.sinusoid(*).mag_sinu* 1.d-7
rec(ind+1) = NAV.roll_misalgn.sinusoid(*).phase_ang_sinu* 1.d-7
rec(st+34) =  NAV.roll_misalgn.NUM_MONO_SINU
ind = st + 35 + lindgen(4)*5
rec(ind) = NAV.roll_misalgn.monomial(*).order_appl_sinu
rec(ind+1)=NAV.roll_misalgn.monomial(*).ORDER_MONO_SINU
rec(ind+2)=NAV.roll_misalgn.monomial(*).MAG_MONO_SINU* 1.d-7
rec(ind+3)=NAV.roll_misalgn.monomial(*).PHASE_ANG_SINU* 1.d-7
rec(ind+4)=NAV.roll_misalgn.monomial(*).ANG_FROM_EPOCH* 1.d-7
st = 172+55+55
rec(st) = NAV.pitch_misalgn.exp_mag* 1.d-7
rec(st+1) = NAV.pitch_misalgn.exp_time_const*1.d-2
rec(st+2) = NAV.pitch_misalgn.mean_att_ang_const* 1.d-7
rec(st+3) = NAV.pitch_misalgn.num_sinu_per_angle
ind = st+ 4 + lindgen(15)*2
rec(ind) = NAV.pitch_misalgn.sinusoid(*).mag_sinu* 1.d-7
rec(ind+1) = NAV.pitch_misalgn.sinusoid(*).phase_ang_sinu* 1.d-7
rec(st+34) =  NAV.pitch_misalgn.NUM_MONO_SINU
ind = st + 35 + lindgen(4)*5
rec(ind) = NAV.pitch_misalgn.monomial(*).order_appl_sinu
rec(ind+1)=NAV.pitch_misalgn.monomial(*).ORDER_MONO_SINU
rec(ind+2)=NAV.pitch_misalgn.monomial(*).MAG_MONO_SINU* 1.d-7
rec(ind+3)=NAV.pitch_misalgn.monomial(*).PHASE_ANG_SINU* 1.d-7
rec(ind+4)=NAV.pitch_misalgn.monomial(*).ANG_FROM_EPOCH* 1.d-7
rec = rec(1:336)

end
;=================================================================
;=================================================================
pro bcd_time,time,year,jday,hh,mm,sec,msec,iout=iout
;+
; NAME:
;    BCD_TIME
;
; PURPOSE:
;    THIS FUNCTION CONVERTS A BCD TIME ARRAY TO UTC VARIABLES
;
; CATEGORY:
;    ISCCP NAVIGATION
;
; RESTRICTIONS:
;
;
; MODIFICATION HISTORY:
;    Created 1/23/2004 Ken Knapp NOAA/NCDC
;
;-
btime = byte(time,0,8)
if (keyword_set(iout)) then iout = 1 else iout = 0

yy1 = btime(3)/16
yy2 = btime(3) mod 16
yy3 = btime(2)/16
yy4 = btime(2) mod 16
year = (yy1*1000l + yy2*100l + yy3*10l + yy4) mod 10000
jd1 = btime(1) / 16
jd2 = btime(1) mod 16
jd3 = btime(0) / 16
jday = jd1*100 + jd2*10 + jd3

hh1 = btime(0) mod 16
hh2 = btime(7) / 16
mm1 = btime(7) mod 16
mm2 = btime(6) / 16
ss1 = btime(6) mod 16
ss2 = btime(5) / 16
ms1 = btime(5) mod 16
ms2 = btime(4) / 16
ms3 = btime(4) mod 16

hh = hh1*10 + hh2
mm = mm1*10 + mm2
sec = ss1*10 + ss2
msec = ms1*100 + ms2*10 + ms3
if keyword_set(iout) then print,year,jday,hh,mm,sec,msec
end
;===================================================================
;=================================================================
;=================================================================
;=================================================================
;=================================================================
;****I* idl/idl.b1u_imc
;+
; NAME:
;    IMCNAV
;
; PURPOSE:
;    Navigate GVAR imagery. It converts either:
;       line/pixel --> lat/lon
;                  or
;       lat/lon --> line/pixel
;
; CATEGORY:
;    ISCCP Navigation
;
;
; INPUTS:
;    dlat = latitude (in degrees, North is positive)
;    dlon = longitude(in degrees, East is positive)
;    line = scan line number (in full resolution ... visible elements)
;    pixel= pixel number (in full resolution ... visible elements)
;    key = direction of transformation
;          key=0 means line/pixel --> lat/lon
;          key=1 means lat/lon --> line/pixel
;    imc = IMC status
;    instr = GVAR instrument
;           Imager: Instr = 1
;           Sounder:Instr = 2 [NOT TESTED!!!]
;
; OPTIONAL INPUTS:
;    file = filename of file being navigated ... necessary for
;           AES navigation
;    rec = rec array containing Navigation information
;    AREAstr = McIdas area structure
;    NAVstr = McIdas Navigation structure
;
; OUTPUTS:
;    Either dlat/dlon or line/pixel depending on KEY=1 or 2
;
; OPTIONAL OUTPUTS:
;    glon = satellite subpoint longitude
;    glat = satellite subpoint latitude
;
; USES
; * atan2.pro
; * bcd_time.pro
; * elcomm.pro
; * elcons.pro
; * evln.pro
; * evsc2l.pro
; * gatt.pro
; * gpoint.pro
; * inst2er.pro
; * instco.pro
; * lmodel.pro
; * lpoint.pro
; * navtorec.pro
; * setcon.pro
; * sndelo.pro
; * tempfile.pro
; * time50.pro
; 
; USED BY
; * b1u_ij2ll.pro
; * b1u_ll2ij.pro
; 
; MODIFICATION HISTORY:
;    $Id: b1u_imc.pro 22797 2010-09-16 13:07:34Z kknapp $
;    Created 1/23/2004 Ken Knapp NOAA/NCDC
;                      based on code developed in FORTRAN
;                      for GVAR navigation
;            3/11/2004 Imported to CVS
;
;-
; SOURCE 

pro b1u_imc,dlat,dlon,line,pixel,key,imc,instr,NAVstr,AREAstr,$
            glat=glat,glon=glon,rec=rec,file=file,grad=grad, $
            rpyaw=rpyaw,spnax=spnax,ioutput=ioutput,MA=MA, $
            SCANANGLE=SCANANGLE
if (keyword_set(ioutput)) then iout = 1 else iout = 0

if (keyword_set(file)) then f1 = strsplit(file,'.',/extr)

if keyword_set(SCANANGLE) then begin
    print,'SCANANGLE not yet implemented'
    stop
endif


@elcons.pro
@instco.pro
@elcomm.pro

nav = navstr
area=areastr

;SET UP THE REC parameter
navtype = size(nav.ref_long,/type)
if (navtype eq 3) then begin
    ;nav.ref_long = LONG
    ;this means it is a CSU nav header
    iarea = 1
endif else if (navtype = 5) then begin
    ;nav.ref_long = DOUBLE
    ;this is an AES nav header
    iarea = 2
endif else begin
    ;ELSE stop and see what it is
    stop
endelse

if (iarea eq 1) then begin
    ;AREA file from CIRA/Mcidas output
    ;prepare input for fortran routine
    navtorec,NAV,rec
    EW_NAD_CY  = nav.ew_cyl
    EW_NAD_INC = nav.ew_inc
    NS_NAD_CY  = nav.ns_cyl
    NS_NAD_INC = nav.ns_inc
    if (iout) then print,nav.epoch_time
    bcd_time,nav.epoch_time,year,jday,hh,mm,sec,msec,iout=iout
    if (year lt 1990 or year gt 2050) then begin
        bcd_time,swap_endian(long(nav.epoch_time,0,2)),year,jday,hh,mm,sec,msec
        if (year lt 1990 or year gt 2050) then begin
            message,'Bad Epoch time in iarea=1',/info
        endif
    endif

    nye = long(year) & nde = long(jday)
    nhe = long(hh) & nme = long(mm)
    se = double(sec) + msec / 1000.d0
    imgdate = nav.img_date
    imgtime = nav.img_time
    ny = long(imgdate / 1000 ) + 1900l
    nd = long(imgdate mod 1000)
    nh = long(imgtime / 10000000)
    nm = long((imgtime / 100000) mod 100)
    s = double(imgtime mod 100000) / 1000.d0
    ;Originally, this has long been commented out, leaving imc=0
;    IMC = long(bittest(nav.imc_status,7))
;   Now I include this since it provides more accurate input as
;   compared with GPCP figures. But this does not help the HURSAT-B1
;   data. so will leave IMC hardcoded to 0 for now!
    IMC=0
    if (nav.imc_status ne 0) then IMC = 1
    ew_offset = area.west_vis_elem
    ns_offset = area.north_bound
    line_res = area.line_res
    elem_res = area.elem_res
endif else if (iarea eq 2) then begin
    EW_NAD_CY  = long(nav.nadir_offsets.EW_NAD_CY)
    NS_NAD_CY  = long(nav.nadir_offsets.NS_NAD_CY)
    NS_NAD_INC = long(nav.nadir_offsets.NS_NAD_INC)
    EW_NAD_INC = long(nav.nadir_offsets.EW_NAD_INC)
    ew_offset = long(string(area(34:38)))
    ns_offset = long(string(area(22:26)))
    name = tempfile()
    bsd = byte((long(nav.bcd_epoch_time)),0,8)
    if (iout) then print,bsd
    bcd_time,bsd,nye,nde,nhe,nme,sec,msec
    if (iout) then print,'Epoch:',nye,nde,nhe,nme,sec,msec
    se = float(sec) + msec / 1000.
    imc = nav.imc_flag
    ny = long(string(area(2:5)))
    mon = fix(f1(5))
    day = fix(f1(6))
    nd = julday(mon,day,ny)-julday(1,1,ny)+1
    nh = fix(f1(7)) / 100
    nm = fix(f1(7)) mod 100
    s = 0.d0
;    nhe = 0
    elem_res = 16
    line_res = 8
endif

;HARDCODING
IMC=0


if (iout) then print,'Epoch:',nye,nde,nhe,nme,sec,msec
;make the calls
;      DATA SVX,SVY/-21.E-6,14.E-6/
svx = -21.E-6
svy = 14.E-6
;DATA DOFF/28.E-6,56.E-6,-28.E-6,-56.E-6,84.E-6,112.E-6,14.E-6,42.E-6/
doff = [28.E-6,56.E-6,-28.E-6,-56.E-6,84.E-6,112.E-6,14.E-6,42.E-6]
doff = reform(doff,4,2)
;DATA ICX,INCX,ICY,INCY/1,2715,5,2580/
icx = 1
incx = 2715
icy = 5
incy = 2580

SETCON,INSTR,NS_NAD_CY,NS_NAD_INC,EW_NAD_CY,EW_NAD_INC

;COMPUTE EPOCH TIME                                                        
TU=TIMEX(NYE,NDE,NHE,NME,SE)                                              
;COMPUTE ACTUAL TIME                                                       
T=TIMEX(NY,ND,NH,NM,S)
;COMPUTE DATA NEEDED FOR INSTRUMENT TO EARTH COORDINATES                   
;TRANSFORMATION                                              

if (iout) then print,'T,TU:',t,tu,t-tu

LMODEL,T,TU,REC,IMC,RLAT,RLON, $
  grad=grad,rpyaw=rpyaw,iout=iout,spnax=spnax
glat=rlat
glon=rlon
;IF (KEY .EQ. 1)GO TO 10
if (key ne 1) then begin
    if (n_elements(line) ne n_elements(pixel)) then begin
        print,'Number of points are not equal'
        return
    endif
    ;TRANSFORM LINE/PIXEL TO GEOGRAPHIC COORDINATES:
    ;SET INPUT LINE/PIXEL NUMBERS                                            
    rl = line
    rp = pixel

    ss = size(rl)
    rl = reform([rl],n_elements(rl))
    rp = reform([rp],n_elements(rp))

    ;COMPUTE ELEVATION AND SCAN ANGLES (E,S) RELATED TO INPUT                
    ;LINE AND PIXEL NUMBERS                                                  
    E=EVLN(INSTR,RL)                                                  
    S=SCPX(INSTR,RP)
;if (iout) then stop
    ;TRANSFORM ELEVATION AND SCAN ANGLES TO GEOGRAPHIC COORDINATES           
    LPOINT,E,S,RLAT,RLON,IER,iout=iout
    dlat = RLAT*deg
    dlon = RLON*deg

    ;reform the array as needed
    if (ss(0) gt 0) then begin
        dlat = reform(dlat,ss(1:ss(0)))
        dlon = reform(dlon,ss(1:ss(0)))
    endif
endif else begin
    ;IF KEY.EQ.1
    ;GEOGRAPHIC TO LINE/PIXEL COORDINATES TRANSFORMATION:                      
    ;SET INPUT LATITUDE AND LONGITUDE
    ss = size(dlat)
    dlat = reform([dlat],n_elements(dlat))
    dlon = reform([dlon],n_elements(dlon))
    if (n_elements(dlat) ne n_elements(dlon)) then begin
        print,'Number of points are not equal'
        return
    endif
    RLAT=DLAT*RAD                                                     
    RLON=DLON*RAD  
    ;TRANSFORM LAT/LON TO ELEVATION AND SCAN ANGLES                          
    GPOINT,RLAT,RLON,E,S,IER,iout=iout

    if (keyword_set(iout)) then print,'angles::',e,s
    ;CONVERT ELEVATION AND SCAN ANGLES TO LINE/PIXEL COORDINATES
    onearth = where(ier eq 0,non)
    line = dblarr(n_elements(dlat))
    pixel =dblarr(n_elements(dlat))
    line(*) = -1.d0
    pixel(*)= -1.d0
    if (non gt 0) then begin    ;IF (IER EQ 0) THEN begin
        EVSC2L,INSTR,E,S,RL,RP,iout=iout
        pixel = rp
        line = rl
    ENDIF
    if (keyword_set(iout)) then print,'l,p',rl,rp
    if (ss(0) gt 0) then begin
        pixel = reform(pixel,ss(1:ss(0)))
        line  = reform(line, ss(1:ss(0)))
    endif
endelse

MA= [RMA,PMA]

end

;***
