;****I* idl/idl.b3_channel_order
; NAME
; b3_channel_order
; 
; PURPOSE
; To determine the instrument channel order based on B3 data files and
; assign B1U channel names.
; 
; USAGE
; See b1read_calv2.pro
; 
; INPUT
; OUTPUT
; RESULT
; EXAMPLE
; USES
; * asctab.pro
; * b3file_from_b1file.pro
; * b3read.pro
; 
; USED BY
; * b1read_calv2.pro
; 
; NOTES
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; Unknown
; 
; HISTORY
; Jun2010 - Added Header
; 
; SOURCE

pro b3_channel_order,file,order, $
                     b3f=b3f,cal=cal, $
                     bt=bt

f = strsplit(file,'.',/extr)
order = ''
;get b3 filename from b1 filename
if keyword_set(b3f) then begin
    b3file = b3f
endif else begin
    b3file = b3file_from_b1file(file)
endelse
if (b3file eq '') then return
;print,b3file
b3read,b3file,image,lat,lon,mu,mun,azi,nchan,iidr,cal=cal
if (nchan eq 0) then begin
    if (f(3) eq 'GOE-6' and $
        f(4) eq '1988') then begin
        b3ch=['VIS','IR']
        nchan = 2
        order = strarr(nchan)
    endif else begin
        stop
    endelse
    fail = 0
endif else begin
    order = strarr(nchan)
    b3ch = strtrim(string(iidr.chid),2)
    fail = 0
endelse

tt=total(iidr.chid,1)

tryasctab:
;recognized channels
rec = b3ch eq 'VIS' or $
      b3ch eq 'IR' or $
      b3ch eq 'NI' or $
      b3ch eq 'NIR' or $
      b3ch eq 'WV' or $
      b3ch eq 'IR2' or $
      b3ch eq 'I2' or $
      b3ch eq '0.8' or $
      b3ch eq '12.0' or tt eq 0
;Stop if a B3 channel is not recognized
if (total(rec) ne nchan and fail eq 0) then begin
;   Use asctab instead and try again.
    fail = 1
    nnn = (size(iidr.chid))[2]
    b3chorig=b3ch
    b3ch = strarr(nnn)
    for ii=0,nnn-1 do begin
        b3ch(ii) = strtrim(strjoin(asctab(iidr.chid(*,ii))),2)
    endfor
    goto,tryasctab
endif
if (total(rec) ne nchan and fail eq 1) then begin
    message,'Unrecognized channel in the B3 file',/info
    print,'b3file:',b3file
    print,'ASCII  b3ch:',b3chorig
    print,'ASCTAB b3ch:',b3ch
    print,iidr.chid
    stop
endif
;Visible
c = where(b3ch eq 'VIS',nc)
if (nc gt 0) then order(c) = 'VSCHN'
;IR Window
c = where(b3ch eq 'IR',nc)
if (nc gt 0) then order(c) = 'IRWIN'
;Near IR
c = where(b3ch eq 'NI',nc)
if (nc gt 0) then order(c) = 'IRNIR'
;Water Vapor
c = where(b3ch eq 'WV',nc)
if (nc gt 0) then order(c) = 'IRWVP'
;IR Split Window
c = where(b3ch eq 'IR2' or b3ch eq 'I2',nc)

if (nc gt 0) then begin
    if (f(3) eq 'GOE-12') then begin
        order(c) = 'IRCO2'
    endif else begin
        order(c) = 'IRSPL'
    endelse 
endif

;adjust the order to match the BT file if a BT is specified.
if keyword_set(BT) then begin
    ;BT files always have 5 entries. Generally, the first order is:
    ;VISIBLE, IR, IR (IRWVP), IR, IR
    btorder=strarr(5)
    btorder(iidr.ch_code(0:nchan-1)-1) = order(0:nchan-1)
    order=btorder
endif 


;Visible
;c = where(b3ch eq 'VIS',nc)
;if (nc gt 0) then order(c) = 'VSCHN'
end

;***
