;+
; NAME:
;    b3read
;
; PURPOSE:
;    Read in ISCCP B3 file and return file headers and B3 image in
;    arrays.
;
; CATEGORY:
;    ISCCP
;
; CALLING SEQUENCE:
;
;
;
; INPUTS:
;
;
;
; OPTIONAL INPUTS:
;
;
;
; KEYWORD PARAMETERS:
;
;
;
; OUTPUTS:
;
;
;
; OPTIONAL OUTPUTS:
;
;
;
; COMMON BLOCKS:
;
;
;
; SIDE EFFECTS:
;
;
;
; RESTRICTIONS:
;    Requires: reset.pro
;
;
; EXAMPLE:
;
;
; USES
; * stripname.pro
; * reset.pro
; 
; USED BY
; * b3_channel_order.pro
; 
; 
; MODIFICATION HISTORY:
;    $Id: b3read.pro 22670 2010-09-10 19:37:13Z kknapp $
;-
; SOURCE

pro b3read,filename,image,lat,lon,mu,mun,azi,nchan,iidr, $
           calrec=calrec
reset
if (n_elements(filename) eq 0) then begin
    print,'Usage:'
    print,'B3READ,file,image,lat,lon,mu,mun,azi,nchan,iidr'
    return
endif
if (filename eq '') then begin
    filename = dialog_pickfile(path='~/data/ISCCP-test/', $
                               filter='*.B3*')
endif

;for reading the B3 data that doesn't match the early data

; CLOSER TO A UNIFIED IDL B3READER
; NOW INCORPORATING THE CHANGES FROM 1996
; CURRENT VERSION READS ALL B3 FILES!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


asctab = [ ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',$
           ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',$
           ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',$
           ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',$
           ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',$
           ' ',' ',' ',' ',' ','.','<','(','+','|','&',' ',' ',' ',$
           ' ',' ',' ',' ',' ',' ','!','$','*',')',';','^','-','/',$
           ' ',' ',' ',' ',' ',' ',' ',' ',' ',',','%','_','>','?',$
           ' ',' ',' ',' ',' ',' ',' ',' ',' ','`',':','#','@','\',$
           '=','"',' ','a','b','c','d','e','f','g','h','i',' ',' ',$
           ' ',' ',' ',' ',' ','j','k','l','m','n','o','p','q','r',$
           ' ',' ',' ',' ',' ',' ',' ','~','s','t','u','v','w','x',$
           'y','z',' ',' ',' ','[',' ',' ',' ',' ',' ',' ',' ',' ',$
           ' ',' ',' ',' ',' ',' ',' ',']',' ',' ','{','A','B','C',$
           'D','E','F','G','H','I',' ',' ',' ',' ',' ',' ','}','J',$
           'K','L','M','N','O','P','Q','R',' ',' ',' ',' ',' ',' ',$
           '\\',' ','S','T','U','V','W','X','Y','Z',' ',' ',' ',' ',$
           ' ',' ','0','1','2','3','4','5','6','7','8','9',' ',' ',$
           ' ',' ',' ',' ']

; image identification header
   IIDR= {record:0l, $
          seq:intarr(2), $
          spcid:bytarr(8),$
          satid:bytarr(8),$
          year:0l,$
          jday:0l,$
          time:0l,$
          nchan:0l,$
          chid:bytarr(4,5),$
          nlines:0l,$
          npixels:0l,$
          firsttime:0l,$
          lasttime:0l,$
          firstday:0l,$
          lastday:0l,$
          nrecords:0l,$
          lat_to_deg:0l,$
          max_err_lat:0l,$
          lon_to_deg:0l,$
          max_err_lon:0l,$
          scale_mu:0l,$
          mu_err:0l,$
          scale_munot:0l,$
          munot_err:0l,$
          scale_relaz:0l,$
          relaz_err:0l,$
          chnse:lonarr(5),$
          ch_info:bytarr(4*10,5),$
          cal_flag_vis:0l,$
          cal_flag_ir:0l,$
          bad_scans:0l,$
          glon:0l,$
          nom_img_time:0l,$
          glat:0l,$
          nom_img_time2:0l,$
          spc_id_code:0l,$
          sat_id_code:0l,$
          ch_code:lonarr(5),$
          ch_flag:lonarr(5),$
          day_flag:0l,$
          pad1:lonarr(643),$
          pad2:lonarr(1250) }

   IIDR96= {record:0l, $
            seq:intarr(2), $
            spcid:bytarr(8),$
            satid:bytarr(8),$
            year:0l,$
            jday:0l,$
            time:0l,$
            nchan:0l,$
            chid:bytarr(4,5),$
            nlines:0l,$
            npixels:0l,$
            firsttime:0l,$
            lasttime:0l,$
            firstday:0l,$
            lastday:0l,$
            nrecords:0l,$
            lat_to_deg:0l,$
            max_err_lat:0l,$
            lon_to_deg:0l,$
            max_err_lon:0l,$
            scale_mu:0l,$
            mu_err:0l,$
            scale_munot:0l,$
            munot_err:0l,$
            scale_relaz:0l,$
            relaz_err:0l,$
            chnse:lonarr(5),$
            ch_info:bytarr(4*10,5),$
            cal_flag_vis:0l,$
            cal_flag_ir:0l,$
            cal_flag_ch3:0l,$
            cal_flag_ch4:0l,$
            cal_flag_ch5:0l,$
            bad_scans:0l,$
            glon:0l,$
            nom_img_time:0l,$
            glat:0l,$
            nom_img_time2:0l,$
            spc_id_code:0l,$
            sat_id_code:0l,$
            ch_code:lonarr(5),$
            ch_flag:lonarr(5),$
            day_flag:0l,$
            pad1:lonarr(640),$
            pad2:lonarr(1250) }

;Location grid record
LGR =  {record:0l, $
       sequence:intarr(2),$
       npix:lonarr(648) }

; Image calibration record
CalTable= { ctable,$
            phy_unt:bytarr(80),$
            phy_src:bytarr(80),$
            phy_scl:0l,$
            phy_scl_slp:0l,$
            phy_scl_off:0l,$
            phy_scl_rms:0l,$
            phy_scl_min:0l,$
            phy_scl_max:0l,$
            phy_table:lonarr(256)}

ICR = { record:0l,$
        sequence:intarr(2),$
        ch_id:0l,$
        RPU_nom:CalTable,$      ;RPU = radiance physical units
        RPU_nrm:CalTable,$
        RPU_abs:CalTable,$
        APU_nom:CalTable,$      ;APU = alternate physical unit
        APU_nrm:CalTable,$
        APU_abs:CalTable }


; Record ID
RID = { record:0l,$
       sequence:intarr(2),$
       scan_beg:0, scan_end:0,$
       scl_min_lat:0, scl_max_lat:0,$ ;for units of deg, multiply by 0.05
       scl_min_lon:0, scl_max_lon:0,$ ;for units of deg, multiply by 0.05
       scl_min_mu:0,scl__max_mu:0,$   ;for mu values, multiply by 0.01
       scl_min_mun:0, scl_max_mun:0,$ ;for mun values, multiply by 0.01
       undefined:lonarr(2)}


; Scanline directory
SLD = { pointer:0,$
       number:0,$
       unused:0,$
       rad_pntr:0,$
       n_lat:0,$
       n_lon:0,$
       n_mu:0,$
       n_mun:0,$
       n_azi:0,$
       n_data:0,$
       quality:0,$
       ch1_qual:0,$
       ch2_qual:0,$
       ch3_qual:0,$
       ch4_qual:0,$
       ch5_qual:0,$
       time:0l } ;HHMMSS
;SCAN QUALITY: SLD.quality
; 0=good, 1=bad scan, 2=nav error,3=nav fit error
; if SCD.quality > 0, then only SLD is present
;
;Channel quality
; -1 = missing, 0 = good data, >0 = bad data
; if bad data, then all data set to 255 on scan for given channel

; Navigation directory
ND = {  beg_pix:0, end_pix:0,$
       fo:0l,$
       d1:0l, d2:0l }

; data directory
DD = {  nbytes:0,$  ;bytes per pixel
       pointer:0,$ ;pointer to 1st data byte
       code:0,$  ;data code
       npix:0 }  ;number of pixels in range
       ;data range is defined in consecutive pixels that
       ;have the same data code.
       ;code = -1 off planet
       ;      0 on  planet - daytime
       ;     +1 on  planet - night time



;****************************************************************
; USER INITIALIZED DATA
;****************************************************************
ioutput=0
;if (ioutput eq 1) then

;****************************************************************
;****************************************************************
; satellite radiance data

;files=findfile('h:\data\isccp-test\*B3.*',count=nfiles)
files=filename
nfiles=1
; CHANGE THE FILE LOOP
for ifile=0,nfiles-1 do begin

file=files(ifile)
;print,file

f = stripname(file)
ff = strsplit(f,'.',/extr)

icomp=0
if (ff(n_elements(ff)-1) eq 'gz') then icomp=1

;stop
irec = 0

iall = 0

; OPEN FILE
openr,1,file,/swap_end,comp=icomp

; READ IMAGE IDENTIFICATION RECORD
point_lun,1,0*8000
readu,1,IIDR
i96 = 0
if (iidr.year ge 1996) then i96 = 1
if (i96) then begin
    point_lun,1,0*8000
    readu,1,iidr96
    iidr=iidr96
endif

; READ LOCATION GRID RECORD
point_lun,1,1*8000
readu,1,LGR

; READ IMAGE CALIBRATION RECORDS
nchan = iidr.nchan
if (nchan lt 1) then begin
    image = -1
    nchan = 0
    goto,endd
endif
CalRec = replicate (ICR,nchan)
for i=0,nchan-1 do begin
    point_lun,1,(2l+i)*8000
    readu,1,ICR
    CalRec(i) = ICR
endfor
close,1

;-----------------------------------------------------
;set up image variables
nrecords = iidr.nrecords
nchan = iidr.nchan
nx = iidr.npixels
ny = iidr.nlines
if (ioutput) then print,'NREC,NCHAN,NX,NY:',nrecords,nchan,nx,ny

image = bytarr(nchan,nx,ny)
lat = fltarr(nx,ny)
lat(*) = -900.
lon = lat
mu=lat
mun=lat
azi=lat

coverage = reform(lgr.npix,18,36)

;-----------------------------------------------------
openr,1,file ,/swap_end,comp=icomp
;LOOP over Data Records
for irec=0l,nrecords-1 do begin
    point_lun,1,(2l+nchan+irec)*8000l

    readu,1,rid
    begscan = rid.scan_beg
    endscan = rid.scan_end
    sld.pointer = 0
    for scanline=begscan,endscan do begin
;the following line allows the reader to work for all files EXCEPT
;  GOES-8/AES
        if (sld.pointer ne 0) then begin
            if (ff(8) eq 'AES' and ff(3) eq 'GOE-8') then begin
                point_lun,1,(2l+nchan+irec)*8000l+(sld.pointer)*2-2 
            endif else begin
                point_lun,1,(2l+nchan+irec)*8000l+(sld.pointer-1)*2
            endelse
        endif else begin
            if (scanline ne begscan) then stop
        endelse

        readu,1,sld
        ;The following test failed for MET-8 files:
        ;ISCCP.B3.1.MET-8.2006.08.28.2100.EUM.gz
        ;Data Out of Range
        ;3      10      32       4       3       1       0
        ;so bumping up the limits on sld.n_*
        ;originally, all were limited to < 30 except azi < 100
        ;print,sld.n_lon
        if (sld.n_lat lt 0 or sld.n_lat gt 30 or $
            sld.n_lon lt 0 or sld.n_lon gt 50 or $
            sld.n_mu  lt 0 or sld.n_mu  gt 30 or $
            sld.n_mun lt 0 or sld.n_mun gt 30 or $
            sld.n_azi lt 0 or sld.n_azi gt 100) then begin
                print,'Data Out of Range'
               print,sld.n_data,sld.n_lat,sld.n_lon,sld.n_mu,sld.n_mun,sld.n_azi,sld.quality
               ;goto,nextfile
                point_lun,1,(4l+irec)*8000l
               b = bytarr(40000l)
               readu,1,b
               ;goto,nextfile
               stop
        endif
    ; READ Navigation Directory
        if (sld.n_lat gt 0) then begin
            latnav = replicate(nd,sld.n_lat)
            readu,1,latnav
            for j=0,sld.n_lat-1 do begin
                j1 = latnav(j).beg_pix-1
                j2 = (latnav(j).end_pix-1) < (nx-1)
                if (j1 le (nx-1)) then begin
                    n = j2-j1+1
                    l = lindgen(n)+1
                    lat0 = latnav(j).fo + l*latnav(j).d1 + l*(l-1)*latnav(j).d2/2.
                    lat(j1:j2,scanline-1) = lat0 / 2.^22
                endif
            endfor
        endif
        if (sld.n_lon gt 0) then begin
            lonnav = replicate(nd,sld.n_lon)
            readu,1,lonnav
            for j=0,sld.n_lon-1 do begin
                j1 = lonnav(j).beg_pix-1
                j2 = (lonnav(j).end_pix-1) < (nx-1)
                if (j1 le (nx-1)) then begin
                    n = j2-j1+1
                    l = lindgen(n)+1
                    lat0 = lonnav(j).fo + l*lonnav(j).d1 + l*(l-1)*lonnav(j).d2/2.
                    lon(j1:j2,scanline-1) = lat0 / 2.^22
                endif
            endfor
        endif
        if (sld.n_mu gt 0) then begin
            munav = replicate(nd,sld.n_mu)
            readu,1,munav
            for j=0,sld.n_mu-1 do begin
                j1 = munav(j).beg_pix-1
                j2 = (munav(j).end_pix-1) < (nx-1)
                if (j1 le (nx-1)) then begin
                    n = j2-j1+1
                    l = lindgen(n)+1
                    lat0 = munav(j).fo + l*munav(j).d1 + l*(l-1)*munav(j).d2/2.
                    mu(j1:j2,scanline-1) = lat0 / 2.^22
          endif
            endfor
        endif
        if (sld.n_mun gt 0) then begin
            munnav = replicate(nd,sld.n_mun)
            readu,1,munnav
            for j=0,sld.n_mun-1 do begin
                j1 = munnav(j).beg_pix-1
                j2 = (munnav(j).end_pix-1) < (nx-1)
                if (j1 le (nx-1)) then begin
                    n = j2-j1+1
                    l = lindgen(n)+1
                    lat0 = munnav(j).fo + l*munnav(j).d1 + l*(l-1)*munnav(j).d2/2.
                    mun(j1:j2,scanline-1) = lat0 / 2.^22
                endif
            endfor
        endif
        if (sld.n_azi gt 0) then begin
            azinav = replicate(nd,sld.n_azi)
            readu,1,azinav
            for j=0,sld.n_azi-1 do begin
                j1 = azinav(j).beg_pix-1
                j2 = (azinav(j).end_pix-1) < (nx-1)
                if (j1 le (nx-1)) then begin
                    n = j2-j1+1
                   l = lindgen(n)+1
                    lat0 = azinav(j).fo + l*azinav(j).d1 + l*(l-1)*azinav(j).d2/2.
                    azi(j1:j2,scanline-1) = lat0 / 2.^22
          endif
            endfor
        endif
        if (sld.quality eq 0) then begin
            datadir = replicate(dd,sld.n_data)
            readu,1,datadir
            i1 = 0
            for i=0,sld.n_data-1 do begin
               rad = bytarr(datadir(i).nbytes,datadir(i).npix)
               i2 = (i1 + datadir(i).npix-1) < (nx-1)
               readu,1,rad
               image(*,i1:i2,scanline-1) = rad(*,0:i2-i1)
               i1 = i2
            endfor
        endif
noread:
    endfor
endfor
goto,OK
nextfile:
print,'********* ERROR *********'
OK:
close,1

endfor
endd:
l6000:

close,1

end
;***
