;****I* idl/idl.bcd_time
;+
; NAME:
;    BCD_TIME
;
; PURPOSE:
;    Convert time in BCD format (encoded in binary data) to a more
;    conventional format.
;
; CATEGORY:
;    GOES
;
; INPUTS:
;    8 bytes of data (can be 8 BYTEs, 4 INTs, 2 LONGs or 2 FLOATs
;
; OUTPUTS:
;    YEAR (YYYY)
;    JDAY (day of year)
;    Hour (HH)
;    Minute (MM)
;    Seconds
;    # of milliseconds (the fraction of a second *1000)
;
; USES
; * Nothing.
; 
; USED BY
; * b1read_aes_g8.pro
; * b1read_csu_g8.pro
; * b1read_csu_g91012.pro
; * b1read_noa_goe.pro
; * b1u_imc.pro
; 
; 
; MODIFICATION HISTORY:
;    $Id: bcd_time.pro 22775 2010-09-16 12:30:21Z kknapp $
;-
; SOURCE

pro bcd_time,time,year,jday,hh,mm,sec,msec
time = byte(time,0,8)
yy1 = time(3)/16
yy2 = time(3) mod 16
yy3 = time(2)/16
yy4 = time(2) mod 16
year = (yy1*1000l + yy2*100l + yy3*10l + yy4) mod 10000
jd1 = time(1) / 16
jd2 = time(1) mod 16
jd3 = time(0) / 16
jday = jd1*100 + jd2*10 + jd3

hh1 = time(0) mod 16
hh2 = time(7) / 16
mm1 = time(7) mod 16
mm2 = time(6) / 16
ss1 = time(6) mod 16
ss2 = time(5) / 16
ms1 = time(5) mod 16
ms2 = time(4) / 16
ms3 = time(4) mod 16

hh = hh1*10 + hh2
mm = mm1*10 + mm2
sec = ss1*10 + ss2
msec = ms1*100 + ms2*10 + ms3
end
;***
