;****I* idl/idl.cal_b1u_monthly
; NAME
; cal_b1u_monthly
; 
; PURPOSE
; To compile matchups of HIRS/ISCCP B1U over many files.
; 
; USAGE
; INPUT
; OUTPUT
; RESULT
; EXAMPLE
; USES
; * flines.pro
; * hirs_b1u_month.pro
; * hirs_irwvp_correction.pro
; * itime.pro
; * kploterror.pro
; * lr.pro
; 
; USED BY
; * cal_b1u_correct.pro
; 
; NOTES
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; ???
; HISTORY
; SOURCE

pro cal_b1u_monthly,part, $
                    iplot=iplot, $
                    iwu=iwu, $
                    correct=correct, $
                    debug=debug
if n_elements(part) eq 0 then part='N12.GOE-8.1998.08'
if n_elements(iwu) eq 0 then iwu = 0 
;get list of possible matchups

if keyword_set(correct) then begin
    ;read in correction file
    s=''
    cfile='/net/glaze/home/kknapp/idl/isccp/calib/b1u.calibration.correction.v2'
    
    nl=flines(cfile)-1
    cor_satmon=strarr(nl)
    cor_slop=fltarr(nl)
    cor_yint=fltarr(nl)
    openr,lun,cfile,/get_lun
    readf,lun,s                 ;version header
    for i=0,nl-1 do begin
        readf,lun,s
        t=strsplit(s,' ',/extr)
        cor_satmon(i) = t(0)
        cor_slop(i) = t(1)
        cor_yint(i) = t(2)
    endfor
    close,lun
    free_lun,lun
endif 

correct_slop = 1.0
correct_yint = 0.0
if keyword_set(correct) then begin
    ;select the satmon
    p=strsplit(part,'.',/extr)
    if (p(1) eq 'GOES9') then p(1) = 'GOE-9'
    satmon=strjoin(p(1:3),'.')
    q=where(satmon eq cor_satmon,nq)
    if (nq eq 0) then begin
        print,'Error: Satellite-Month not found in the correction'
        stop
    endif else if (nq gt 1) then begin
        print,'Error: Too many satellite months found.'
        stop
    endif else begin
        correct_slop = cor_slop(q(0))
        correct_yint = cor_yint(q(0))
    endelse
endif


print,'Part=',part
print,'IWU =',strtrim(iwu,2)
if keyword_set(correct) then print,'Correcting Calibration!!'
;settings
maxdtemp = 1.                   ;deg Celsius
maxdtime = 15.                  ;minutes
maxdmu = 0.05                   ;in mu: 1./cos(VZA)
maxdazi= 30.                    ;degrees
ofile=  '/net/glaze/raid1a/raid/nb/kknapp/data/isccp/b1ucal/monthly/Proportionate/'+part+'.stat'
ofilell='/net/glaze/raid1a/raid/nb/kknapp/data/isccp/b1ucal/monthly/Proportionate/'+part+'.latlon'

;calibrate a month

p = strsplit(part,'.',/extr)
noaasat = p(0)

; The first test is to reduce difference in time. Initially, had
; dtime=120 which produced a valley for most times with a minimum in
; error centered on 0 min ... however, some times still did not
; produce the valley. Will limit to 60 minutes to see what other error
; limits to include.

;Set up limits for passing to HIRS_B1U_MONTH
if (iwu eq 1) then begin
    maxdmu   =  0.05
    maxdtime = 15.
    maxdazi  = 30.
endif else begin
    maxdmu   =   0.1
    maxdtime =  30.
    maxdazi  = 200.

;for the test set up
    maxdmu   =   0.05
    maxdtime =  30.
    maxdazi  =  200.
endelse


hirs_b1u_month,part, $
               output=output,ichan=chan, $
               dtime=maxdtime,maxdazi=maxdazi,maxdmu=maxdmu
if n_elements(output) ne 0 then  begin
match=output.matches
ch=where(chan ge 0,nch)

openw,lun,ofile,/get_lun
;for each B1U channel present
print,nch
for ich=0L,nch-1 do begin
    limits=0
    hch = ch(ich)               ;hirs channel
    bch = ich                   ;b1u channel
    hirs=reform(match.hdata(0,hch,*)/100.+50.) ;output.hirs(*)
    b1ut=reform(match.fov(  3,bch,*)/100.+50.) ;output.b1ut(*)
b1ut = b1ut * correct_slop + correct_yint
    hvza = reform(match.hangle(2,*)) / 100.
    hazi = reform(match.hangle(3,*)) / 100.
    bvza = reform(match.b1angle(0,*))/ 100.
    bazi = reform(match.b1angle(1,*))/ 100.
    dazi = abs(hazi - bazi)
    hmu = 1./ cos(hvza*!pi/180.)
    gmu = 1./ cos(bvza*!pi/180.)
    dhirs= reform(match.hdata(2,hch,*)) / 100.
    db1u = reform(match.fov(  4,bch,*)) / 100.
    out = where(dazi gt 180.,nout)
    if (nout gt 0) then dazi(out) = 360. - dazi(out)
    dmu = abs(hmu-gmu)
    dtime = abs(match.time)

;    q=where(dhirs lt 0,nq)
;    if (nq gt 0) then dhirs(q) = dhirs(q) + 327.67*2.
;    dhirs = sqrt(dhirs)



;Wu limits
    if (iwu eq 1) then begin
        maxdtemp =  1.
        limit = where(dhirs gt 0. and                       $
                      db1u  gt 0. and db1u  lt maxdtemp and $
                      dmu   lt maxdmu and $
                      dtime lt maxdtime and $
                      (  (hvza lt 5. and bvza lt 5.) or $
                         ((hvza ge 5. or  bvza ge 5.) and dazi lt maxdazi )),nlimit)
        if (nlimit lt 20) then begin
            print,ich,nlimit
            goto,nextchannel
        endif
    endif else begin
        ;PROPORTIONATE FILTERING
        cost = db1u^2 + dhirs^2
;collect the best 10% of points for each temp range
        dtemp = 10.
        mint=100
        maxt=350.
        nt = round((maxt-mint)/dtemp) + 1
        frac=0.1
        for i=0,nt-1 do begin
            low=mint+dtemp*i
            upp=mint+dtemp*(i+1)
            wi=where(b1ut ge low and b1ut lt upp and $
                     db1u  gt 0 and db1u  lt 50 and $
                     dhirs gt 0 and dhirs lt 50 and $
                     dmu lt maxdmu and $
;                 gmu gt hmu and $
                     dtime lt maxdtime,nwi)
;print,low,upp,nwi
            if (nwi gt 100) then begin
                s = sort(cost(wi))
            ;print,low,upp,nwi,cost(wi(s(frac*nwi)))
                if n_elements(limits) lt 2 then begin
                    limits = (wi(s(0:nwi*frac)))
                endif else begin
                    limits = [limits,(wi(s(0:nwi*frac)))]
                endelse
            endif
        endfor
        limit=limits
        if n_elements(limits) lt 100 then goto,nextchannel
    endelse

    ;analyze results
    hirs=match.hdata(0,hch,limit)/100.+50. ;output.hirs(limit)

;   NORMALIZE HIRS IRWVP OBS TO NOAA-14
    if (hch eq 1) then begin
        hirs = hirs_irwvp_correction(noaasat,hirs)
    endif

    b1ut=match.fov(  3,bch,limit)/100.+50. ;output.b1ut(limit)
    b1ut = b1ut * correct_slop + correct_yint
    bnum=match.fov(  0,bch,limit)

    lat=output.matches.lat(limit)/100. ;output.lat(limit)
    lon=output.matches.lon(limit)/100. ;output.lon(limit)
    match=output.matches
    ltime=output.ltimes(limit)

    dtime=output.matches.time(limit)
    dvza = abs(output.matches.hangle(2,limit)-output.matches.b1angle(0,limit))/100.
    dazi = abs(output.matches.hangle(3,limit)-output.matches.b1angle(1,limit))/100.
    hnoise = output.matches.hdata(2,hch,limit)/100.
    bnoise = output.matches.fov(  4,bch,limit)/100.
;convert local time to synoptic time
    stime = round(ltime/3.) mod 8
    x=findgen(1000)

;itime
    itime = 0
    if (itime eq 1) then begin
        for i=0,7 do begin
            use = where(stime eq i,nn)
            if (nn gt 0) then begin
                x = b1ut(use)
                y = b1ut(use) - hirs(use)
                st=lr(b1ut(use),hirs(use))
                wi = where(abs(b1ut(use)*st(2)+st(3)-hirs(use)) lt st(4)*3,nwi)
                if keyword_set(iplot) then begin 
                    window,i,xs=800,ys=600,xpos=800,ypos=100
                    !p.multi=[0,3,2]
                    plot,b1ut(use),hirs(use),psym=1,/yno,title='Time='+strtrim(i*3,2)+'UTC', $
                         chars=2,/nodata,xtitle='B1U',ytitle='HIRS'
                    oplot,b1ut(use),hirs(use),psym=3,col=170
                    oplot,x,x*st(2)+st(3),col=170
                    oplot,b1ut(use(wi)),hirs(use(wi)),psym=3,col=85
                endif
                                ;remove outliers
                stf=lr(b1ut(use(wi)),hirs(use(wi)))
                print,st([0,1,2,3,4,6])
                
;                printf,11,part,i*3,st,format='(a,1x,i2,11f)'
                
                
                if keyword_set(iplot) then begin 
                    oplot,x,x*st(2)+st(3),col=85
                    err = b1ut(use)*st(2)+st(3)-hirs(use)
                    kploterror,dtime(use),err ,xtitle='Time',chars=2
                    kploterror,dvza(use),err, $
                               xtitle='dVZA',chars=2
                    kploterror,dazi(use),err, $
                               xtitle='dAZI',chars=2
                    kploterror,hnoise(use),err, $
                               xtitle='HIRS Spat Std Dev',chars=2
                    kploterror,bnoise(use),err, $
                               xtitle='B1U Spat Std Dev',chars=2
                endif
            endif 
        endfor
    endif else begin
        use = where(stime ge 0,nn)
        if (nn gt 0) then begin
            x = b1ut(use)
            y = hirs(use)
            st=lr(x,y)
            wi = where(abs(b1ut(use)*st(2)+st(3)-hirs(use)) lt st(4)*3,nwi)
            if keyword_set(iplot) then begin 
                loadct,38,/sil
                window,xs=800,ys=600,xpos=800,ypos=100,/free
                !p.multi=[0,3,2]
                plot,b1ut(use),hirs(use),psym=1,/yno, $
                     chars=2,/nodata,xtitle='B1U',ytitle='HIRS'
                oplot,b1ut(use),hirs(use),psym=3,col=170
                oplot,x,x*st(2)+st(3),col=170
                oplot,b1ut(use(wi)),hirs(use(wi)),psym=3,col=85
            endif
            ;remove outliers
            ;st=lr(b1ut(use(wi)),hirs(use(wi)))
            print,st([0,1,2,3,4,6])
            printf,lun,part,hch,st,format='(a,1x,i2,f12.1,2f12.6,f12.3,f12.4,2f12.3,f14.2,3f12.5,10f12.5)'

            ;output lat/lon points
            openw,unit,ofilell+'.'+strtrim(hch,2),/get_lun
            writeu,unit,fix(lat(use)*100.), $
                        fix(lon(use)*100.)
            close,unit
            free_lun,unit

            if keyword_set(iplot) then begin 
                oplot,x,x*st(2)+st(3),col=85
                err = b1ut(use)*st(2)+st(3)-hirs(use)
                kploterror,dtime(use),err ,xtitle='Time',chars=2
                kploterror,dvza(use),err, $
                           xtitle='dVZA',chars=2
                kploterror,dazi(use),err, $
                           xtitle='dAZI',chars=2
                kploterror,hnoise(use),err, $
                           xtitle='HIRS Spat Std Dev',chars=2
                kploterror,bnoise(use),err, $
                           xtitle='B1U Spat Std Dev',chars=2
            endif
        endif 
    endelse 
nextchannel:
endfor 

;add information to the output regarding limits

printf,lun,maxdtemp,maxdtime,maxdmu,maxdazi
close,lun
free_lun,lun


if keyword_set(debug) then stop
;output calibration adjustment
endif
end
;***
