;+
; NAME:
;    Cheb2Cart
;
; PURPOSE:
;    Calculate a cartesian coordinate given Chebyshev polynomial.
;
; CATEGORY:
;    Navigate
;
; INPUTS:
;    cheb - DBLARR(11) - Chebyshev polynomials
;    time - Array/Scalar - Time(s) of requested positions (seconds)
;
; OUTPUTS:
;    parm - The resulting cartesian location
; MODIFICATION HISTORY:
;    $Id: cheb2cart.pro 22776 2010-09-16 12:34:07Z kknapp $
;
; USES
; * Nothing (aside from IDL internals)
; 
; USED BY
; * navcheck.pro
; * b1u_ij2ll.pro
; * b1u_ll2ij.pro
; 
;-
; SOURCE

pro cheb2cart,cheb,time,parm

;The normalization of time is based on 13 hours.
D = 13.d0 * 3600.d0

siz=size(time)
npoints = n_elements(time)
itime = reform([time],npoints)

u = 2.d0 * itime / d -1.d0
b = dblarr(13,npoints)
for i=10,0,-1 do begin
    b(i,*) = 2.d0 * u * b(i+1,*) - b(i+2,*) + cheb(i)
endfor
parm = 0.5d0 * (b(0,*) - b(2,*))
parm = reform(parm)
if (siz(0) gt 0) then parm = reform(parm,siz(1:siz(0)))

end
