function fitcma,x,p,bias=bias,fstdev=fstdev,iout=iout, $
                debug=debug
common STR,timstr,orbstr,attstr,imgstr
common grid,gridi,gridj ;,gmsflag

if keyword_set(debug) then iout=1

if keyword_set(iout) then begin 
    print,x,p
    help,attstr,/str
    help,orbstr,/str
    help,imgstr,/str
    help,TIMstr,/str
endif 
;print,p(8)
timstr.spra1      = p(0)
timstr.spra2      = p(0)
timstr.spdc1      = p(1)
timstr.spdc2      = p(1)
attstr.roll      = p(2)
attstr.pitch     = p(3)
attstr.yaw       = p(4)
orbstr.semima    = p(5)
orbstr.incl      = p(6)
orbstr.eccen     = p(7)
orbstr.meana     = p(8)
orbstr.rgtasc    = p(9)
orbstr.perigee   = p(10)
imgstr.cen_lin   = p(11)
imgstr.cen_pix   = p(12)
imgstr.linang    = p(13)
imgstr.pixang    = p(14)
;timstr.spra2      = p(15)
;timstr.spdc2      = p(16)
orbstr.sper      = p(17)

latstart  = x(0)
lonstart  = x(1)


dlat = -5
dlon = 5.
nlat = 25
nlon = 25
di = dblarr(25,25)
dj = dblarr(25,25)
lt1 = di
ln1 = di

lat = transpose(rebin(reform(findgen(nlat)*dlat + latstart,nlat,1),nlat,nlon))
lon = transpose(rebin(reform(findgen(nlon)*dlon + lonstart,1,nlon),nlat,nlon))

b1u_ll2ij,lat,lon,iv,jv,b1TIMstr=TIMstr,b1ORBstr=ORBstr, $
          b1IMGstr=IMGstr,b1ATTstr=ATTstr, $
          glat=glat,glon=glon,grad=grad,/sat

di = transpose(iv-gridi)
dj = transpose(jv-gridj)

if keyword_set(IOUT) then begin
    print,total(lat),total(lon),total(iv),total(jv)
    help,di,dj
    print,'grd',gridi(3,4),gridj(3,4)
    print,'err',di(3,4),dj(3,4)
    print,'l,l',lat(3,4),lon(3,4)
    print,'i,j',iv(3,4),jv(3,4)
    print,'sum',total(di),total(dj)
    print,total(di*di),total(dj*dj)
endif 

if (arg_present(bias)) then bias = [mean(di),mean(dj)]
if (arg_present(fstdev)) then fstdev=[stdev(di),stdev(dj)]

if keyword_set(debug) then stop

;b1u_ll2ij,lat(0),lon(0),iv,jv,b1TIMstr=TIMstr,b1ORBstr=ORBstr, $
;          b1IMGstr=IMGstr,b1ATTstr=ATTstr, $
;          glat=glat,glon=glon,grad=grad,/sat,/iout

return,[di,dj]
end
;================================================================
;================================================================
;================================================================
function kenfitc,x,answer,parinfo=parinfo,parname=parname, $
                result=result,debug=debug
inx=x
itime = systime(/seconds)

iscore = total(fitcma(x,parinfo.value,debug=debug)^2)

np = n_elements(parinfo)
parf = where(parinfo.fixed eq 0)
npf = n_elements(parf)
nruns = 0

iprint = 0
iquiet = 0
if keyword_set(debug) then iprint=1

;variables which will be adjusted by the algorithm
result = result ;parinfo.value
step1 = parinfo.step

;count the number of decreases in the step size
dstep = intarr(npf)
dstepn= intarr(npf)
pruns = intarr(npf)

rerun:

step1 = parinfo.step

;loop over variables
for ivar=0,npf-1 do begin
    ip = parf(ivar)
    ;calculate the score triplets
    pstart = result(ip)
rescore:
    newscore = dblarr(3)
    step = step1(ip)
    p = result
    param = result(ip)

    pp = [param - step,param,param + step]

if (iprint) then print,parname(ip)
if (iprint) then print,pp
    ;check bounds as needed
    if (parinfo(ip).limited(0) eq 1) then begin
        if (pp(0) lt parinfo(ip).limits(0)) then begin
            pp(0) = parinfo(ip).limits(0)
            pp(1) = (pp(0)+pp(2))/2.
            step1(ip) = (pp(2)-pp(0))/3.
        endif else if (pp(2) gt parinfo(ip).limits(1)) then begin
            pp(2) = parinfo(ip).limits(1)
            pp(1) = (pp(0)+pp(2))/2.
            step1(ip) = (pp(2)-pp(0))/3.
        endif
    endif
    pruns(ivar)++
    pruns(ivar)++
    pruns(ivar)++

    x=inx
    p(ip) = pp(0)
    newscore(0) = total(fitcma(x,p)^2)
    p(ip) = pp(1)
    newscore(1) = total(fitcma(x,p)^2)
    p(ip) = pp(2)
    newscore(2) = total(fitcma(x,p)^2)

    mxs = max(newscore)
    mns = min(newscore)

;    if (iprint) then print,float([newscore,((mxs-mns)/median(newscore))])

    if (iprint) then print,newscore

    ;interpolate using the quadratic
    y = newscore
    a1 = y(0) / ((pp(0)-pp(1))*(pp(0)-pp(2)))
    a2 = y(1) / ((pp(1)-pp(0))*(pp(1)-pp(2)))
    a3 = y(2) / ((pp(2)-pp(0))*(pp(2)-pp(1)))
    a = a1 + a2 + a3
    b = - (a1*(pp(1)+pp(2)) + a2*(pp(0)+pp(2)) + a3*(pp(0)+pp(1)))
;   if a>0, then parabola has no minimum ... in this case, head in the
;   direction of the minimum
    dx = pp(1)-pp(0)
    if (a lt 0) then begin
        new = pp(1) + 10. * dx * sign(y(0)-y(1))
    endif else begin
        new = -b / (2.*a)
        dn = new - pp(1)
        if (abs(dn) gt (10.*dx)) then begin
            new = pp(1) + 10*dx*sign(dn)
        endif
    endelse
    

    if (((mxs-mns)/median(newscore)) gt 0.0001) then begin
        if (newscore(0) eq mns) then begin
            result(ip) = new 
            if (iprint) then print,parname(ip),result(ip),newscore(0)
            if (parinfo(ip).limited(0) eq 0 or $
                (parinfo(ip).limited(0) eq 1 and $
                 parinfo(ip).limits(0) lt new)) then goto,rescore else begin
                if (iprint) then print,'PEGGED: ',parname(ip),pp(0)
                result(ip) = parinfo(ip).limits(0)
            endelse
        endif else if (newscore(2) eq mns) then begin
            result(ip) = new 
            if (iprint) then print,parname(ip),result(ip),newscore(2)
            if (parinfo(ip).limited(0) eq 0 or $
                (parinfo(ip).limited(0) eq 1 and $
                 parinfo(ip).limits(1) gt new)) then goto,rescore else begin
                if (iprint) then print,'PEGGED: ',parname(ip),pp(2)
                result(ip) = parinfo(ip).limits(1)
            endelse
        endif else begin
            result(ip) = new 
            step1(ip) = step1(ip) * 0.1
            dstep(ivar)++
            dstepn(ivar)++
            if (iprint) then print,'STEP ',parname(ip),step1(ip)
;            goto,rescore

            ; Use brent function to get minumum
            ; This should make fewer function calls

            ax = pp(0)
            bx = pp(1)
            cx = pp(2)
            tol = 1.e-15

            input = p
            output = x
            if (iprint) then print,'Brent: ',parname(ip)
; SNIP <<<<<< FROM idl/libs/nr/brent.pro <<<<<<<<<<<<<<<<<<<
; a and b must be in ascending order, though the input abcissas need
; not be
            itmax = 100
            cgold = .3819660d0
            zeps = 1.d-15
            
            a = ax < cx
            b = ax > cx
            
; Initilizations
            v = bx
            w = v
            x = v
; This will be the distance moved on the step before last
            e = 0.
;fx = call_function(F,x)
            input(ip) = x
    pruns(ivar)++
            fx = total(fitcma(output,input)^2)
            
            fv = fx
            fw = fx
            
; Main program loop
            for iter=0,itmax do begin
if (iprint) then print,'ITER:',iter
if (iprint) then print,'bound:',a,b
if (iprint) then print,'x:   ',x,w,v
if (iprint) then print,'f(x):',fx,fw,fv
                xm = 0.5*(a+b)
                tol1 = tol*abs(x)+zeps
                tol2 = 2. * tol1
                                ;test for done here
                if (iter gt 0 and $
                    abs(x-xm) le (tol2-0.5*(b-a))) then goto, three

                ;A TEST FROM KEN
                if (iter gt 1 and $
                    ((fw-fx)/fx) lt 0.0001) then goto,three
                                ;construct a trial parabolic fit
                if (abs(e) gt tol1) then begin
                    r = (x-w)*(fx-fv)
                    q = (x-v)*(fx-fw)
                    p = (x-v)*q - (x-w)*r
                    q = 2.*(q-r)
                    if (q gt 0) then p = -p
                    q = abs(q)
                    etemp = e
                    e = d
                    if (abs(p) ge abs(0.5*q*etemp) or $
                        p le (q*(a-x)) or $
                        p ge (q*(b-x))) then goto,one
                                ;the above conditions determine the
                                ;acceptability of the parabolic fit. Here, it is OK
                                ;Take the parabolic step
                    d = p/q
                    u = x+d
if (iprint) then print,'NewU',u
                    if ((u-a) lt tol2 or $
                        (b-u) lt (tol2)) then d = abs(tol1) * (((xm-x) ge 0.) - ((xm-x) lt 0))
                    goto,two
                endif
one:
;   We arrive here for a golden section step, which we take into the
;   larger of the two segments.
                if (x ge xm) then e=a-x else e=b-x
;   take the golden step
                d=cgold*e
two:
;   Arrive here with d computed either from parabolic fit or else from
;   golden section
                if (abs(d) ge tol1) then u=x+d else u = x + abs(tol1)*((d ge 0)-(d lt 0))
;   This is the one function call per iteration
;    fu = call_function(F,u)
                input(ip) = u
    pruns(ivar)++
                fu = total(fitcma(output,input)^2)
                if (iprint) then print,'New eval:',u,fu
;   and now we have to decide what to do with our function
;   evaluation. Housekeeping follows:
;stop
                if (fu le fx) then begin
                    if (u ge x) then a=x else b=x
                    v = w
                    fv = fw
                    w = x
                    fw = fx
                    x = u
                    fx = fu
                endif else begin
                    if (u lt x) then a=u else b=u
                    if (fu le fw or w eq x) then begin
                        v = w
                        fv = fw
                        w = u
                        fw = fu
                    endif else if (fu le fv or v eq x or v eq w) then begin
                        v = u
                        fv = fu
                    endif
                endelse         ;done with housekeeping
            endfor
            if (iprint) then print,'Brent exceeded maximum iterations'
three:
; Arrive here ready to exit with best values.
            xmin=x
            newscore(1) = fx
            result(ip) = xmin
            
            if (iprint) then print,'Iterations:',iter
;            break
; END SNIP >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
nobrent:
            
        endelse
    endif
endfor
delta = abs((newscore(1)-iscore)/iscore)
delta = iscore - newscore(1)
if (delta gt 1.) then begin
    nruns++
    iscore=newscore(1)
;    print,nruns,'=======',newscore(1)
    if (nruns lt 1000) then goto,rerun
endif
;print,total(pruns)
;print,'Time:',systime(/sec)-itime
;print,'Min score:',newscore(1)

x = inx
return,1
end
;================================================================
;================================================================
;================================================================
;****I* idl/idl.cma_besfit
; NAME
; cma_bestfit
; 
; PURPOSE
; To help navigate CMA files which have (to put it mildly) strange
; navigation constants. This algorithm minimizes the navigation error
; by adjusting the roll/pitch/yaw as well as other parameters.
; 
; USAGE
; INPUT
; OUTPUT
; RESULT
; EXAMPLE
; USES
; * b1u_ll2ij.pro
; * itime.pro
; * mean.pro
; * mpfit.pro
; * mpfitfun.pro
; * remove_gz.pro
; * sign.pro
; * stdev.pro
; * stripname.pro
; * yymmdd_2_yyjjj.pro
; 
; 
; USED BY
; * b1read_cma_fy2.pro
; 
; NOTES
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; ?????
; 
; HISTORY
; 
; SOURCE

pro cma_bestfit,file=file, $
                IMGstr_=IMGstr_, $
                ATTstr_=ATTstr_, $
                ORBstr_=ORBstr_, $
                TIMstr_=TIMstr_, $
                PT=PT, $
                bias=bias, $
                fstdev=fstdev, $
                cost=cost, $
                RERUN=RERUN, $
                debug=debug

common grid,gridi,gridj
common STR,timstr,orbstr,attstr,imgstr
;this routine derives the navigation parameters for CMA imagery
;
; SUCCESSFULLY TESTED ON ...
;             STD DEV ERRORS ->  DI   DJ
;       ... ISCCP-test/*GOES9*  ~0.3  ~1.1 itle = 1
;       ... ISCCP-test/*GMS-5*  ~0.5  ~1.3 itle = 1
;       ... ISCCP-test/*GMS-4*  ~3.4  ~1.3 itle = 0
;       ... ISCCP-test/*GMS-3*  ~4.8  ~1.6 itle = 0
;       ... ISCCP-test/*GMS-2*  ~4.8  ~1.4 itle = 0
;       ... ISCCP-test/*GMS-1*  ~4.4  ~1.4 itle = 0
;
;   The ~6 pixel error is likely due to a small bias in the pixang
;   value. Otherwise, the calculations are excellent!
rad = 180. / !pi

VERSION = 1


; Get the parameters from the "b1kep" file ... if it exists.
b1kfile = file + '.b1kep'

; Get the parameters from the "b1kep" file ... if it exists.
;check for compression
f=strsplit(file,'.',/extr)
n=n_elements(f) 
b1kfile = remove_gz(file) + '.b1kep'
if (file_test(b1kfile)) then begin
    print,'Found:'+stripname(b1kfile)
    icomp=0
endif else begin
    if (file_test(b1kfile+'.gz')) then begin
        b1kfile=b1kfile+'.gz'
        icomp=1
        print,'Found:'+stripname(b1kfile)
    endif else begin
        ;can't find a b1kep file
        goto,file_error
    endelse
endelse

on_ioerror,file_error


inbias = fltarr(2)
instdev =fltarr(2)
fversion = 0
oldscore = 0.
oldparam = dblarr(18)
; The version number is only incremented when a noticeable change in
; the parameter fitting is made. If the version in the b1kep file does
; not match this, then it will be rerun
openr,lun,b1kfile,/get,comp=icomp
readu,lun,fversion
readu,lun,inbias
readu,lun,instdev
readu,lun,oldscore
readu,lun,oldparam
close,lun
free_lun,lun
if (fversion eq version) then begin
    timstr_.spra1   = oldparam(0)
    timstr_.spdc1   = oldparam(1)
    timstr_.spra2   = oldparam(0)
    timstr_.spdc2   = oldparam(1)
    attstr_.roll    = oldparam(2)
    attstr_.pitch   = oldparam(3)
    attstr_.yaw     = oldparam(4)
    orbstr_.semima  = oldparam(5)
    orbstr_.incl    = oldparam(6)
    orbstr_.eccen   = oldparam(7)
    orbstr_.meana   = oldparam(8)
    orbstr_.rgtasc  = oldparam(9)
    orbstr_.perigee = oldparam(10)
;    imgstr_.cen_lin = oldparam(11)
;    imgstr_.cen_pix = oldparam(12)
;    imgstr_.linang  = oldparam(13)
;    imgstr_.pixang  = oldparam(14)
;    orbstr_.sper    = oldparam(17)
endif else goto,getkep
bias = inbias
fstdev = instdev
cost = oldscore

;Moved this RERUN here ... when it was above, the rerun was not using
;the precomputed values, but was starting over from scratch
if keyword_set(RERUN) then goto,getkep

return
; In the event the b1kep file is not found, then find best fit
; parameters and output them to a b1kep file.
file_error:
print,'Missing file:',stripname(b1kfile)

getkep:
on_ioerror,null
ORBstr = ORBstr_
ATTstr = ATTstr_
IMGstr = IMGstr_
TIMstr = TIMstr_

iprint = 0
if keyword_set(debug) then iprint=1
f = strsplit(file,'.',/extr)

sat = f(3)
cyr = f(4)


;   LOOP OVER OPTIONS
itle = 0
optiona = 2

; STARTING points for lat/lon grids
if (f(3) eq 'GOES9' or f(3) eq 'GOE-9') then begin
    GMSFLAG = 0
    latstart= 50.
    lonstart = 105.
endif else begin
    GMSFLAG=1
    latstart= 50.
    lonstart = 90.
endelse
dlat = -10.
dlon = 10.
nlat = 11
nlon = 11

if (f(3) eq 'FY2-C' or $
    f(3) eq 'FY2-E' or $
    strmid(f(3),0,3) eq 'FY2') then begin
    latstart = 60.0
    lonstart = 45.0
    nlat = 25
    nlon = 25
    dlat = -5.
    dlon = 5.
endif

x = [latstart,lonstart]

;set up the i/j test grids
gridi = reform(pt(1,*,*))
gridj = reform(pt(0,*,*))

; set up the initial guess for the parameters
parname = ['spra1  ','spdc1  ','roll   ','pitch  ','yaw    ', $
           'semima ','incl   ','eccen  ','meana  ','rgtasc ','perigee', $
           'cen_lin','cen_pix','linang ','pixang ','spra2  ','spdc2  ', $
           'sper   ']



param = [timstr.spra1,timstr.spdc1, $
         attstr.roll,attstr.pitch,attstr.yaw, $
         orbstr.semima,orbstr.incl,orbstr.eccen, $
         orbstr.meana,orbstr.rgtasc,orbstr.perigee, $
         imgstr.cen_lin,imgstr.cen_pix, $
         imgstr.linang,imgstr.pixang, $
         timstr.spra2,timstr.spdc2,orbstr.sper]
; SPARAM = starting parameters
sparam=param

; Calculate initial error
if (iprint) then print,'First call'
origdiff = total(fitcma(x,param)^2)
if (iprint) then print,'Original DIFF vec =',origdiff

; Set up the fitting parameters
parinfo = replicate({value:0.d, $
                     fixed:0, $
                     limited:[1,1], $
                     limits: [0.d,0.d], $
                     tied:'', $ $
                     step:0.d0,$
                     mpside:0},18)

;Attitude info
parinfo(0:1).fixed = 1
parinfo(0).step = 0.2d0
parinfo(1).step = 0.01d0
if (optiona ge 1) then parinfo(0:1).fixed = 0
parinfo(0).limited=[0,0]
parinfo(0).value =  param(0)
parinfo(1).value =  param(1)
parinfo(1).limited=[0,0]

parinfo(2:4).fixed = 0
parinfo(2:4).step = 0.05d0
parinfo(2).value =  param(2)
parinfo(2).limited=[0,0]
parinfo(3).value =  param(3)
parinfo(3).limited=[0,0]
parinfo(4).value =  param(4)
parinfo(4).limited=[0,0]

;Orbit info
parinfo(5:10).fixed = 1
if (optiona ge 2) then parinfo(5:10).fixed = 0
parinfo(5).value =  param(5)    ;  semima
parinfo(5).limits= [param(5)*.93,  param(5)*1.07]
parinfo(5).step = 10.d0
parinfo(6).value =  param(6)    ;  incl
parinfo(6).limits= [param(6)-5.,   param(6)+5.]
parinfo(6).step = 0.01
parinfo(7).value =  param(7)    ;  eccen
parinfo(7).limits= [param(7)*.0001,param(7)*10000.]
parinfo(7).step = param(7)
parinfo(8).value =  param(8)    ;  meana
parinfo(8).limits= [param(8)-360., param(8)+360.]
parinfo(8:10).step = 1.d-2
parinfo(9).value =  param(9)    ;  rgtasc
parinfo(9).limits= [param(9)-360., param(9)+360.]
parinfo(10).value=  param(10)   ;  perigee
parinfo(10).limits=[param(10)-360.,param(10)+360.]

;center of image
parinfo(11:12).fixed = 1
parinfo(11).value=  param(11)
parinfo(12).value=  param(12)
parinfo(11:12).limits = [0,1e10]

;angle of pixel
parinfo(13:14).fixed = 1
parinfo(13:14).limited=[1,1]
parinfo(13).value=  param(13)   ;linang
parinfo(13).limits=[param(13)*1.5,param(13)*0.5]
parinfo(14).value=  param(14)   ;pixang
parinfo(14).limits=[param(14)*1.5,param(14)*0.5]

;2nd spin axis parameters
parinfo(15:16).fixed   = 1
parinfo(15).tied = 'P(0)'       ;tied parameters are considered fixed
parinfo(16).tied = 'P(1)'
parinfo(15:16).limited = [0,0]
parinfo(15).value =  param(15)  ;timstr.spra2
parinfo(16).value =  param(16)  ;timstr.spdc2

parinfo(17).fixed=1
parinfo(17).limited=[0,0]
parinfo(17).value = param(17)
parinfo(17).limits = [param(17)*.0,param(17)*2]


goto,nompfit
; Get the best result from MPFIT
if (iprint) then itime = systime(/seconds)

result = mpfitfun('FITCMA', x, pt*0., pt*0.+ 1.d0, $
                  parinfo=parinfo,yfit=yfit,npegged=npegged, $
                  covar=covar,maxiter=500,quiet=1,bestnorm=score, $
                  niter=niter,status=stat,xTOL=1.d-20,ftol=1.d-20)

if (iprint) then print,'Time:',systime(/sec)-itime

; The best result from MPFIT is not ALWAYS the best fit. Thus, run
; KENFIT using the result from MPFIT. KENFIT is slower, but if MPFIT
; is the best, it wuickly outputs, otherwise, it searches for the best
; result. It iterates each parameter and cycles through until the
; difference between the reported and calculated grids is changes little.
if (iprint) then print,'MPFIT result: ',total(fitcma(x,result)^2)

; OUTPUT results as desired
if (iprint) then print,'     BEST = ',score
if (iprint) then print,'  NPEGGED = ',npegged
if (iprint) then print,'    NITER = ',niter
if (iprint) then print,'   STATUS = ',stat
if (npegged gt 0) then begin
    if (iprint) then print,where(result eq parinfo.limits(0))
    if (iprint) then print,where(result eq parinfo.limits(1))
endif 

if (iprint) then begin
    print,'NAME      STARTED       ENDED'
    for i=0,17 do begin
        if (parinfo(i).fixed eq 0) then $
          print,parname(i),param(i),result(i)
    endfor
endif
nompfit:
input = sparam
if (iprint) then print,'DIFF=',total(fitcma(x,input)^2)
resultk = kenfitc(x,pt*0,parinfo=parinfo,parname=parname,result=input,debug=debug)
cost = total(fitcma(x,input,bias=bias,fstdev=fstdev)^2)
;print,cost
;print,input
;print,bias,fstdev
if (iprint) then print,'KENFIT result:',cost
if (iprint) then begin
    print,'NAME      STARTED       ENDED'
    for i=0,17 do begin
        if (parinfo(i).fixed eq 0) then $
          print,parname(i),param(i),input(i)
    endfor
    print,'bias:   ',bias
    print,'std dev:',fstdev
    print,'avg cost',sqrt(cost/625.)
endif
if (iprint eq 0) then begin
;    print,sat,' ',cyr,' ',origdiff,total(fitcma(x,input)^2)
endif

TIMstr_ = TIMstr
ATTstr_ = ATTstr
ORBstr_ = ORBstr

openw,lun,b1kfile,/get,comp=icomp
writeu,lun,fix(version)
writeu,lun,float(bias)
writeu,lun,float(fstdev)
writeu,lun,float(cost)
writeu,lun,double(input)
close,lun
free_lun,lun
end
;***
