;****I* idl/idl.date_arr2iso
; NAME
; date_arr2iso
; 
; PURPOSE
; To convert date in a set of arrays  to ISO formated date.
; 
; The ISO format of date is:
;    YYYY-MM-DD HH:MM:SS.SSSS
; or
;    YYYY-MM-DDTHH:MM:SS.SSSS
; where the space between date and time can be replaced by "T".
; 
; USAGE
; * See idl.date_jul2iso
; 
; INPUT
; A 1-D or 2-D array :: time
; time(0,*) :: time(0) = year
; time(1,*) :: time(1) = month (1-12)
; time(2,*) :: time(2) = day of month (1-31)
; time(3,*) :: time(3) = hour of day (00-23)
; time(4,*) :: time(4) = minute (0-59)
; time(5,*) :: time(5) = second (0-59)
; 
;
; KEYWORDS
; withT - (optional) if present, the ISO date will use a "T" instead
;         of a space
; 
; OUTPUT
; * string (scalar or array) of a ISO formatted date
; 
; EXAMPLE
; USES
; * IDL internal function caldat
; 
; USED BY
; * date_jul2iso.pro
; 
; NOTES
; * While ranges above are stated, no tests are made here to ensure
;   that values are within the stated range
;   
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; HISTORY
; Jun 2010 - Added headers
; 
; SOURCE

function date_arr2iso,b, $
  withT=withT
space=' '
if keyword_set(withT) then space='T'
s=size(b)
if (s(0) eq 1) then begin
    str=string(b(0),format='(i4.4)')+'-'+ $
        string(b(1),format='(i2.2)')+'-'+ $
        string(b(2),format='(i2.2)')+space+ $
        string(b(3),format='(i2.2)')+':'+ $
        string(b(4),format='(i2.2)')+':'+ $
        string(b(5),format='(i2.2)')
endif else begin
    str=strarr(s(1))
    str(*) =              string(b(*,0),format='(i4.4)')
    str(*) = str(*) + '-'+string(b(*,1),format='(i2.2)')
    str(*) = str(*) + '-'+string(b(*,2),format='(i2.2)')
    str(*) = str(*)+space+string(b(*,3),format='(i2.2)')
    str(*) = str(*) + ':'+string(b(*,4),format='(i2.2)')
    str(*) = str(*) + ':'+string(b(*,5),format='(i2.2)')
endelse
return,str
end

;***
