;****I* idl/idl.date_jul2iso
; NAME
; date_jul2iso
; 
; PURPOSE
; To convert Julian date to ISO formated date.
; 
; The ISO format of date is:
;    YYYY-MM-DD HH:MM:SS.SSSS
; or
;    YYYY-MM-DDTHH:MM:SS.SSSS
; where the space between date and time can be replaced by "T".
; 
; USAGE
; IDL> isotime = date_jul2iso(julday(1,1,1980,12,45,35))
; 
; INPUT
; jdate - (scalar or array)
;
; KEYWORDS
; withT - (optional) if present, the ISO date will use a "T" instead
;         of a space
; 
; OUTPUT
; * string (scalar or array) of a ISO formatted date
; 
; EXAMPLE
; USES
; * idl.date_arr2iso
; * IDL internal function caldat
; 
; USED BY
; * b1uread.pro
; * b1u_2_grisat.pro
; 
; NOTES
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; HISTORY
; Jun 2010 - Added headers
; 
; SOURCE

function date_jul2iso,jdate, $
  withT=withT
caldat,reform(jdate),month,day,year,hour,min,sec
string=date_arr2iso([[year],[month],[day],[hour],[min],[sec]],withT=withT)
return,string
end

;***
