;****I* idl/idl.date_mmm_2_mm
; NAME
; date_mmm_2_mm
; 
; PURPOSE
; To convert 3 letter month to a value from 1 to 12
; 
; USAGE
; 
; IDL> print,date_mmm_2_mm('January')
; 01
; 
; INPUT
; * Months - string (scalar or array)
;            this can be either 3-letter month names or the complete
;            month name (because the input is trimmed to the 1st three
;            letters. 
; OUTPUT
; * array or scalar - zero-filled strings with range '01' to '12'
; 
; EXAMPLE
; * See b1uread.pro
; 
; USES
; Only internal IDL routines
; 
; USED BY
; * b1u_2_grisat.pro
; 
; NOTES
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; HISTORY
; 2010 Jun - Added headers  & strmid call to allow entire months as
;            input
;            
; SOURCE

function date_mmm_2_mm,input
in = strupcase(input)
in = strmid(in,0,3)
month=['JAN','FEB','MAR', $
       'APR','MAY','JUN', $
       'JUL','AUG','SEP', $
       'OCT','NOV','DEC']
mm = where(month eq in) + 1
mm = string(mm,format='(i2.2)')
return,mm
end
;***
