;****I* idl/idl.def_oagoes7
; NAME
; def_oagoes7
; 
; PURPOSE
; To define a structure used in reading orbit and attitude data from
; older GOES data 
; 
; USAGE
; INPUT
; OUTPUT
; RESULT
; EXAMPLE
; USES
; * Nothing.
; 
; USED BY
; * b1read_oa.pro
; 
; NOTES
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; ???
; 
; HISTORY
; SOURCE

function def_oagoes7
;qac = {qac,$
;       tdate:intarr(3),$        ;transmission date
;       ttime:intarr(3),$        ;transmission time
;       sscan:0,$                ;start scan
;       escan:0,$                ;end scan
;       covar:lonarr(128),$      ;covariance table
;       freq_a:lonarr(64),$      ;histogram of visible A data
;       freq_ir:lonarr(256),$    ;histogram of IR B1 data
;       freq_vis:lonarr(256),$   ;histogram of Vis B1 data
;       ac_grid:lonarr(2,2,10),$ ;AC domain NW/SE corner lats and lons
;       as_grid:lonarr(2,2,10),$ ;AS domain NW/SE corner lats and lons
;       pres:bytarr(8),$         ;archiving record (character)
;       vol:bytarr(7,6),$        ;archiving volume identifiers
;       comments:bytarr(20,72),$ ;transmission commentary
;       oa_doc:intarr(12,2,10),$ ;orbit/attitude documentation
;       oa_complete:intarr(2,10)} ;unknown
qac = {qac4,$
       tdate:intarr(3),$        ;transmission date
       ttime:intarr(3),$        ;transmission time
       sscan:0,$                ;start scan
       escan:0,$                ;end scan
       covar:lonarr(128),$      ;covariance table
       freq_a:lonarr(64),$      ;histogram of visible A data
       freq_ir:lonarr(256),$    ;histogram of IR B1 data
       freq_vis:lonarr(256),$   ;histogram of Vis B1 data
       ac_grid:lonarr(2,2,10),$ ;AC domain NW/SE corner lats and lons
       as_grid:lonarr(2,2,10),$ ;AS domain NW/SE corner lats and lons
       pres:bytarr(8),$         ;archiving record (character)
       vol:bytarr(7,6),$        ;archiving volume identifiers
       comments:bytarr(20,72),$ ;transmission commentary
       oa_date1:0l,$            ;epoch date
       oa_time1:0l,$            ;epoch time
       spare1:lonarr(4),$
       oa_x1:0l,$               ;satellite position X coordinate
       oa_y1:0l,$               ;satellite position Y coordinate
       oa_z1:0l,$               ;satellite position Z coordinate
       oa_vx1:0l,$              ;satellite velocity X coor.
       oa_vy1:0l,$              ;satellite velocity Y coor.
       oa_vz1:0l,$              ;satellite velocity Z coor.
       oa_sper:0l,$             ;satellite spin period
       oa_spr1:0l,$             ;spin axis rgt. asc.
       oa_spdc1:0l,$            ;spin axis declination
       oa_zeta:0l,$             ;line bias
       oa_rho:0l,$              ;element bias
       oa_eta:0l,$              ;skew bias
       oa_gamma:0l,$            ;sun pulse to vissr angle
       names:0l,$               ;coded information on data provider
       oa_id:0l,$               ;method used for O&A
       oa_sra1:0l,$             ;sun right ascension
       oa_sdc1:0l,$           ;sun declination
       oa_gra1:0l,$           ;greenwich right ascension
       oa_est:0l,$            ;eclipse start time
       oa_eet:0l,$            ;eclipse end time
       oa_fper:0l,$           ;satellite spin period
       oa_tc:0l,$             ;eclipse thermal time constant
       oa_ce:lonarr(4),$      ;equatorial scan count cheb. param.
       oa_cr:lonarr(4),$      ;cheb. retransmission param.
       oa_cb:lonarr(10),$     ;chebyshev beta parameters
       oa_pnl:0l,$            ;primary scanner north limit
       oa_rnl:0l,$            ;redundant scanner north limit
       oa_cx:lonarr(11),$     ;chebyshev X position parameters
       spare6:0l,$
       oa_cy:lonarr(11),$     ;chebyshev Y position parameters
       spare7:0l,$
       oa_cz:lonarr(11),$     ;chebyshev Z position parameters
       oa_epy:0l,$            ;epoch date
       oa_eph:0l,$            ;epoch time
       oa_sma:0l,$            ;semimajor axis
       oa_ecc:0l,$            ;eccentricity
       oa_inc:0l,$            ;inclination
       oa_ma:0l,$             ;mean anomaly
       oa_ap:0l,$             ;argument of perigee
       oa_ran:0l,$            ;righ ascension of ascending node
       oa_sbscan:0l,$         ;sub.sat. line
       oa_sbsamp:0l,$         ;sub sat. pixel
       oa_sblat:0l,$          ;sub. sat. latitude
       oa_sblon:0l,$          ;sub. sat. longitude
       oa_yaw:0l,$            ;YAW angle
       oa_sbscanb:0l,$        ;SBSCAN reformated in BCD
       oa_sbsamb:0l,$         ;SBSAMP in BCD
       oa_sblatb:0l,$         ;SBLAT in BCD
       oa_sblonb:0l,$         ;SBLON in BCD
       oa_yawb:0l,$           ;YAW in BCD
       spare8:0l,$
       oa_x2:0l,$             ;satellite X position
       oa_y2:0l,$             ;satellite Y position
       oa_z2:0l,$             ;satellite Z position
       oa_vx2:0l,$            ;satellite X velocity
       oa_vy2:0l,$            ;satellite Y velocity
       oa_vz2:0l,$            ;satellite Z velocity
       spare9:0l,$
       oa_spra2:0l,$          ;spin axis rgt. asc.
       oa_spdc2:0l,$          ;spin axis declination
       oa_deltal:0l,$         ;un-documented
       oa_deltas:0l,$         ;un-documented
       spare10:lonarr(4),$
       oa_sraz:0l,$           ;sun rgt. axis asc.
       oa_sdc2:0l,$           ;sun declination
       oa_gra2:0l,$           ;greenwich rgt. asc.
       oa_complete:intarr(2,10)} ;unknown


return,qac
end

;***
