;****I* idl/idl.define_mcidas_nav_goes
; NAME
; define_mcidas_nav_goes
; PURPOSE
; Set up the McIDAS navigation structure for the older GOES files. (pre-GVAR)
; USAGE
; from mcidas_read.pro
; define_mcidas_nav_goes,VISRstr
; NOTES
; Derived from McIDAS documentation:
; http://www.ssec.wisc.edu/mcidas/doc/misc_doc/area.txt
; USED BY
; * mcidas_read.pro
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; CREATION DATE
; ???
; HISTORY
; * 27 Jun07 - Added documentation
; SOURCE

pro define_mcidas_nav_goes,vissr

vissr  = {nav,$
          type:bytarr(4),$      ;'GOES'
          iddate:0l,$           ;ID and date: SSSYYJJJ
          time:0l,$             ;nominal start time of image
          orbit:0l,$            ;orbit type (=1 for GOES)        ORBITAL
          etimy:0l,$            ;epoch date:YYMMDD               PARAMETERS
          etimh:0l,$            ;epoch time:HHMMSS
          semima:0l,$           ;semi major axis KM*100
          eccen:0l,$            ;eccentricity * 100000
          orbinc:0l,$           ;orbital inclination deg*1000
          meana:0l,$            ;mean anomoly deg*1000
          perigee:0l,$          ;argument of perigee deg*1000
          asnode:0l,$           ;right ascension of asc. node deg*1000
          declin:0l,$           ;declin. sat. axis: dddmmss (+ =north)    SATELLITE
          rascen:0l,$           ;rgt. asc. of sat. axis.                  PARAMETERS
          piclin:0l,$           ;picture center line #
          spinp:0l,$            ;spin period: period in usec or revolutions/min.
          deglin:0l,$           ;total sweep angle, line direction DDDMMSS
          lintot:0l,$           ;# scan lines NNLLLLL NN = # sensors, LLL: # scans NN*LL
          degele:0l,$           ;sweep angle element direction
          eletot:0l,$           ;# elem in scan line
          pitch:0l,$            ;forward leaning  DDDMMSS
          yaw:0l,$              ;sideways leaning DDDMMSS
          roll:0l,$             ;rotation         DDDMMSS
          reserved:0l,$         ;  OTHER
          iajust:0l,$           ;east-west adjustment value              PARAMETERS
          iajtim:0l,$           ;time of iajust from first valid landmark
          res2:0l,$
          iseang:0l,$           ;angle between vissr and sun sensor DDDMMSS
          skew:0l,$             ;for a later implementation
          res3:0l,$
          bt1_scan:0l,$         ;scan line of beta 1
          bt1_time:0l,$         ;time of above scan HHMMSS
          bt1_tim2:0l,$         ;time (continued) millisec * 10
          bt1_cnt:0l,$          ;beta count 1
          bt2_scan:0l,$         ;scan line of beta 1
          bt2_time:0l,$         ;time of above scan HHMMSS
          bt2_tim2:0l,$         ;time (continued) millisec * 10
          bt2_cnt:0l,$          ;beta count 1
          gamma:0l,$            ;element offset * 100
          gamdot:0l,$           ;element drift per hour * 100
          res4:lonarr(80),$
          memo:bytarr(32) }     ;EBCDIC comment

end
;***
