;****I* idl/idl.flines
; NAME
; flines
; 
; PURPOSE
; To count the number of lines in an ASCII text file.
; 
; USAGE
; IDL> print,flines('startup.pro')
;     34
;     
; INPUT
; filename - string (scalar) - filename
;
; KEYWORDS
; compress - OPTIONAL - if set, use IDL compressed open (to read
;            gzipped files)
; 
; OUTPUT
; * Returns long integer of number lines in the ASCII file
; 
; EXAMPLE
; * see b1uread.pro
; 
; USES
; Only uses internal IDL routines
; 
; USED BY
; * b1uread.pro
; 
; NOTES
; TODO
; * Remove failure if file doesn't exist
; 
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; HISTORY
; 2010 Jun - Added robodoc header
; 
; SOURCE

function flines,file,compress=compress

;Send error (returns -1) if no file is provided
if (n_elements(file) eq 0) then goto,eout

;Open file ... compressed or not.
if (keyword_set(compress)) then begin
    openr,unit,file,/get_lun,/compress
endif else begin
    openr,unit,file,/get_lun
endelse

; read in each line of text and when EOF, then output that number of
; lines.
on_ioerror,out
c=0L
test=''
while not eof(unit) do begin
  readf,unit,format='(a1)',test
  c=c+1L
endwhile
out:
close,unit
free_lun,unit
flines=c
return,flines

;ON ERROR return -1
eout:
flines=-1
;print,file
Message,'Input file missing'
return,flines
end

;***
