;****I* idl/idl.geo_hirs_limb_corr
; NAME
; geo_hirs_limb_corr
; 
; PURPOSE
; To perform limb correction for HIRS data.
; 
; USAGE
; INPUT
; OUTPUT
; RESULT
; EXAMPLE
; USES
; * Only IDL internal routines
; * External data file:
;   '/net/glaze/home/raid/kknapp/data/geo-vza-tables/'+sat+'.WV.VZA-corr.dat'
;   
; USED BY
; * b1u_2_grisat.pro
; 
; 
; NOTES
; * Requires the presence of the correction table
;   '/net/glaze/home/raid/kknapp/data/geo-vza-tables/'+sat+'.WV.VZA-corr.dat'
;   
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; ???
; 
; HISTORY
; 
; SOURCE

function geo_hirs_limb_corr,sat,temp,vza

if (n_elements(temp) ne n_elements(vza) ) then begin
    message,'temp and vza inputs should have same number of arguments'
endif

openr,unit,'/net/glaze/home/raid/kknapp/data/geo-vza-tables/'+sat+'.WV.VZA-corr.dat',/get_lun
readf,unit,nang,nt
aarr=fltarr(nang)
tarr=fltarr(nt)
corr = fltarr(nang,nt)
readf,unit,aarr                    ;angle array
readf,unit,tarr                    ;temp array
readf,unit,corr
close,unit
free_lun,unit


;convert vza to aarr index
ivza = (vza / 4.0) > 0. < float(nang-1)

;convert temp to tarr index
itemp = ((temp-tarr(0))/4.) 

;set up bad data mask
mask = corr gt (-30)

outcorr = interpolate(corr,ivza,itemp,missin=-1)

outmask = interpolate(mask,ivza,itemp,missing=0)
;print,outcorr,outmask,outcorr*outmask

return,outcorr*outmask

end
;***
