;****I* idl/idl.geo_struct
;+
; NAME
;    geo_struct
;
; PURPOSE

; This "include" file defines the structures used in the b1read
; algorithms. It includes structures used by other routines as well as
; informational structures. It does NOT include format specific
; structures (for example, the numerous types of NAV or AREA headers.

; The GEO_NAV structure are the minimum requirements to navigate
; geostationary data using the GEO_IJ2LL and GEO_LL2IJ unified
; routines (that is, all satellites can use this routine once the
; structure variables are in place).
;      NOTE: The spin axis variables (rgtasc and decl) are primarily
;      for spin-stabilised satellites. The parameters are used to
;      convert between instrument and Earth-centered coordinates. For
;      this reason, these parameters are defined for 3-axis
;      stabilized, where the values represent an equivalent spin axis
;      declination and right ascension.
;             ----------------------------------------------------
;             ALERT: The GEO_NAV is no longer used. It is obsolete
;             since the geo_ij2ll and geo_ll2ij routines are also
;             obsolete.
;             ----------------------------------------------------
;GEO_NAV = {Geo_nav_data,$
;           lat:0.d0,$           ;Sub-Sat. latitude (deg)
;           lon:0.d0,$           ;Sub-Sat. longitude (deg)
;           radius:0.d0,$        ;Distance satellite-Earth-center (km)
;           rgtasc:0.d0,$        ;Spin axis right ascension (deg)
;           decl:0.d0,$          ;Spin Axis declination
;           GOESNEXT:0,$         ;GOES-Next flag 1=GOES-8,9,10,...
;           rma:0.d0,$           ;Roll misalignment (GOES-Next only)
;           pma:0.d0}            ;Pitch misalignment (GOES-Next only)
; USES
; Nothing.
;
; USED BY
; b1read.pro 
;        b1read_aes_g7.pro
;        b1read_aes_g8.pro
;        b1read_cma_fy2.pro
;        b1read_csu_g6.pro
;        b1read_csu_g7.pro
;        b1read_csu_g8.pro
;        b1read_csu_g91012.pro
;        b1read_csu_met3.pro
;        b1read_eum_met.pro
;        b1read_eum_msg.pro
;        b1read_jma_gms.pro
;        b1read_jma_mts.pro
;        b1read_noa_goe.pro
;        b1read_uws_g56.pro
;
; SOURCE
; 

; Keplarian orbital parameters are stored in the ORBIT structure
ORBIT = {keplarian_components,$
         ep_date:0l,$           ;Epoch date (GMT) YYJJJ
         ep_time:0l,$           ;Epoch time (GMT) HHMMSS
         ep_msec:0.0d0,$        ;Epoch fraction of a second
         semima:0.d0,$          ;Orbit Semimajor axis (km)
         incl:0.d0,$            ;Orbit inclination (deg)
         eccen:0.d0,$           ;Orbit eccentricity
         meana:0.d0,$           ;Orbit mean anomaly (deg)
         rgtasc:0.d0,$          ;Orbit right ascension
         perigee:0.d0,$         ;Orbit perigee
         sper:0.d0}             ;Satellite spin period (RPM)

; The IMGINFO structure stores information on the image, such as
; parameters necessary to convert from line-pixel to elevation-scan
; angle.
IMGINFO = {image_info,$
           frdate:0.d0,$        ;Image date (GMT)
           frtime:0.d0,$        ;Image time (GMT)
           cen_lin:0.d0,$       ;Image Center line
           cen_pix:0.d0,$       ;Image center pixel
           linres:0,$           ;Image line resolution
           pixres:0,$           ;Image pixel resolution
           sline:0l,$           ;Starting image line
           spix:0l,$            ;Starting image pixel
           nlin:0l,$            ;number of lines
           npix:0l,$            ;number of pixels
           linang:0.d0,$        ;Angular step of 1 image line (deg)
           pixang:0.d0}         ;Angular step of 1 image pixel (deg)

; The ATT structure defines the attitude of the satellite in orbit
ATT = {sat_attitude,$
       roll:0.d0,$              ;satellite roll
       pitch:0.d0,$             ;satellite pitch
       yaw:0.d0,$               ;satellite yaw
       spra:0.d0,$              ;satellite spin axis declination
       spdc:0.d0,$              ;satellite spin axis rgt. ascension
       rma:0.d0,$               ;Roll misalignment (GOES-Next only)
       pma:0.d0}                ;Pitch misalignment (GOES-Next only)

;The SATTIM structure defines temporally dependent parameters for the
;data.
SATTIM = {temporal_data,$
          d:0.d0,$              ;dfference intime from gt1 to gt2 (sec)
          gt1:0.d0,$
          gt2:0.d0,$
          gra1:0.d0,$           ;Greenwich rgt. asc. at time1
          gra2:0.d0,$           ;Greenwich rgt. asc. at time1
          spra1:0.d0,$          ;Spin axis rgt. asc. at time1
          spra2:0.d0,$          ;Spin axis rgt. asc. at time1
          spdc1:0.d0,$          ;Spin axis declination at time1
          spdc2:0.d0,$          ;Spin axis declination at time1
          cheb_x:dblarr(13),$   ;Chebyshev polynomials for X position
          cheb_y:dblarr(13),$   ;Chebyshev polynomials for Y position
          cheb_z:dblarr(13),$   ;Chebyshev polynomials for Z position
          cheb_a:dblarr(13),$   ;Chebyshev polynomials for A position
          cheb_b:dblarr(13),$   ;Chebyshev polynomials for B position
          cheb_c:dblarr(13)}    ;Chebyshev polynomials for C position

; The CHNINFO provides information on the channels provided by the B1
; data. This is derived from the channel information provided.
CHNINFO = {channel_info,$
           nchan:0,$            ;# of channels
           ch_name:strarr(20),$ ;chan. name provided by b1 read
           ch_info:strarr(20)}  ;misc. info on the channel

; Some B1 files include a test grid with which one can compare their
; navigation with the pixel/line pairings from the data provider. This
; is primarily used in the early GOES sats. and the JMA data stream
; (where it is used to correct the roll/pitch/yaw).
TGRID = {TestGrid,$
         n:0l,$                 ;number of points in each direction
         slat:0.d0,$            ;starting latitude (deg)
         slon:0.d0,$            ;starting longitude (deg)
         dlat:0.d0,$            ;latitude step (deg)
         dlon:0.d0,$            ;longitude step (deg)
         pixl:lonarr(15,15),$   ;pixel values for the grid
         line:lonarr(15,15)}    ;line values for the grid
;***
