;****I* idl/idl.get_file_size
; NAME
; get_file_size
; 
; PURPOSE
; To determine the file size (by decompressing as necessary)
; 
; USAGE
; 
; INPUT
; file - filename
;
; OUTPUT
; * File size in bytes
; 
; KEYWORDS
; compress - the file was compressed so used gzip to output the
;            uncompressed file size
; 
; RESULT
; EXAMPLE
; USES
; * external call to GNU gzip
;  
; USED BY
; * b1read_oa.pro
; * b1read_aes_g7.pro
; * b1read_aes_g8.pro
; * b1read_csu_g6.pro
; * b1read_csu_g7.pro
; * b1read_csu_g8.pro
; * b1read_csu_g91012.pro
; * b1read_csu_met3.pro
; * b1read_eum_met.pro
; * b1read_eum_msg.pro
; * b1read_jma_gms.pro
; * b1read_uws_g56.pro
; 
; NOTES
; Some file calculations need the uncompressed file size, and this
; seems to be the only way to get it.
; 
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; CREATION DATE
; Thu Dec  3 16:17:05 EST 2009
; HISTORY
; SOURCE

function get_file_size,file,status,compress=compress

if n_elements(file) eq 0 then begin
    print,'Usage: size = get_file_size(file,status,compress=compress)'
    print,'size = size of file in bytes'
    print,'status = (optional) returns status 0=successful, otherwise a return error'
    print,'compress=returns compression status'
    status=2
    return,-1
endif

f=strsplit(file,'.',/extr)
n=n_elements(f) 
if (f(n-1) eq 'gz') then begin
    spawn,'gzip -l '+file,out1,out2
    if (strlen(out2) gt 2) then begin
        print,'Problem finding size of file.'
        print,'Error message is:'
        print,out2
        stop
    endif
    s=strsplit(out1(1),' ',/extr)
    filesize=long(s(1))
    status=0
    compress=1
    return,filesize
endif else begin
    finfo=file_info(file)
    status=0
    compress=0
    return,finfo.size
endelse
status=1
end

;***
