;****I* idl/idl.get_histry
; NAME
; get_histry
; 
; PURPOSE
; 
; USAGE
; INPUT
; OUTPUT
; RESULT
; EXAMPLE
; USES
; * reset.pro
; * geo_struct.pro
; * b3read.pro
; 
; USED BY
; * b1read_aes_g7.pro
; 
; NOTES
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; CREATION DATE
; Thu Dec  3 16:17:05 EST 2009
; HISTORY
; SOURCE

pro get_histry,b1file, $
;               GEOstr=GEOstr,$  ;use with geo_ij2ll and geoll2ij
               ORBstr=ORBstr,$  ;keplarian orbital info
               IMGstr=IMGstr,$  ;image size and scan geometry
               ATTstr=ATTstr,$  ;satellite attitude info
               TIMstr=TIMstr,$  ;temporal data
               CHNstr=CHNstr,$  ;channel information
               TGRstr=TGRstr,$  ;the test grid data
               histry=histry

@geo_struct.pro


;common kepler, semima, incl, eccen, meana, rasc, perigee, $
;  edate, etime,msec
;common CONST, PI,rad, rot, gr_ang0, ae, be, gme, j2, ree
;common att, roll, pitch, yaw,spra1,spra2,spdc1,spdc2
;common image,cen_lin,cen_pix,linres,pixres,sline,spix,linang,pixang, $
;  frdate,frtime,spper,pma,rma
;common green,gra1,gra2,gt1,gt2
;common latlongrid,pt

;find b3 file
f=strsplit(b1file,'.',/extr)
g=where(f eq 'b1')
if (g(0) ge 0) then f(g) = 'b3'
f(8)='???'
hhmm = fix(f(7))
time = float(hhmm / 100) + float(hhmm mod 100) / 60.
b3time = round(time / 3.)*3 * 100
f(7) = string(b3time,format='(i4.4)')
f(1) = 'B3'
f(2) = '?'

b3f = strjoin(f,'.')
q = strpos(b3f,'b1')
if (q gt 0) then b3f = strmid(b3f,0,q)+'b3'+strmid(b3f,q+2,strlen(b3f))

b3file=file_search(b3f,count=nfiles)

if (nfiles gt 0) then begin
    b3read,b3file(0),image,lat,lon,mu,mun,azi,nchan,iidr
    ;assign the histry data to the navigation data
    year = long(f(4))
    mm = long(f(5))
    dd = long(f(6))
    yymmdd = (year-1900)*10000l + mm * 100l + dd
    yyjjj = yymmdd_2_yyjjj(yymmdd)
    loc = where(iidr.pad2 eq yyjjj or $
                iidr.pad2 eq (yyjjj-1) or $
                iidr.pad2 eq (yyjjj+1),nmatch)
    known = [2,77,192,334]
    for im=0,nmatch-1 do begin
        m = where(loc(im) eq known,nm)
        if (nm gt 0) then begin
            start = known(m(0))
            goto,getit
        endif
    endfor
    print,'Starting point of HISTRY not found'
    stop
getit:
    HISTRY = iidr.pad2(start:start+119)
    f = strsplit(b3file(0),'.',/extr)
    ver = f(2)
    sat = f(3)
    spc = f(8)
    if (spc eq 'CSU') then begin
        if (sat eq 'GOE-6') then begin
            if (ver eq '1') then begin
                semima= histry(85) / 100.d0
                eccen = histry(86) / 1.d6
                incl  = histry(87) / 1000.d0
                meana = histry(88) / 1000.d0
                rasc  = histry(90) / 1000.d0
                perigee=histry(89) / 1000.d0
                edate = yymmdd_2_yyjjj(histry(83))
                etime = histry(84)
                msec  = 0.0

                gra1 = histry(23) /2.d0^21 /rad
                gra2 = histry(119)/2.d0^21 /rad
                gt1 = histry(1) * 0.01
                gt2 = histry(1) * 0.01 + 13.d-*3600.d0

                roll = histry(16)/2.d0^21 / rad
                pitch= histry(15)/2.d0^21 / rad
                yaw  = histry(17)/2.d0^21 / rad

                spra1 = histry(13) /2.d0^21 /rad
                spra2 = histry(109)/2.d0^21 /rad
                spdc1 = histry(14) /2.d0^21 /rad
                spdc2 = histry(110)/2.d0^21 /rad
            endif else begin
                stop
            endelse
        endif else begin
            stop
        endelse
    endif else if (spc eq 'UWS') then begin
        if (sat eq 'GOE-5' or $
            sat eq 'GOE-6') then begin
                semima= histry(85) / 100.d0
                eccen = histry(86) / 1.d6
                incl  = histry(87) / 1000.d0
                meana = histry(88) / 1000.d0
                rasc  = histry(90) / 1000.d0
                perigee=histry(89) / 1000.d0
                edate = yymmdd_2_yyjjj(histry(83))
                etime = histry(84)
                msec  = 0.0

                gra1 = histry(23) /2.d0^21 /rad
                gra2 = histry(119)/2.d0^21 /rad
                gt1 = histry(1) * 0.01
                gt2 = histry(1) * 0.01 + 13.d-*3600.d0

                roll = histry(16)/2.d0^21 / rad
                pitch= histry(15)/2.d0^21 / rad
                yaw  = histry(17)/2.d0^21 / rad

                spra1 = histry(13) /2.d0^21 /rad
                spra2 = histry(109)/2.d0^21 /rad
                spdc1 = histry(14) /2.d0^21 /rad
                spdc2 = histry(110)/2.d0^21 /rad
        endif else begin
            stop
        endelse
    endif else if (spc eq 'AES') then begin
        if ((sat eq 'GOE-7' and ver eq '3') or $
            (sat eq 'GOE-7' and ver eq '2') or $
            (sat eq 'GOE-7' and ver eq '1')) then begin
            semima= histry(85) / 100.d0
            eccen = histry(86) / 1.d6
            incl  = histry(87) / 1000.d0
            meana = histry(88) / 1000.d0
            rasc  = histry(90) / 1000.d0
            perigee=histry(89) / 1000.d0
            edate = yymmdd_2_yyjjj(histry(83))
            etime = histry(84)
            msec  = 0.0
            
;            gra1 = histry(23) /2.d0^21 /rad
;            gra2 = histry(119)/2.d0^21 /rad
            gra1 = -999.
            gra2 = -999.
            gt1 = histry(1) * 0.01
            gt2 = histry(1) * 0.01 + 13.d0*3600.d0

            roll = histry(16)/2.d0^21 ;/ rad
            pitch= histry(15)/2.d0^21 ;/ rad
            yaw  = histry(17)/2.d0^21 ;/ rad
            
            spra1 = histry(13) /2.d0^21 ;/rad
            spra2 = histry(109)/2.d0^21 ;/rad
            spdc1 = histry(14) /2.d0^21 ;/rad
            spdc2 = histry(110)/2.d0^21 ;/rad
        endif else begin
            stop
        endelse
    endif else begin
        stop
    endelse
    
endif else begin
    print,'No B3 file found.'
    histry = -1
    return
endelse

;All values are directly from the common block, so this if can be
;copied to each B1READ routine
orbit.ep_date = edate
orbit.ep_time = etime
orbit.ep_msec = msec
orbit.semima = semima
orbit.incl = incl
orbit.eccen = eccen
orbit.meana = meana
orbit.rgtasc = rasc
orbit.perigee = perigee
ORBstr = orbit

;All values are directly from the common block, so this if can be
;copied to each B1READ routine
;imginfo.frdate = frdate
;imginfo.frtime = frtime
;imginfo.cen_lin = cen_lin
;imginfo.cen_pix = cen_pix
;imginfo.linres = linres
;imginfo.pixres = pixres
;imginfo.nlin = ny
;imginfo.npix = nx
;imginfo.sline = sline
;imginfo.spix = spix
;imginfo.linang = linang
;imginfo.pixang = pixang
;IMGstr = imginfo

att.roll = roll
att.pitch = pitch
att.yaw = yaw
;need to calculate the current spra and spdc first
;att.spra = spnax(0)
;att.spdc = spnax(1)
att.rma = 0.d0
att.pma = 0.d0
ATTstr = att

sattim.d = gt2-gt1
sattim.gt1 = gt1
sattim.gt2 = gt2
sattim.gra1 = gra1
sattim.gra2 = gra2
sattim.spra1 = spra1
sattim.spra2 = spra2
sattim.spdc1 = spdc1
sattim.spdc2 = spdc2
sattim.cheb_x = -999.d0         ;nav.oa_cx
sattim.cheb_y = -999.d0         ;nav.oa_cy
sattim.cheb_z = -999.d0         ;nav.oa_cz
sattim.cheb_a = -999.d0
sattim.cheb_b = -999.d0
sattim.cheb_c = -999.d0
TIMstr = sattim

;Call navigation info to get satellite information
inlat = 0.0d0
inlon = -96.d0

;latlon2ij,inlat,inlon,i,j, $
;  glat=glat,glon=glon,sathgt=grad,spnax=spnax, $
;  TIMstr=TIMstr,ORBstr=ORBstr,ATTstr=ATTstr,IMGstr=IMGstr

;spnax = spnax*180.d0/!dpi mod 360.d0

;if (glon lt (-180)) then glon = glon + 360.d0
;if (glon gt 180)    then glon = glon - 360.d0

;geo_nav.lat = glat
;geo_nav.lon = glon
;geo_nav.radius = grad
;geo_nav.rgtasc = spnax(0)
;geo_nav.decl   = spnax(1)
;geo_nav.GOESNEXT = 0
;geo_nav.rma = 0.0d0
;geo_nav.pma = 0.0d0
;GEOstr = geo_nav

;att.spra = spnax(0)
;att.spdc = spnax(1)
;ATTstr = att

end

;***
