;****I* idl/idl.gpoint
; NAME
; gpoint
; 
; PURPOSE
; this subroutine converts geographic lat/lon to the related elevation
; and scan angles.
; 
; USAGE
; INPUT
; OUTPUT
; RESULT
; EXAMPLE
; USES
; * elcons.pro
; * elcomm.pro
; 
; USED BY
; * b1u_imc.pro
; 
; NOTES
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; ???
; HISTORY
; SOURCE

pro gpoint,rlat,rlon,alf,gam,ierr

@elcons.pro
@elcomm.pro
;COMPUTES SINUS OF GEOGRAPHIC (GEODETIC) LATITUDE
SING=SIN(RLAT)
W1=AEBE4*SING*SING

;SINUS OF THE GEOCENTRIC LATITUDE
SLAT=((0.375D0*W1-0.5D0)*W1+1.0D0)*SING/AEBE2

;COMPUTES LOCAL EARTH RADIUS AT SPECIFIED POINT
W2=SLAT*SLAT
W1=AEBE3*W2
W1=(0.375D0*W1-0.5D0)*W1+1.D0

;COMPUTES CARTESIAN COORDINATES OF THE POINT
u = dblarr(3)
U(3-1)=SLAT*W1                                                              
W2=W1*SQRT(1.0D0-W2)
U(1-1)=W2*COS(RLON)
U(2-1)=W2*SIN(RLON)

;POINTING VECTOR FROM SATELLITE TO THE EARTH POINT
f = dblarr(3)
F(1-1)=U(1-1)-XS(1-1)
F(2-1)=U(2-1)-XS(2-1)  
F(3-1)=U(3-1)-XS(3-1)  
W2=U(1-1)*F(1-1)+U(2-1)*F(2-1)+U(3-1)*F(3-1)*AEBE2


;VERIFIES VISIBILITY OF THE POINT
IF (W2 GT 0.0D0) THEN begin
    ;INVISIBLE POINT ON THE EARTH                    
    IERR=1
    ALF=99999.0D0
    GAM=99999.0D0
ENDIF else begin
    ;CONVERTS POINTING VECTOR TO INSTRUMENT COORDINATES
    ft = dblarr(3)
    FT(1-1)=BT(1-1,1-1)*F(1-1)+BT(2-1,1-1)*F(2-1)+BT(3-1,1-1)*F(3-1)
    FT(2-1)=BT(1-1,2-1)*F(1-1)+BT(2-1,2-1)*F(2-1)+BT(3-1,2-1)*F(3-1)
    FT(3-1)=BT(1-1,3-1)*F(1-1)+BT(2-1,3-1)*F(2-1)+BT(3-1,3-1)*F(3-1)
;    CONVERTS POINTING VECTOR TO SCAN AND ELEVATION ANGLES AND
;     CORRECTS FOR THE ROLL AND PITCH MISALIGNMENTS
    GAM=ATAN(FT(1-1)/SQRT(FT(2-1)^2+FT(3-1)^2))
    ALF=-ATAN(FT(2-1)/FT(3-1))
    W1=SIN(ALF)
    W2=COS(GAM)
    ALF=ALF+RMA*(1.0D0-COS(ALF)/W2)+PMA*W1*(1.0D0/W2+TAN(GAM))
    GAM=GAM-RMA*W1
    IERR=0
endelse

end
;***
