;****I* idl/idl.hirs_b1u_month
; NAME
; hirs_b1u_month
; 
; PURPOSE
; To read matchup data from a HIRS/B1U matchup file and filter the
; output based on input limits on time, view and spatial noise.
; 
; USAGE
; INPUT
; OUTPUT
; RESULT
; EXAMPLE
; USES
; * ftime.pro
; 
; USED BY
; * cal_b1u_monthly.pro
; 
; NOTES
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; ???
; 
; HISTORY
; 
; SOURCE

pro hirs_b1u_month,part,channum,ifov=ifov, $
                   dtime=dtime,output=output, $
                   ichan=ichan, $
                   dvza=dvza, $
                   dtemp=dtemp, $
                   maxdmu=maxdmu, $
                   maxdazi=maxdazi, $
                   wvp_only=wvp_only, $
                   return_value=return_value, $
                   files=files

if n_elements(part) eq 0 then    part='N12.GOE-8.1998.05'
if n_elements(channum) eq 0 then channum = 0
if n_elements(ifov) eq 0 then    ifov=1
if n_elements(dtime) eq 0 then   dtime= 30 ;minutes
if n_elements(dvza) eq 0 then    dvza = 45 ;degrees
if n_elements(dtemp) eq 0 then   dtemp = 90 ;degrees
if n_elements(maxdmu) eq 0 then  maxdmu = 0.1
if n_elements(maxdazi) eq 0 then maxdazi=200 ;degrees
f=strsplit(part,'.',/extr)
cy=f(2)
;print,part
files=file_search(getenv('ISCCPDIR')+'/b1ucal/matchup/'+cy+'/'+part+'*', $
                  count=nfiles)
if (nfiles eq 0) then begin
    print,'NO FILES FOUND:'
    print,getenv('ISCCPDIR')+'/b1ucal/matchup/'+cy+'/'+part+'*'
    stop
endif

return_value=1

nmax=5000000l

icount=0l
for ifile=0,nfiles-1 do begin
    file=files(ifile)
    finfo=file_info(file)
    size=finfo.size
    chan = lonarr(3)            ;12 bytes
    s=0l
    close,1

;===OLD=== matchup output file format:
;12 byte header: lonarr(3) : B1 chan order of: IRWIN, IRWVP, IRSPL (-1=Missing)
;XX-byte records
; time   =DOUBLE       (8=8)
; satid  =INT*2        (2=10)
; lat    =INT*2        (2=12)
; lon    =INT*2        (2=14)
; hangle =INT*2(4)     (8=22)
; hdata  =INT*2(3,nch) (6*nch   =22+ 6*nch)
; b1angle=INT*2(2)     (4       =26+ 6*nch)
; fov    =3(INT*2,INT*2(3,nch))
;                      (6+18*nch=32+24*nch)*nrec + 12
;===NEW=== matchup output file format:
;12 byte header: lonarr(3) : B1 chan order of: IRWIN, IRWVP, IRSPL (-1=Missing)
;XX-byte records
; time   =DOUBLE       (8=8)
; satid  =INT*2        (2=10)
; lat    =INT*2        (2=12)
; lon    =INT*2        (2=14)
; hangle =INT*2(4)     (8=22)
; hdata  =INT*2(3,nch) (6*nch   =22+ 6*nch) <--- changed to include DN
;                                                valued
; b1angle=INT*2(2)     (4       =26+ 6*nch)
; fov    =3(INT*2,INT*2(5,nch))
;                      (6+30*nch=32+36*nch)*nrec + 12

;===NEWER=== matchup output file format:
;Even newer!!!!!!
; ich 12 byte header: lonarr(3) : B1 chan order of: IRWIN, IRWVP, IRSPL (-1=Missing)
; XX-byte record
; time   =DOUBLE       (8=8)
; satid  =int*2        (2=10)
; latlon =int*2(2)     (4=14)
; hangle =int*2(4)     (8=22)
; hdata  =int*2(3,nch) (   6*nch=22+ 6*nch)
; b1angle=int*2(2)     (4       =26+ 6*nch)
; fov    =1(int*2,int*2(5,nch))
;                      (2+10*nch=28+16*nch)

    openr,1,file
    readu,1,chan

    if (keyword_set(WVP_ONLY) and chan(1) lt 0) then begin
        close,1
        goto,nextfile
    endif

                                ;chan order is IRWIN, IRWVP, IRSPL
    nch = total(chan ge 0)
    okchan = where(chan ge 0)
                                ;nrec = (size - 12) / (32l + 36l * nch) ;previous method
    nrec = (size - 12) / (28l + 16l * nch)
;print,ifile,' ',stripname(file),' ',nrec
    if (nch lt 1) then begin
        print,'No channels?'
        goto,nextfile
    endif
    if ((nrec mod 1.) ne 0.) then begin
        print,'Non integer records?'
;        stop
        goto,nextfile
    endif

    fov = {num:0, data:reform(intarr(5,nch),5,nch) }
    rec = {time:0.d0, $ 
           satid:0,$            ;spare:0, $           
           lat:0, lon:0, $      
           hangle:intarr(4), $  
           hdata:reform(intarr(3,nch),3,nch), $ 
           b1angle:intarr(2), $ 
           fov:replicate(fov,1)}
    ifov=0
    match = replicate(rec,nrec)
    readu,1,match
    close,1
                                ;separate out close values
    hvza = match.hangle(2) / 100.
    gvza = match.b1angle(0) / 100.
    hmu = cos(hvza*!pi/180.)
    gmu = cos(gvza*!pi/180.)
    hazi = match.hangle(3)/100.
    bazi = match.b1angle(1)/100.
    dazi = abs(hazi - bazi)
    out = where(dazi gt 180.,nout)
    if (nout gt 0) then dazi(out) = 360. - dazi(out)
;    hnoise = match.hdata(2,ichan)/100.
;    bnoise = match.fov(ifov).data(2,ichan)/100.
    ok = where(abs(hvza-gvza) lt dvza   and $ ;VZA diff is small (in deg)
               abs(hmu-gmu)   lt maxdmu and $ ;VZA diff is small (in mu)
               dazi           lt maxdazi,nok) ;AZI diff is small
    if (nok gt 0) then begin
                                ;store local time of satellite
        f=strsplit(file,'.',/extr)
        localtime=fltarr(nok)
        jul1=dblarr(nok)
        localtime(*)=ftime(f(5)*100l)
                                ;change time sorted to dTIME=time-localtime (minutes)
        jul = julday(f(3),f(4),f(2),f(5)/100,f(5) mod 100,0)
        jul1(*) = jul
                                ;14 minutes is included to adjust the image time in the filename
                                ;to the scan time at the Equator (most imagery takes 28 minutes)
        match.time = (match.time - (jul+14/60./24.)) * 24. * 60.


        if n_elements(matches) eq 0 then begin
            matches = {time:fltarr(nmax), $
                       date:dblarr(nmax), $
                       satid:bytarr(nmax), $
                       lat:intarr(nmax), $
                       lon:intarr(nmax), $
                       hangle:intarr(4,nmax), $
                       hdata:intarr(3,3,nmax), $
                       b1angle:intarr(2,nmax), $
                       fov:intarr(5,3,nmax)}
            matches.time = 1e9
            ltimes = lonarr(nmax)
            jtimes = lonarr(nmax)
;            matches = {time:   match(ok).time, $
;                       satid:  match(ok).satid, $
;                       lat:    match(ok).lat, $
;                       lon:    match(ok).lon, $
;                       hangle: match(ok).hangle, $
;                       hdata:  match(ok).hdata, $
;                       b1angle:match(ok).b1angle, $
;                       fov:    match(ok).fov(ifov).data}
;            ltimes=localtime
;            jtimes=jul1


            proctime=systime(/sec)
        endif 
;            if (nch gt 1) then begin
        if n_elements(proctime) eq 0 then proctime = systime(/sec)
        s=systime(/sec)
                                ;print,stripname(file),ifile,nfiles-1,s-proctime
        proctime=s
;                matches = {time:   [matches.time, match(ok).time], $
;                           satid:  [matches.satid,match(ok).satid], $
;                           lat:    [matches.lat,  match(ok).lat], $
;                           lon:    [matches.lon,  match(ok).lon], $
;                           hangle: [[matches.hangle], [match(ok).hangle]], $
;                           hdata:  [[[matches.hdata]],  [[match(ok).hdata]]], $
;                           b1angle:[[matches.b1angle],[match(ok).b1angle]], $
;                           fov:    [[[matches.fov]],[[match(ok).fov(ifov).data]]]}

        matches.time(icount:icount+nok-1)    = float(match(ok).time)
        matches.date(icount:icount+nok-1)    = jul1
        matches.satid(icount:icount+nok-1)   = match(ok).satid
        matches.lat(icount:icount+nok-1)     = match(ok).lat
        matches.lon(icount:icount+nok-1)     = match(ok).lon
        matches.hangle(*,icount:icount+nok-1)  = match(ok).hangle 
        matches.hdata(*,okchan,icount:icount+nok-1)   = match(ok).hdata 
        matches.b1angle(*,icount:icount+nok-1) = match(ok).b1angle
        matches.fov(*,okchan,icount:icount+nok-1)     = match(ok).fov(ifov).data
;            plot,matches.fov(3,1,0:icount+nok)/100.
;print,nch
;stop
        ltimes(icount:icount+nok-1) = localtime
        jtimes(icount:icount+nok-1) = jul1

        icount=icount+nok
;            endif else begin
;                matches = {time:   [matches.time, match(ok).time], $
;                           satid:  [matches.satid,match(ok).satid], $
;                           lat:    [matches.lat,  match(ok).lat], $
;                           lon:    [matches.lon,  match(ok).lon], $
;                           hangle: [[matches.hangle], [match(ok).hangle]], $
;                           hdata:  [[matches.hdata],  [match(ok).hdata(*,*)]], $
;                           b1angle:[[matches.b1angle],[match(ok).b1angle]], $
;                           fov:    [[matches.fov],    [match(ok).fov(ifov).data(*,*)]]}
;            endelse
;            ltimes=[ltimes,localtime]
;            jtimes=[jtimes,jul1]

;help,jtimes,ltimes
    endif 
nextfile:
    close,1
endfor
;print,'N=',n_elements(matches) 
if n_elements(matches) gt 0 then begin
    ok = where(abs(matches.time) le dtime,nok)
    if (nok gt 0) then begin
        hirs = matches.hdata(0,ok)  /100. + 50.
        b1ut = matches.fov(3,ok)/100. + 50.
        lat =  matches.lat(ok)/100.
        lon =  matches.lon(ok)/100.
        matches1 = {time:   matches.time(ok), $
                    date:   matches.date(ok), $
                    satid:  matches.satid(ok), $
                    lat:    matches.lat(ok), $
                    lon:    matches.lon(ok), $
                    hangle: matches.hangle(*,ok), $
                    hdata:  matches.hdata(*,*,ok), $
                    b1angle:matches.b1angle(*,ok), $
                    fov:    matches.fov(*,*,ok)}
        ltimes=ltimes(ok)
        jtimes=jtimes(ok)
        output = {matches:matches1, $
                  ltimes:ltimes, $
                  jtimes:jtimes}
;help,jtimes
        ichan=chan
    endif else begin
        return_value=-3
    endelse
endif else begin
    return_value=-2
endelse

end
;***
