;****I* idl/idl.inst2er
; NAME
; inst2er
; 
; PURPOSE
; INST2ER ACCEPTS THE SINGLE PRECISION ROLL, PITCH AND YAW ANGLES
; OF AN INSTRUMENT AND RETURNS THE DOUBLE PRECISION INSTRUMENT TO
; EARTH COORDINATES TRANSFORMATION MATRIX.
; 
; WE COMPUTE INSTRUMENT TO BODY COORDINATES TRANSFORMATION
; MATRIX BY USING A SMALL ANGLE APPROXIMATION OF TRIGONOMETRIC
; FUNCTIONS OF THE ROLL, PITCH AND YAW.
;
; USAGE
; INPUT
; OUTPUT
; RESULT
; EXAMPLE
; USES
; * Nothing.
; 
; USED BY
; * b1u_imc.pro
; * lmodel.pro
; 
; NOTES
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; CREATION DATE
; Thu Dec  3 16:17:05 EST 2009
; HISTORY
; SOURCE

pro inst2er,r,p,y,a,at
;INST2ER ACCEPTS THE SINGLE PRECISION ROLL, PITCH AND YAW ANGLES
;OF AN INSTRUMENT AND RETURNS THE DOUBLE PRECISION INSTRUMENT TO
;EARTH COORDINATES TRANSFORMATION MATRIX.

;WE COMPUTE INSTRUMENT TO BODY COORDINATES TRANSFORMATION
;MATRIX BY USING A SMALL ANGLE APPROXIMATION OF TRIGONOMETRIC
;FUNCTIONS OF THE ROLL, PITCH AND YAW.

rpy = dblarr(3,3)

RPY(1-1,1-1)=1.0D0-0.50D0*(P*P+Y*Y)
RPY(1-1,2-1)=-Y
RPY(1-1,3-1)=P
RPY(2-1,1-1)=Y+P*R
RPY(2-1,2-1)=1.0D0-0.50D0*(Y*Y+R*R)
RPY(2-1,3-1)=-R
RPY(3-1,1-1)=-P+R*Y
RPY(3-1,2-1)=R+P*Y
RPY(3-1,3-1)=1.0D0-0.50D0*(P*P+R*R)

;MULTIPLICATION OF MATRICES A AND RPY
at = dblarr(3,3)
for i = 0,2 do begin ;DO 20 I=1,3
    for j = 0,2 do begin ;DO 10 J=1,3
        AT(I,J)=A(I,1-1)*RPY(1-1,J)+A(I,2-1)*RPY(2-1,J)+A(I,3-1)*RPY(3-1,J)
    endfor                      ;10 CONTINUE
endfor                          ;20 CONTINUE

end
;***
