module kmod_constants
!****h* /kmod_constants
! NAME
! kmod_constants
! SYNOPSIS
! Used for reading and navigating McIDAS Area format data
! USES
! none!
!***
  implicit none
!****d* kmod_constants/EnchiladaDataDir
! NAME
! EnchiladaDataDir
! SYNOPSIS
! Location of data files used in some of the routines.
! SOURCE

  character(len=33) :: EnchiladaDataDir = '/net/haze/home/kknapp/for/fixins/'
!***
  
  !Define numerical constants
  real(kind=8), PARAMETER ::  dpi = 3.141592653589793116d0
  real(kind=4), PARAMETER ::   pi = 3.141592653589793e0
  real(kind=8), PARAMETER :: drad = 180.d0/dpi
  real(kind=4), PARAMETER ::  rad = 180.e0/ pi
  integer(kind=4),parameter, dimension(12) :: daysinmonth = (/31,28,31,30,31,30,&
       &                                                      31,31,30,31,30,31/)
  integer(kind=4),parameter, dimension(12) :: daysinmonth_leap = (/31,29,31,30,31,30,&
       &                                                           31,31,30,31,30,31/)

  !Define geophysical constants
!****d* kmod_constants/ae
! NAME
! ae
! SYNOPSIS
! This is the value of the equatorial radius used by many previous
! routines developed at NESDIS. It is used here for continuity.
! SOURCE

  real(kind=8), PARAMETER :: ae = 6378.144e0 !radius of earth at Equator (km)
!***
!****d* kmod_constants/be_
! NAME
! be
! SYNOPSIS
! This is the value of the polar radius used by many previous
! routines developed at NESDIS. It is used here for continuity.
! SOURCE

  real(kind=8), PARAMETER :: be = 6356.759e0 !radius of earth at Poles   (km)
!***
!****d* kmod_constants/gme
! NAME
! gme
! SYNOPSIS
! Orbital constant used in deriving position from kepler
! elements, in units of m^3 / s^2
! NOTES
! Found in Kidder & Vonder Haar 1995: Introduction to Satellite Meteorology
! SOURCE

  real(kind=8), PARAMETER :: gme = 3.986005d14 !orbital constant (m^3 / s^2) 
!***
!****d* kmod_constants/j2
! NAME
! j2
! SYNOPSIS
! Quadropole gravitational coefficient of Earth
! NOTES
! Found in Kidder & Vonder Haar 1995: Introduction to Satellite Meteorology
! SOURCE

  real(kind=8), PARAMETER :: j2 = 1.08263d-3   
!***
!****d* kmod_constants/ree
! NAME
! ree
! SYNOPSIS
! Equatorial radius of the Earth in km.
! NOTES
! Found in Kidder & Vonder Haar 1995: Introduction to Satellite Meteorology
! SOURCE
!
  real(kind=8), PARAMETER :: ree = 6.378137d3  
!***
  real(kind=8), PARAMETER :: omf2 = (be / ae)*(be / ae)


end module kmod_constants
