;****I* idl/idl.ksmooth
; NAME
; ksmooth
; 
; PURPOSE
; To simulate the ISL SMOOTH function, but allow missing data as well
; as use a binoial filter (optional).
; 
; USAGE
; INPUT
; OUTPUT
; RESULT
; EXAMPLE
; USES
; * Nothing.
; 
; USED BY
; * b1qc.pro
; 
; NOTES
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; 
; HISTORY
; SOURCE

function ksmooth,data,flag,w, $
                 binomial=binomial
;calculate the smooth values without the missing data
; FLAG=0 means ignore the point
;

if not(keyword_set(binomial)) then begin
    s=size(data)
    if (s(0) ne 2) then begin
        print,'ksmooth currently only works for 2-D imagery'
        print,'Use the BINOMIAL flag'
        return,0
    endif
    nx=s(1)
    ny=s(2)
    ii=lindgen(nx,ny) mod nx
    jj=lindgen(nx,ny)  /  nx
    
    num=data*0.
    den=data*0.
    for di=-w,w do begin
        for dj=-w,w do begin
            num=num+data[ii+di,jj+dj]
            den=den+flag[ii+di,jj+dj]
        endfor
    endfor
    out=num/(den>1.)
    return,out
endif




;filter applies a binomial filter
if keyword_set(binomial) then begin 
    n = n_elements(data)
    ind=lindgen(n)
    d = data
    f = flag
    k = 1
    for iw = 1,w do begin
        num = fltarr(n)
        den = fltarr(n)
        for i=0,1 do begin
            num = num + f(ind+i*k) * d(ind+i*k)
            den = den + f(ind+i*k)
        endfor
        k = -1 * k
        d = num / (den>1)
        none=where(den eq 0.,nnone)
        f(*) = 1
        if (nnone gt 0) then begin
            num(none) = -1.
            f(none) = 0.
        endif   
    endfor
    
    s=size(data)
    if (s(0) eq 2) then d=reform(d,s(1),s(2))
    return,d
endif

end
;***
