;****I* idl/idl.lpoint
; NAME
; lpoint
; 
; PURPOSE
; THIS SUBROUTINE CONVERTS THE INSTRUMENT ELEVATION AND SCAN
; ANGLES TO THE RELATED GEOGRAPHIC LATITUDE AND LONGITUDE.
; 
; USAGE
; INPUT
; OUTPUT
; RESULT
; EXAMPLE
; USES
; * atan2.pro
; * elcons.pro
; * elcomm.pro
; 
; USED BY
; * sndelo.pro
; * b1u_imc.pro
; 
; NOTES
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; CREATION DATE
; Thu Dec  3 16:17:05 EST 2009
; HISTORY
; SOURCE

pro lpoint,ALPHA,ZETA,RLAT,RLON,IERR
;THIS SUBROUTINE CONVERTS THE INSTRUMENT ELEVATION AND SCAN
;ANGLES TO THE RELATED GEOGRAPHIC LATITUDE AND LONGITUDE.

@elcons.pro
@elcomm.pro

ierr=1

;COMPUTES TRIGONOMETRIC FUNCTIONS OF THE SCAN AND ELEVATION                
;ANGLES CORRECTED FOR THE ROLL AND PITCH MISALIGNMENTS                     
CA=COS(ALPHA)
SA=SIN(ALPHA)
CZ=COS(ZETA)
DA=ALPHA-PMA*SA*(1.0D0/CZ+TAN(ZETA))-RMA*(1.0D0-CA/CZ)
DZ=ZETA+RMA*SA                                                            
;CORRECTED SCAN ANGLE                             
CZ=COS(DZ)
;COMPUTES POINTING VECTOR IN INSTRUMENT COORDINATES                        
G(1-1)=    SIN(DZ)
G(2-1)=-CZ*SIN(DA)
G(3-1)= CZ*COS(DA)
;TRANSFORMS THE POINTING VECTOR TO EARTH FIXED COORDINATES                 
G1(1-1)=BT(1-1,1-1)*G(1-1)+BT(1-1,2-1)*G(2-1)+BT(1-1,3-1)*G(3-1)
G1(2-1)=BT(2-1,1-1)*G(1-1)+BT(2-1,2-1)*G(2-1)+BT(2-1,3-1)*G(3-1)
G1(3-1)=BT(3-1,1-1)*G(1-1)+BT(3-1,2-1)*G(2-1)+BT(3-1,3-1)*G(3-1)
;COMPUTES COEFFICIENTS AND SOLVES A QUADRATIC EQUATION TO                  
;FIND THE INTERSECT OF THE POINTING VECTOR WITH THE EARTH                  
;SURFACE                                                                   
Q1=G1(1-1)^2+G1(2-1)^2+AEBE2*G1(3-1)^2
Q2=XS(1-1)*G1(1-1)+XS(2-1)*G1(2-1)+AEBE2*XS(3-1)*G1(3-1)
D=Q2*Q2-Q1*Q3
IF (DABS(D).LT.1.D-9) D=0.0D0
;IF THE DISCIMINANTE OF THE EQUATION, D, IS NEGATIVE, THE
;INSTRUMENT POINTS OFF THE EARTH
IF (D LT 0.0D0) THEN begin
    RLAT=999999.0D0
    RLON=999999.0D0
ENDIF else begin
    D=SQRT(D)                                                                
    ;SLANT DISTANCE FROM THE SATELLITE TO THE EARTH POINT                      
    H=-(Q2+D)/Q1                                                              
    ;CARTESIAN COORDINATES OF THE EARTH POINT                                  
    U(1-1)=XS(1-1)+H*G1(1-1)
    U(2-1)=XS(2-1)+H*G1(2-1)
    U(3-1)=XS(3-1)+H*G1(3-1)
    ;SINUS OF GEOCENTRIC LATITUDE                                              
    D1=U(3-1)/SQRT(U(1-1)^2+U(2-1)^2+U(3-1)^2)
    ;GEOGRAPHIC (GEODETIC) COORDINATES OF THE POINT                            
    RLAT=ATAN(AEBE2*D1/SQRT(1.0D0-D1*D1))
    RLON=ATAN2(U(2-1),U(1-1))
    IERR=0                                                                    
endelse
end
;***
