;****I* idl/idl.match_b1u_poes
; NAME
; match_b1u_poes
; 
; PURPOSE
; Set up files which provide a mapping between ISCCP B1U files and
; other non-geostationary data.
; 
; USAGE
; 
; INPUT
; Nothing.
; 
; RESULT
; Creates files in getenv('ISCCPDIR')+'match-poes/'
; 
; USES
; 
; USED BY
; Numerous algorithms which require colocating ISCCP B1U files in
; time with other satellites (either GEO or polar).
; * isscp_b1u_production
; 
; NOTES
; Currently includes the following satellites:
; 1. ISCCP B1U     from getenv('ISCCPDIR')+'b1u'
; 2. HIRS Level 1C from getenv('DATDIR')+'/hirs/l1c/'
; 3. ISCCP B2      from getenv('ISCCPDIR')+'b2/'
; 4. HIRS Pathfinder All Sky from getenv('DATDIR')+'/hirs/path/as/'
; 5. SSM/I netCDF  from /net/mirage/raid1a/data/ssmi_netcdf/
;                       /net/mirage/raid1b/data/ssmi_netcdf/
; 6. SSM/IS netCDF from /net/mirage/raid1a/data/ssmi_netcdf/
;                       /net/mirage/raid1b/data/ssmi_netcdf/
; 
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; * Unknown
; 
; HISTORY
; * 26 Jun07 - Added this documentation
; 
; SOURCE

pro match_b1u_poes

year1 = 1978
year2 = 2010
ddir = '/net/glaze/'

;get list of B1U files
print,'Get B1U'
b1udir = getenv('ISCCPDIR')+'b1u/'
print,b1udir
filters = ['/*.B1U.*.???','/*.B1U.*.???.??']

for iyear=year1,year2 do begin
    for icomp=0,1 do begin
        files = file_search(b1udir+strtrim(iyear,2)+filters(icomp),count=nfiles)
        if (nfiles gt 0) then print,iyear,nfiles
        files=ddir+files
        if (nfiles gt 0) then begin
            if (n_elements(b1ufiles) eq 0) then begin
                b1ufiles = files
                nbf = nfiles
            endif else begin
                b1ufiles = [b1ufiles,files]
                nbf=nbf+nfiles
            endelse
        endif
    endfor
endfor
;get list of hirs L1C files
print,'Get HIRS L1C'            ;These files are gone - replaced by HIRS Pathfinder -
;                                however, I leave it here to keep from needing to
;                                rewrite code that depends on this.
l1cdir = getenv('DATDIR')+'/hirs/l1c/'
print,l1cdir
ncf = 0
for iyear=year1,year2 do begin
    files = file_search(l1cdir+strtrim(iyear,2)+'/nss*',count=nfiles)
    if (nfiles gt 0) then print,iyear,nfiles
    files=ddir+files
    if (nfiles gt 0) then begin
        if (n_elements(l1cfiles) eq 0) then begin
            l1cfiles = files
            ncf = nfiles
        endif else begin
            l1cfiles = [l1cfiles,files]
            ncf=ncf+nfiles
        endelse
    endif
endfor
;if (ncf eq 0) then stop

;get list of ISCCP B2 files
print,'Get ISCCP B2'
b2dir = getenv('ISCCPDIR')+'b2/'
print,b2dir
for iyear=year1,year2 do begin
    files = file_search(b2dir+strtrim(iyear,2)+'/ISCCP.B2.*',count=nfiles)
    print,iyear,nfiles
    files=ddir+files
    if (nfiles gt 0) then begin
        if (n_elements(b2files) eq 0) then begin
            b2files = files
            nb2f = nfiles
        endif else begin
            b2files = [b2files,files]
            nb2f=nb2f+nfiles
        endelse
    endif
endfor
if (nb2f eq 0) then stop

;get list of hirs Pathfinder Version 2 Clear Sky files
print,'GET HIRS Pathfinder All Sky - on glaze'
csdir = getenv('DATDIR')+'/hirs/path/as/'
print,csdir
npf = 0
for iyear=year1,year2 do begin
    files = file_search(csdir+strtrim(iyear,2)+'/HIRS*',count=nfiles)
    files=ddir+files
    if (nfiles gt 0) then begin
        if (n_elements(csfiles) eq 0) then begin
            csfiles = files
            npf = nfiles
        endif else begin
            csfiles = [csfiles,files]
            npf=npf+nfiles
        endelse
    endif
endfor
if (npf eq 0) then stop

print,'GET HIRS Pathfinder All Sky - on crystal (Lei''s files)'
lsdir = '/net/crystal/san3/hirs/hirsas/'
print,lsdir
lsfiles = file_search(lsdir,'HIRS.*.V22.AS',count=nls)

if (nls eq 0) then stop
;stop
csfiles = [csfiles,lsfiles]
npf=npf+nls



;get list of SSMI files (for Hilawe)
;I have to loop over year since the years are symbollic links
print,'Getting SSM/I information'
ssmidir='/net/sleet/raid1/hsemuneg/data/ssmi_netcdf/sdr/'
print,ssmidir
nssmi=0
for iyear=year1,year2 do begin
    files = file_search(ssmidir+strtrim(iyear,2)+'/*SDRR*.nc',count=nfiles,/nosort)
    print,iyear,nfiles
    if (nfiles gt 0) then begin
        if (n_elements(ssmifiles) eq 0) then begin
            ssmifiles = files
            nssmi = nfiles
        endif else begin
            ssmifiles = [ssmifiles,files]
            nssmi=nssmi+nfiles
        endelse
    endif
endfor
if (nssmi eq 0) then stop


;get list of SSMIS files (for Hilawe)
print,'Getting SSM/IS information'
ssmisdir=ssmidir                ;'/net/mirage/raid1a/data/ssmi_netcdf/'
print,ssmisdir

nssmis=0
for iyear=year1,year2 do begin
    files = file_search(ssmidir+strtrim(iyear,2)+'/*SDRN*.nc',count=nfiles,/nosort)
    print,iyear,nfiles
    if (nfiles gt 0) then begin
        if (n_elements(ssmisfiles) eq 0) then begin
            ssmisfiles = files
            nssmis = nfiles
        endif else begin
            ssmisfiles = [ssmisfiles,files]
            nssmis=nssmis+nfiles
        endelse
    endif
endfor
if (nssmis eq 0) then stop

;...include those still on mirage
;print,'Getting SSM/IS information'
;print,ssmidir
;ssmisfilesm=file_search(ssmidir,'*SDRN*.nc',count=nssmims)
;if (nssmims ne 0) then begin
;;    nssmis=nssmis+nssmims
;;    ssmisfiles=[ssmisfiles,ssmisfilesm]
;    nssmis=nssmims
;    ssmisfiles=ssmisfilesm
;endif else begin
;    print,'No SSM/IS files found on raid1b'
;    stop
;endelse


;Get synoptic times for all b1u files
print,'Get Synoptic Times'
b1time = dblarr(nbf)
for i=0l,nbf-1 do begin
    file=b1ufiles(i)
    f=strsplit(file,'.',/extr)
    b1time(i) = julday(f(5),f(6),f(4),f(7)/100,f(7) mod 100,0)
endfor
b1time_syn = round(b1time * 8.0d0)/8.0d0

;Get times of the Pathfinder files
print,'Get Pathfinder times'
if (npf gt 0) then begin
    cstime1 = dblarr(npf)
    cstime2 = dblarr(npf)
endif
for i=0l,npf-1 do begin
    f = strsplit(csfiles(i),'.',/extr)
    yy = strsplit(f(2),'Y',/extr)
    year = yy(0)
    year = year + 1900 + 100*(year lt 75)
;    if (year lt 75) then year=year+2000 else year=year+1900
    jjj = strsplit(f(3),'D',/extr)
    jjj = jjj(0)
    cstime1(i) = julday(1,jjj  ,year,0,0,0)
    cstime2(i) = julday(1,jjj+1,year,0,0,0)
endfor

;Get times of the Level 1C files
print,'Get Level 1C times'
if (ncf gt 0) then begin
    l1time1 = dblarr(ncf)
    l1time2 = dblarr(ncf)
endif
for i=0l,ncf-1 do begin
    f = strsplit(l1cfiles(i),'.',/extr)
    yyjjj = strsplit(f(3),'d',/extr)
    year = yyjjj(0)/1000
    if (year lt 75) then year=year+2000 else year=year+1900
    jjj = yyjjj(0) mod 1000
    time1 = strsplit(f(4),'s',/extr)
    time2 = strsplit(f(5),'e',/extr)
    if (time2(0) lt time1(0)) then d = 1.0d0 else d = 0.0d0
    l1time1(i) = julday(1,jjj  ,year,time1(0)/100,time1(0) mod 100,0)
    l1time2(i) = julday(1,jjj+d,year,time2(0)/100,time2(0) mod 100,0)
endfor

;Get times of the b2 files
print,'Get ISCCP B2 times'
if (nb2f gt 0) then begin
    b2time1 = dblarr(nb2f)
    b2time2 = dblarr(nb2f)
endif
for i=0l,nb2f-1 do begin
    f = strsplit(b2files(i),'.',/extr)
    y = fix(f(4))
    m = fix(f(5))
    d = fix(f(6))
    hhmm = fix(f(7))
    b2time1(i) = julday(m,d,y,hhmm/100,hhmm mod 100, 0)
endfor
;the end time for the ISCCP files is missing, so will use a 102 minute
;orbit as a rule of thumb ... it fits for most files.
b2time2 = b2time1 + 102.0d0 / 60.d0 / 24.0d0

;get SSM/I times
print,'Get SSM/I times'
if (nssmi gt 0) then begin
    ssmitime1=dblarr(nssmi)
    ssmitime2=ssmitime1
    for i=0l,nssmi-1 do begin
        f=strsplit(ssmifiles(i),'.',/extr)
        year = fix(strmid(f(3),1,2))
        doy  = fix(strmid(f(3),3,3))
        year = year + 1900 + 100 * (year lt 70)
        hhmm1 = fix(strmid(f(4),1,4))
        hhmm2 = fix(strmid(f(5),1,4))
        ssmitime1(i)=julday(1,doy,year,hhmm1/100,hhmm1 mod 100,0)
        ssmitime2(i)=julday(1,doy,year,hhmm2/100,hhmm1 mod 100,0)
        ssmitime2(i) = ssmitime2(i) + 1.0d0 * (hhmm2 lt hhmm1)
;        if (hhmm2 lt hhmm1) then ssmitime2(i) = ssmitime2(i) + 1.0d0
    endfor
endif
;get SSM/IS times
print,'Get SSM/IS times'
if (nssmis gt 0) then begin
    ssmistime1=dblarr(nssmis)
    ssmistime2=ssmistime1
    for i=0l,nssmis-1 do begin
        f=strsplit(ssmisfiles(i),'.',/extr)
        year = fix(strmid(f(3),1,2))
        doy  = fix(strmid(f(3),3,3))
        year = year + 1900 + 100 * (year lt 70)
        hhmm1 = fix(strmid(f(4),1,4))
        hhmm2 = fix(strmid(f(5),1,4))
        ssmistime1(i)=julday(1,doy,year,hhmm1/100,hhmm1 mod 100,0)
        ssmistime2(i)=julday(1,doy,year,hhmm2/100,hhmm1 mod 100,0)
        ssmistime2(i) = ssmistime2(i) + 1.0d0 * (hhmm2 lt hhmm1)
;        if (hhmm2 lt hhmm1) then ssmistime2(i) = ssmistime2(i) + 1.0d0
    endfor
endif


;sort B1U files by time and output all files with same synoptic time
;along with matching POES files
s = sort(b1time_syn)
u = uniq(b1time_syn(s))

nu = n_elements(u)
outdir = getenv('ISCCPDIR')+'match-poes/'
print,'Make Match files'
for iu=0l,nu-1 do begin
    i1 = 0
    if (iu gt 0) then i1 = u(iu-1)+1
    i2 = u(iu)
    tim = b1time_syn(s(u(iu)))
    caldat,tim,mon,day,year,hour,min,sec
    nlmatch = 0 & npmatch=0 & nbmatch = 0
    
                                ;No more L1C data files
                                ;Lmatch = where(  l1time1 lt (tim + 2.0d0/24.0d0) and $
                                ;                 l1time2 gt (tim - 2.0d0/24.0d0),nlmatch)
    nlmatch=0
    Bmatch = where(  b2time1 lt (tim + 2.0d0/24.0d0) and $
                     b2time2 gt (tim - 2.0d0/24.0d0),nbmatch)
    Pmatch = where(  cstime1 lt (tim + 2.0d0/24.0d0) and $
                     cstime2 gt (tim - 2.0d0/24.0d0),nPmatch)
                                ;SSM/I
    Smatch = where(ssmitime1 lt (tim + 2.0d0/24.0d0) and $
                   ssmitime2 gt (tim - 2.0d0/24.0d0),nSmatch)
                                ;SSM/IS
    SSmatch= where(ssmistime1 lt (tim + 2.0d0/24.0d0) and $
                   ssmistime2 gt (tim - 2.0d0/24.0d0),nSSmatch)
    

;    if (nlmatch gt 0 or $
;        npmatch gt 0 or $
;        nbmatch gt 0 or $
;        nsmatch) then begin
    
    openw,1,outdir+strtrim(year,2)+'/B1U-POES.'+ $
          strjoin([strtrim(year,2), $
                   string(mon,format='(i2.2)'), $
                   string(day,format='(i2.2)'), $
                   string(hour,format='(i2.2)')],'.')+'.match'
    printf,1,strtrim(i2-i1+1,2)+'               # B1U files'
    printf,1,b1ufiles(s(i1:i2))
    printf,1,strtrim(nbmatch,2) +'               # ISCCP B2 files'
    if (nbmatch gt 0) then printf,1,b2files(Bmatch)
    printf,1,strtrim(nlmatch,2) +'               # Level 1C files'
    if (nlmatch gt 0) then printf,1,l1cfiles(Lmatch)
    printf,1,strtrim(npmatch,2) +'               # HIRS Pathfinder Clear Sky'
    if (npmatch gt 0) then printf,1,csfiles(pmatch)
    printf,1,strtrim(nSmatch,2) +'               # SSMI netCDF files'
    if (nSmatch gt 0) then printf,1,ssmifiles(Smatch)
    printf,1,strtrim(nSSmatch,2) +'               # SSMIS netCDF files'
    if (nSSmatch gt 0) then printf,1,ssmisfiles(SSmatch)
    close,1
;    endif
endfor

stop



end
;***
