;****I* idl/idl.parallax_correct_swath
; NAME
; parallax_correct_swath
; 
; PURPOSE
; To correct satellite data for parallax. In particular, GEO data.
; 
; USAGE
; 
; INPUT
; btemp - input brightness temperature. (Kelvin)
; gvza -  input view zenith angle (degrees)
; gazi -  input azimuth angle (degrees)
; glat -  input latitude (degrees North)
; btemp_corr - Parallax corrected temperatures (Kelvin)
; remap - Indices used in remapping data.    
; 
; KEYWORDS
; This needs some work.
; 
; OUTPUT
; 
; RESULT
; 
; EXAMPLE
; 
; USES
; * timer.pro (these calls are primarily for developement, so they could
;              be removed)
; * mean.pro
; 
; USED BY
; * b1u_2_grisat.pro
; 
; NOTES
; 
; TODO
; 
; BUGS
; 
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; 2008
; 
; HISTORY
; 
; SOURCE

pro parallax_correct_swath,btemp,gvza,gazi,glat,btemp_corr,remap, $
  debug=debug, $
  ang=ang, $
  fill=fill, $
  height=height, $
  doy=doy,$                     ;day of year
  tropical=tropical, $
  nearby=nearby, $
  use=use, $
  nofill=nofill, $
  fast=fast, $
  dxout=dxout, $                      ;return dx
  dyout=dyout, $                      ;return dy
  donly=donly                   ;only calculate dx & dy

if not(keyword_set(ang)) then ang = 0.07
if n_elements(doy) eq 0 then doy=190 else doy=doy

;only work with valid temperatures
oktemp = where(btemp gt 100 and btemp lt 400,noktemp)

if (noktemp lt 1) then message,'No valid temperatures: '+string(range(btemp))

bt = btemp(oktemp)
vza = gvza(oktemp)
azi = gazi(oktemp)
s=size(bt)

if (s(0) eq 1) then begin
    nx = 1
    ny = s(1)
endif else if (s(0) eq 2) then begin
    nx = s(1)
    ny = s(2)
endif else begin
    message,'Strange image coordinate size: '+string(s)
endelse

lat=glat(oktemp)

;need standard atmosphere
;McClatchey Standard atmosphere ... from SBDART code.
nl = 33
z = [0.,    1.,    2.,    3.,    4.,    5.,    6.,    7.,    8.,   $
     9.,   10.,   11.,   12.,   13.,   14.,   15.,   16.,   17.,   $
     18.,   19.,   20.,   21.,   22.,   23.,   24.,   25.,   30.,   $          
     35.,   40.,   45.,   50.,   70.,  100.]
;tropical
t_trop = [  3.000e+02,2.940e+02,2.880e+02,2.840e+02,2.770e+02,2.700e+02,      $
            2.640e+02,2.570e+02,2.500e+02,2.440e+02,2.370e+02,2.300e+02,      $
            2.240e+02,2.170e+02,2.100e+02,2.040e+02,1.970e+02,1.950e+02,      $
            1.990e+02,2.030e+02,2.070e+02,2.110e+02,2.150e+02,2.170e+02,      $
            2.190e+02,2.210e+02,2.320e+02,2.430e+02,2.540e+02,2.650e+02,      $
            2.700e+02,2.190e+02,2.100e+02]
;mid-latitude Summer
t_mls = [  2.940e+02,2.900e+02,2.850e+02,2.790e+02,2.730e+02,2.670e+02,      $
           2.610e+02,2.550e+02,2.480e+02,2.420e+02,2.350e+02,2.290e+02,      $
           2.220e+02,2.160e+02,2.160e+02,2.160e+02,2.160e+02,2.160e+02,      $
           2.160e+02,2.170e+02,2.180e+02,2.190e+02,2.200e+02,2.220e+02,      $
           2.230e+02,2.240e+02,2.340e+02,2.450e+02,2.580e+02,2.700e+02,      $
           2.760e+02,2.180e+02,2.100e+02]
;mid-latitude winter
t_mlw = [2.722e+02,2.687e+02,2.652e+02,2.617e+02,2.557e+02,2.497e+02,      $
         2.437e+02,2.377e+02,2.317e+02,2.257e+02,2.197e+02,2.192e+02,      $
         2.187e+02,2.182e+02,2.177e+02,2.172e+02,2.167e+02,2.162e+02,      $
         2.157e+02,2.152e+02,2.152e+02,2.152e+02,2.152e+02,2.152e+02,      $
         2.152e+02,2.152e+02,2.174e+02,2.278e+02,2.432e+02,2.585e+02,      $
         2.657e+02,2.307e+02,2.102e+02]
;sub-arctic summer
t_sas = [2.870e+02,2.820e+02,2.760e+02,2.710e+02,2.660e+02,2.600e+02,    $  
         2.530e+02,2.460e+02,2.390e+02,2.320e+02,2.250e+02,2.250e+02,      $
         2.250e+02,2.250e+02,2.250e+02,2.250e+02,2.250e+02,2.250e+02,      $
         2.250e+02,2.250e+02,2.250e+02,2.250e+02,2.250e+02,2.250e+02,      $
         2.260e+02,2.280e+02,2.350e+02,2.470e+02,2.620e+02,2.740e+02,      $
         2.770e+02,2.160e+02,2.100e+02]
;sub-arctic winter
t_saw = [2.571e+02,2.591e+02,2.559e+02,2.527e+02,2.477e+02,2.409e+02,      $
         2.341e+02,2.273e+02,2.206e+02,2.172e+02,2.172e+02,2.172e+02,      $
         2.172e+02,2.172e+02,2.172e+02,2.172e+02,2.166e+02,2.160e+02,      $
         2.154e+02,2.148e+02,2.141e+02,2.136e+02,2.130e+02,2.124e+02,      $
         2.118e+02,2.112e+02,2.160e+02,2.222e+02,2.347e+02,2.470e+02,      $
         2.593e+02,2.457e+02,2.100e+02]
;US Std Atm 62
;     &  2.881e+02,2.816e+02,2.751e+02,2.687e+02,2.622e+02,2.557e+02,      
;     &  2.492e+02,2.427e+02,2.362e+02,2.297e+02,2.232e+02,2.168e+02,      
;     &  2.166e+02,2.166e+02,2.166e+02,2.166e+02,2.166e+02,2.166e+02,      
;     &  2.166e+02,2.166e+02,2.166e+02,2.176e+02,2.186e+02,2.196e+02,      
;     &  2.206e+02,2.216e+02,2.265e+02,2.365e+02,2.534e+02,2.642e+02,      
;     &  2.706e+02,2.197e+02,2.100e+02/                          

;set up variables

;using hard bounds in lat and time creates discontinuities
;using weights makes them smoother

;use a cos function to transition from summer to winter
;170 ~ June 21 ~ summer solstice
wgt_sum = cos(doy-170.)*2.*!pi/365. *0.5 + 0.5
wgt_win = 1.-wgt_sum
;tropical weight
;plot,1-findgen(60)/25.>0
;mid latitude weight
;oplot,(1.-abs(findgen(60)-35)/20.)>0.
;arctic weight
;oplot,(findgen(60)/15.-3)>0.



;Break the work into 4 portions
nport=4
index = [round(findgen(nport)/float(nport) * noktemp),noktemp]

final_hgt = fltarr(noktemp)

for iport = 0,nport-1 do begin
    i1 = index(iport)
    i2 = index(iport+1)-1
;    print,i1,i2
    ny = i2-i1+1

;calculations do not depend on longitude
alat = abs(lat(i1:i2))
wgt_trop = (1. - alat/25.)>0.
wgt_ml   = (1.-abs(alat-35.)/15.)>0.
wgt_arc  = (alat/15.-3.)>0.

if keyword_set(tropical) then begin
    wgt_trop = wgt_trop * 0. + 1.
    wgt_ml   = wgt_ml   * 0.
    wgt_arc  = wgt_arc  * 0.
endif

den = (wgt_trop + $
       wgt_ml * wgt_sum + $
       wgt_ml * wgt_win + $
       wgt_arc* wgt_sum + $
       wgt_arc* wgt_win)


;the following is only a function of latitude.
;so perform all calculations by ignoring longitude

;print,'trop',(memory())/1024.^2
t = rebin(reform(wgt_trop,ny,1),ny,18) * rebin(reform(t_trop(0:17),1,18),ny,18)
;help,t
;stop
;print,'mls',(memory())/1024.^2
;stop
t = temporary(t) + $
    rebin(reform(wgt_ml,ny,1),ny,18) * wgt_sum * rebin(reform(t_mls(0:17),1,18),ny,18)
;help,t
;print,'mlw',(memory())/1024.^2
t = temporary(t) + $
    rebin(reform(wgt_ml,ny,1),ny,18) * wgt_win * rebin(reform(t_mlw(0:17),1,18),ny,18)
;print,'sas',(memory())/1024.^2
t = temporary(t) + $
    rebin(reform(wgt_arc,ny,1),ny,18)* wgt_sum * rebin(reform(t_sas(0:17),1,18),ny,18)
;print,'saw',(memory())/1024.^2
t = temporary(t) + $
    rebin(reform(wgt_arc,ny,1),ny,18)* wgt_win * rebin(reform(t_saw(0:17),1,18),ny,18)

prof = reform(t / rebin(reform(den,1,ny,1),1,ny,18))

;help,prof

if keyword_set(debug) then timer,/start
;assign heights to original map
;t1 = btemp * 0.
;t2 = btemp * 0.
;z1 = btemp * 0.
;z2 = btemp * 0.
;hgt=btemp*0

glon = where(bt gt 100 and $
             bt lt 400,nglon)

iz1 = (total(prof gt rebin(reform(bt(i1:i2),ny,1),ny,18),2)-1)>0<16
t1 = prof(lindgen(ny),iz1)
t2 = prof(lindgen(ny),iz1+1)
z1 = z(iz1)
z2 = z(iz1+1)

;for ig=0l,nglon-1 do begin
;        i = glon(ig)
;        profile = reform(prof(0,i,*))

;        iz1 = (total(profile gt bt(i))-1)>0<16
;        t1(i) = profile(iz1)
;        z1(i) = z(iz1)
;        t2(i) = profile(iz1+1)
;        z2(i) = z(iz1+1)
;        ;spline interpolation is a bit overkill
;        ;hgt(i,j) = interpol(z(0:13),t(0:13),bt(i,j),/spline)
;        ;hgt(i,j) =  hgt(i,j) * (bt(i,j) ge t(13)) + $
;        ;            z(13)    * (bt(i,j) lt t(13))
;        ;if (bt(i,j) lt t(13)) then hgt(i,j) = z(13)
;endfor
;help,z1
if keyword_set(debug) then timer,/stop,/print

;using linear interpolation
hgt = (z1 + (z2-z1) * (bt(i1:i2)-t1) / (t2-t1)) > 0. < 16.
m=where(t1 eq t2,nm)
if (nm gt 0) then hgt(m) = z1(m)


final_hgt(i1:i2) = hgt
endfor

hgt = final_hgt
final_hgt=0

;stop
;remove noise by rounding clouds 3 km or less to the surface
low=where(hgt le 3.,nlow)
if (nlow gt 0) then hgt(low) = 0.

;zero out bad values
badbt = where(bt lt 100,nbadbt)
if (nbadbt gt 0) then hgt(badbt) = -1

;calculate remapping distances = d

if keyword_set(nearby) then begin
    if (n_elements(hgt(nearby)) ne n_elements(vza)) then stop 
    d = hgt*0.
    d(nearby) = (hgt(nearby)>0.) * tan(vza*!pi/180.)
    dx=hgt*0
    dy=dx
    dx(nearby) = d(nearby) * sin(azi*!pi/180.)
    dy(nearby) = d(nearby) * cos(azi*!pi/180.)    
endif else begin
    if (n_elements(hgt) ne n_elements(vza)) then stop 
    d = (hgt>0.) * tan(vza*!pi/180.)
;   calculate remapped indicies
    dx = d * sin(azi*!pi/180.)
    dy = d * cos(azi*!pi/180.)
endelse

if (nbadbt gt 0) then begin
    d(badbt) = 0.01
    dx(badbt) = 0.01
    dy(badbt) = 0.01
endif    

;convert distance from km to pixels (0.07deg lat)
dy = dy / 111.12 / ang

;dx is based on the circumference at the latitude of the point. 
; km/deg = 2*pi*r/360
; r = 6378.*cos(lat*!pi/180.)
circ = 2. * !pi * 6378.*cos(lat*!pi/180.) / 360.
dx = dx / circ / ang
if (nbadbt gt 0) then dx(badbt) = 0.01


if keyword_set(donly) then begin
    n = n_elements(btemp) 
    dxout = fltarr(n)
    dyout = fltarr(n)
    height = fltarr(n)
    dxout(oktemp) = dx
    dyout(oktemp) = dy
;    dx=dxout
;    dy=dyout
    height(oktemp)=hgt
    return
endif

btc = bt*0.
newi = fix(bt)*0
newj = newi


if keyword_set(debug) then timer,/start
tempi = lindgen(nx,ny) mod nx    ;i
newi = round(tempi + dx)
tempj = lindgen(nx,ny)  /  nx    ;j
newj = round(tempj + dy)
ok = where(newi ge 0 and newi lt nx and $
           newj ge 0 and newj lt ny,nok)
btc(newi(ok),newj(ok)) = bt(ok)

;for i=0,nx-1 do begin
;    for j=0,ny-1 do begin
;        circ = 2. * !pi * 6378.*cos(lat(i,j)*!pi/180.) / 360.
;        dx(i,j) = dx(i,j) / circ / ang
;        newi(i,j) = round(i + dx(i,j))
;        newj(i,j) = round(j + dy(i,j))
;        if (newi(i,j) ge 0 and newi(i,j) lt nx and $
;            newj(i,j) ge 0 and newj(i,j) lt ny) then begin
;            btc(newi(i,j),newj(i,j)) = bt(i,j)
;        endif
;    endfor
;endfor
if keyword_set(debug) then timer,/stop,/print


btemp_corr = btc
remap = [[[newi]],[[newj]]]


;points for correction

if (not(keyword_set(nofill))) then begin 
    corr = where(btc lt 100 and bt gt 100,ncorr)
    stop
    
;if keyword_set(fill) then begin
    movei=round(sin(azi*!pi/180.))
    movej=round(cos(azi*!pi/180.))
    once=1
again: 
    once=once+1
    for i=0,nx-1 do begin
        for j=0,ny-1 do begin
            if (btc(i,j) lt 10.) then begin
                                ;fill in values from two point backward
                ni = i-movei(i,j)
                nj = j-movej(i,j)
                if (ni ge 0 and ni lt nx and $
                    nj ge 0 and nj lt ny) then begin
                    btc(i,j) = btc(ni,nj)
                endif
            endif
        endfor
    endfor
    if (once eq 2) then goto,again
endif
;endif
;surface,smooth(hgt,5),shades=bytscl(-(btc>200<300)),ax=85,az=-8
height=hgt
if keyword_set(debug) then stop
end
;***
