;****I* idl/idl.plot_tle
; NAME
; plot_tle
; 
; PURPOSE
; To obtain the nearest TLE (Two Line Elements) Kepler elements.
; 
; USAGE
; INPUT
; OUTPUT
; RESULT
; EXAMPLE
; USES
; * const.pro
; * geo_struct.pro
; * itime.pro
; * kep2cart.pro
; * read_tle.pro
; 
; USED BY
; * b1read_aes_g7.pro
; * b1read_csu_met3.pro
; * b1read_eum_met.pro
; * b1read_eum_msg.pro
; * b1read_jma_gms.pro
; * b1read_jma_mts.pro
; * b1read_uws_g56.pro
; 
; NOTES
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; ???
; 
; HISTORY
; Jul2010 - KRK - Added robodoc headers
; 
; SOURCE

pro plot_tle,tle,sat=sat,year=year,jday=jday,$
             postime=postime,coord=coord,$
             glat=glat,glon=glon,$
             gra=gra,out=out,ORBstr=ORBstr,days=days
;reset

@geo_struct.pro


;plots satellite track for one day
;common CONST, PI,rad, rot, gr_ang0, ae, be, gme, j2, ree
;common kepler, semima, incl, eccen, meana, rasc, perigee, $
;  edate, etime,msec
;common green,gra1,gra2,gt1,gt2

iout=0
if keyword_set(out) then iout = 1

@const.pro
gra1=0.
gra2=0.
gt1=0.
gt2 = 13.d0*3600.d0

if (not(keyword_set(jday))) then sat = 'goes-7'
if (not(keyword_set(year))) then year = 1995
if (not(keyword_Set(jday))) then jday = 280

if (iout) then print,'input'
if (iout) then print,sat,year,jday

date = year + (jday-1) / 366.
if (iout) then print,'Date:',date

;find tle nearest the day
tlefile = getenv('DATDIR')+'/tle/'+sat+'.txt'

ntle = file_lines(tlefile)/2

dtmin = 100.

openr,lun,tlefile,/get_lun
while not(eof(lun)) do begin
    read_tle,lun,tle
    dt = tle.epoch_year+tle.epoch_date/366. - date
    if (abs(dt) lt dtmin and dt lt 0) then begin
        tle_near = tle
        dtmin = abs(dt)
    endif
    if (n_elements(tle_near) eq 0 and dt gt 0) then tle_near = tle
endwhile
close,lun
free_lun,lun

if (iout) then print,'nearest:',dtmin
if (iout) then print,tle_near

tle=tle_near
semima = (7.537122734d13 / tle.motion / tle.motion)^(1./3.)
incl   = tle.incl
eccen  = tle.eccn
meana  = tle.mnan
rasc   = tle.rasc
perigee= tle.prge
edate  = (tle.epoch_year - 1900)*1000l + floor(tle.epoch_date)
etime  = itime((tle.epoch_date mod 1)*24.)
msec   = ((tle.epoch_date mod 1) * 86400.d0) mod 1.

orbit.ep_date = edate
orbit.ep_time = etime
orbit.ep_msec = msec
orbit.semima = semima
orbit.incl = incl
orbit.eccen = eccen
orbit.meana = meana
orbit.rgtasc = rasc
orbit.perigee = perigee
ORBstr = orbit

ndays = 5
nt = 1*24.*4.
dt = ndays*24.d0/double(nt)

if (iout) then begin
    sublat=fltarr(nt)
    sublon=fltarr(nt)
    gra=dblarr(nt)
endif
if (keyword_Set(postime)) then begin
    ijulday = julday(1,jday,year,0,0,0) + $
              postime/3600./24. 
    ejulday = julday(1,tle.epoch_date,tle.epoch_year,0,0,0)+tle.epoch_date mod 1
    kep = [semima, eccen, incl, $
           rasc,perigee,meana]
    dtime = (ijulday-ejulday)*24.*3600.
    days = dtime/24./3600.
    kep2cart,kep,dtime,x,y,z,sattim=ijulday, $
             glat=glat,glon=glon,grad=gra
    coord=[x,y,z]
endif
if (iout) then stop
end
;***
