;****I* idl/idl.range
; NAME
; range
; 
; PURPOSE
; To return a 2-element vector containing the minimum and maximum of
; the input array/matrix
; 
; USAGE
; minmax = range(array)
; 
; INPUT
; array = scalar or multi-dimension vector of same type as input 
; 
; OUTPUT
; 2-element array:
; 
;        range(0) = minimum of input
;        range(1) = maximum of input
;        
; RESULT
; EXAMPLE
; IDL> print,range(1)
;      1       1
; IDL> print,range(findgen(10))
;      0.00000      9.00000

; USES
; Nothing.
; 
; USED BY
; * parallax_correct_swath.pro
; * b1read_csu_g91012.pro
; * mcidas_read.pro
; 
; NOTES
; In the event that a null variable is passed, then range returns -1
; 
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; unknown
; 
; HISTORY
; 
; SOURCE

function range,ARRAY
; returns range of the array ARRAY
;range = fltarr(2)
;range(0) = min(ARRAY)
;range(1) = max(ARRAY)
if (n_elements(array) gt 0) then range = [min(array),max(array)] else range=-1
return,range
end
;***
