;****I* idl/idl.read_tle
; NAME
; read_tle
; 
; PURPOSE
; Read TLE (two line elements) from a file
; 
; USAGE
; INPUT
; unit = logical unit that was opened
; 
; OUTPUT
; tle = structure containing the tle data
;       See below for structure definition.
;       
; RESULT
; EXAMPLE
; USES
; * Nothing
; 
; USED BY
; * plot_tle.pro
; 
; NOTES
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; ???
; 
; HISTORY
; SOURCE

pro read_tle,unit,tle
;read in the next TLE from the file corresponding to the logical UNIT

tle = {two_line_element,$
       epoch_year:0,$
       epoch_date:0.d0,$
       motion1:0.d0,$
       motion2:0.d0,$
       drag:0.d0,$
       incl:0.d0,$
       rasc:0.d0,$
       eccn:0.d0,$
       prge:0.d0,$
       mnan:0.d0,$
       motion:0.d0}

line1 = '(18x,i2,d12.8,1x,d10.8,i7,i2,i7,i2)'
line2 = '(8x,f8.4,f9.4,i8,f9.4,f9.4,d12.8)'

readf,unit,format=line1,ep_y,ep_d,motion1,motion2,m2,drag,d
readf,unit,format=line2,incl,rascn,eccen,perigee,meanan,motion

tle.epoch_year = ep_y+1900.
tle.epoch_date = ep_d
tle.motion1    = motion1
tle.motion2    = motion2*10^(m2)
tle.drag       = drag * 10^(d)
tle.incl       = incl
tle.rasc       = rascn
tle.eccn       = eccen*1.d-7
tle.prge       = perigee
tle.mnan       = meanan
tle.motion     = motion
if (tle.epoch_year lt 1940) then tle.epoch_year = tle.epoch_year +100.
end
;***
