;****I* idl/idl.str2num
; NAME
; str2num
; 
; PURPOSE
; To convert a scalar or array of strings to numbers (integer, etc.)
; 
; USAGE
; 
; INPUT
; 
; OUTPUT
; 
; RESULT
; 
; EXAMPLE
; IDL> print,str2num('4flag')
;           0
; IDL> print,str2num('4')    
;           4
; IDL> print,str2num('4.d4')
;       40000.000
; IDL> print,str2num('4.e4')
;      40000.0
; USES
; - Nothing outside of what IDL has as internal routines.
; 
; USED BY
; * b1read_jma_gms.pro
; 
; NOTES
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; Unknown
; 
; HISTORY
; 
; SOURCE

function str2num,string, $
                 i1=i1,i2=i2,i4=i4,r4=r4,r8=r8, $
                 missing=missing

if keyword_set(missing) then miss = missing else miss = 0.0d0
n = n_elements(string)
s = size(string)
str = strupcase(reform(string,s(2+s(0))))

b = byte(str)

nonnum = (b ge 1 and b le 31) or $
         (b ge 33 and b le 42) or $
         (b eq 44) or (b eq 47) or $
         (b ge 58 and b le 67) or $
         (b ge 70);can throw out all larger values b/c strupcase
present_dot = total(total(b eq 46,1) gt 0)
present_e   = total(total(b eq 69,1) gt 0)
present_d   = total(total(b eq 68,1) gt 0)

bad=total(nonnum,1)

if keyword_set(i1) then num = bytarr(n)
if keyword_set(i2) then num = intarr(n)
if keyword_set(i4) then num = lonarr(n)
if keyword_set(r4) then num = fltarr(n)
if keyword_set(r8) then num = dblarr(n)
; no keyword chosen
if (n_elements(num) le 0) then begin
    nn = float(n)
    if (present_d/n gt 0.5) then num = dblarr(n) else $
      if (present_dot/n gt 0.01) then num = fltarr(n) else $
      num = lonarr(n)
endif

ss = size(num)
type = ss(2)

num(*) = miss
rep = where(bad eq 0,nrep)
if (nrep gt 0) then begin
    if (type eq 1) then $
      if (present_e gt 1 or present_d gt 1) then $
      num(rep) = byte(float(str(rep))) else num(rep) = byte(str(rep))
    if (type eq 2) then $
      if (present_e gt 1 or present_d gt 1) then $
      num(rep) = fix(float(str(rep))) else num(rep) = fix(str(rep))
    if (type eq 3) then $
      if (present_e gt 1 or present_d gt 1) then $
      num(rep) = long(float(str(rep))) else num(rep) = long(str(rep))
    if (type eq 4) then num(rep) = float(str(rep))
    if (type eq 5) then num(rep) = double(str(rep)) 
endif
if (s(0) gt 1) then num = reform(num,s(1:s(0)))
return,num
end
;***
