;****I* idl/idl.stripname
;+
; NAME:
;      STRIPNAME
;
; PURPOSE:
;      Strips path information from a file name (and possibly the file
;      extensions) and returns the string
;
; INPUTS:
;      FILENAME : String - The filename
;
; KEYWORD PARAMETERS:
;      DIR - Returns the directory structure to the DIR keyword
;      NODOT - Removes the file extension (everything after the first dot)
;
;
;
; EXAMPLE:
; IDL> print,stripname('/home/kknapp/work/lib/idl/goes/surface_sens.pro')
;         surface_sens.pro
; IDL> print,stripname('/home/kknapp/work/lib/idl/goes/surface_sens.pro',/nodot)
;         surface_sens
;
; MODIFICATION HISTORY:
;              Written by Kenneth Knapp - CIRA
;                         visiting scientist @ NESDIS/ORA
;                         now @ NESDIS/NCDC
;      08-12-2003 - Documentation header added
;
; SOURCE

function stripname,filename,dir=dir,nodot=nodot

d = '/'
dir=''
b = strpos(filename,d)
file=filename
while (b ge 0) do begin
    dir = dir + strmid(file,0,b) + '/'
    file = strmid(file,b+1,strlen(file))
    b=strpos(file,d)
endwhile
stripname=file

if (keyword_set(nodot)) then begin
    b=strpos(stripname,'.')
    stripname=strmid(stripname,0,b)
endif

return,stripname
end

;***
