;****I* idl/idl.yyjjj_2_julday
; NAME
; yyjjj_2_julday
; 
; PURPOSE
; To convert year/julian day information to Julian Date.
; 
; USAGE
; INPUT
; yyjjj = Current date in YYJJJ format
;         Can be:   Year*1000 + DOY [day of year]
;                        or
;                   (Year-100)*1000 + DOY
;                   
; KEYWORDS
; All keywords are optional
; time =    Time of day in one of the following 3 formats:
; flttime = time of day provided in fractional hours since midnight 
; itime   = time of day provided in HHMMSS
; seconds = time of day provided in seconds since midnight
; debug   = provide status output
; 
; OUTPUT
; Julian date as determined by IDL's julday routine.
; 
; RESULT
; EXAMPLE
; USES
; USED BY
; NOTES
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; ???
; 
; HISTORY
; Jul2010 - KRK - Added Robodoc information
; 
; SOURCE

function yyjjj_2_julday,yyjjj, $
  time=time,$                   ;provide time
  fltime=fltime,$               ;time is a float (HH.ddd)
  itime=itime,$                 ;time is HHMMSS
  seconds=seconds,$             ;time is in seconds of the day
  debug=debug
iny =  floor(yyjjj/1000l)

year = iny + 1900.*(iny lt 1900)

;year = 1900 + floor(yyjjj/1000l)
jjj  = yyjjj mod 1000l

jul1 = julday(1,jjj,year,0,0,0)

if keyword_set(itime) then begin
    t = ftime(itime)/24.d0
    jul1 = jul1 + t
endif    

if keyword_set(time) then begin
    t = time
    if keyword_set(fltime)  then t = t / 24.d0
    if keyword_set(seconds) then t = t / 24.d0/3600.d0
    jul1 = jul1 + t
endif

return,jul1
end
;***
