;****I* idl/idl.yymmdd_2_yyjjj
; NAME
; yymmdd_2_yyjjj
; 
; PURPOSE
; To convert date from style YYMMDD to YYJJJ
; 
; USAGE
; result = yymmdd_2_yyjjj(date)
; 
; where
;          date = integer = of format:
;                           YYMMDD where
;                 YY = 2 or 4 digit year
;                 MM = month of year [1-12]
;                 DD = Day of month  [1-31]
; 
; INPUT
; OUTPUT
; RESULT
; The returned result is an integer of the form:
;   YYJJJ
; where:
;   YY=2 or 3 digit year (Year - 1900)
;   JJJ = day of calendar [001-366]
;   
; EXAMPLE
; IDL> print,yymmdd_2_yyjjj(981231) 
;       98365
; IDL> print,yymmdd_2_yyjjj(011231)
;      101365
;      
; USES
; IDL internal routine: julday
; 
; USED BY
; * navcheck.pro
; * b1read_aes_g8.pro
; * b1read_cma_fy2.pro
; and more ...
; 
; NOTES
; * Assumes all years < 50 are in the 21st millenium, so 100 is also
;   added to the year (in addition to the 1900 which is also added).
; 
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; Unknown
; 
; HISTORY
; Jul2010 - Added header (KRK)
; 
; SOURCE

function yymmdd_2_yyjjj,date
idate = long(date)
iyy = idate/10000
imm = idate/100 mod 100
w=where(imm lt 1 or imm gt 12,nw)
if (nw gt 0) then begin
    message,'Month does not appear to be in YYMMDD format',/info
    message,string(date(w(0))),/info
    return,date
endif
idd = idate mod 100
w = where(idd lt 1 or idd gt 31,nw)
if (nw gt 0) then begin
    Message,'Day of month does not appear to be in YYMMDD format',/info
    message,string(date(w(0))),/info
    return,date
endif
iyy = iyy+1900
iyy = iyy + 100*(iyy lt 1950)
iyy=iyy-1900
jday = julday(imm,idd,iyy) - julday(1,1,iyy) + 1
jday = iyy * 1000l + jday
return,jday
end
;***
